﻿using System;
using System.Collections.Generic;
using System.Dynamic;
using System.Linq;
using System.Web.Mvc;
using Xenta.Utils;
using Xenta.Attributes;
using Xenta.Web.Controllers;

namespace Xenta.Web.Areas.Catalog.Controllers
{
    [Authenticate, InRole("ADMIN")]
    public sealed class ItemController : ExtController
    {
        #region GET/POST Actions

        [HttpGet]
        public ActionResult Index()
        {
            return View();
        }

        [HttpGet, RestoreState]
        public ActionResult Create()
        {
            return View();
        }

        [HttpPost, PersistState]
        public ActionResult Create(int supplierID,
            int fileGroupID,
            string[] flags)
        {
            try
            {
                var m = Svc.ProcessOperation("CreateCatalogItem", new
                {
                    SupplierID = supplierID,
                    FileGroupID = fileGroupID,
                    Flags = flags.Merge(",")
                });
                return RedirectToAction(Loc.Act("Edit").Set(new
                {
                    id = JDM.Int32(m.EntityID)
                }));
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Create"));
        }

        [HttpGet, RestoreState]
        public ActionResult Edit(int id)
        {
            var m = Svc.ProcessOperation("GetCatalogItem", new
            {
                EntityID = id
            }, new
            {
                DisclosureLevel = "Entire"
            });
            if(m == null)
                return HttpNotFound();
            return View(m);
        }

        [HttpPost, PersistState]
        public ActionResult Update(int id,
            int supplierID,
            int fileGroupID,
            string[] flags)
        {
            try
            {
                Svc.ProcessOperation("UpdateCatalogItem", new
                {
                    EntityID = id,
                    SupplierID = supplierID,
                    FileGroupID = fileGroupID,
                    Flags = flags.Merge(",")
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }

            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id
            }));
        }

        [HttpGet, PersistState]
        public ActionResult Delete(int id, bool deleteFileGroup)
        {
            try
            {
                Svc.ProcessOperation("DeleteCatalogItem", new
                {
                    EntityID = id,
                    DeleteFileGroup = deleteFileGroup
                });
                return RedirectToAction(Loc.Act("Index"));
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id
            }));
        }

        [HttpPost, PersistState, ValidateInput(false)]
        public ActionResult CreateTranslation(int itemID,
            int languageID,
            string name,
            string shortDescription,
            string description)
        {
            try
            {
                Svc.ProcessOperation("CreateCatalogItemTranslation", new
                {
                    HolderID = itemID,
                    LanguageID = languageID,
                    Name = HtmlUtility.StripTags(name),
                    ShortDescription = HtmlUtility.StripTags(shortDescription),
                    Description = HtmlUtility.Encode(description)
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id = itemID
            }));
        }

        [HttpPost, PersistState, ValidateInput(false)]
        public ActionResult UpdateTranslation(int id,
            int itemID,
            int languageID,
            string name,
            string shortDescription,
            string description)
        {
            try
            {
                Svc.ProcessOperation("UpdateCatalogItemTranslation", new
                {
                    EntityID = id,
                    LanguageID = languageID,
                    Name = HtmlUtility.StripTags(name),
                    ShortDescription = HtmlUtility.StripTags(shortDescription),
                    Description = HtmlUtility.Encode(description)
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id = itemID
            }));
        }

        [HttpGet, PersistState]
        public ActionResult DeleteTranslation(int id, int itemID)
        {
            try
            {
                Svc.ProcessOperation("DeleteCatalogItemTranslation", new
                {
                    EntityID = id
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id = itemID
            }));
        }

        #endregion

        #region AJAX Actions

        [HttpPost]
        public ActionResult Grid(int? page,
            int? rows,
            string sord,
            string sidx,
            string searchTerm,
            int? supplierID,
            int? categoryID,
            int? tagID,
            string attributeIDs,
            int? groupID,
            string createdFrom,
            string createdTo,
            string flagMask)
        {
            page = page.HasValue ? page.Value - 1 : 0;
            rows = rows.HasValue ? rows.Value : 30;
            sord = sord.Equals("desc") ? "Descending" : "Ascending";

            var m = Svc.ProcessOperation("SearchCatalogItems", new
            {
                Term = new
                {
                    Expression = searchTerm
                },
                SupplierID = supplierID,
                CategoryID = categoryID,
                TagID = tagID,
                AttributeIDs = !String.IsNullOrWhiteSpace(attributeIDs) 
                    ? attributeIDs.Split(',').Select(x => x.ParseAs<Int32>()) 
                    : null,
                GroupID = groupID,
                FlagMask = flagMask,
                CreatedFrom = !String.IsNullOrWhiteSpace(createdFrom) 
                    ? Svc.DateTime(createdFrom) 
                    : (DateTime?)null,
                CreatedTo = !String.IsNullOrWhiteSpace(createdTo) 
                    ? Svc.DateTime(createdTo) 
                    : (DateTime?)null,
                Index = page * rows,
                Count = rows,
                SortBy = new[] 
                {
                    sidx
                },
                SortDir = sord
            }, new
            {
                DisclosureLevel = "Base",
                DisclosureOptions = new Dictionary<String, String>
                {
                    { "TranslatedTo", JDM.String(Svc.Localization.Language.Code) }
                }.Unload()
            });
            var data = new List<Object>();

            foreach(var i in m.Items)
            {
                var t_item = (i.Translations as IEnumerable<dynamic>)
                    .SingleOrDefault(x => JDM.Int32(x.Language.EntityID) ==
                        JDM.Int32(Svc.Localization.Language.EntityID));
                data.Add(new[] { 
                    JDM.Int32(i.EntityID),
                    JDM.Int32(i.Supplier.EntityID),
                    t_item != null ? JDM.String(t_item.Name) : String.Empty,
                    JDM.Int32(i.CommentCount),
                    JDM.Int32(i.RatingCount),
                    JDM.Byte(i.RatingAverage)
                });
            }

            return Json(new
            {
                pageIndex = page + 1,
                pageCount = Math.Ceiling(JDM.Double(m.TotalCount) / rows),
                totalCount = JDM.Int64(m.TotalCount),
                data = data.ToArray()
            });
        }

        [HttpPost]
        public ActionResult Autocomplete(string term)
        {
            var m = Svc.ProcessOperation("GetCatalogItemDictionary", new
            {
                LanguageID = JDM.Int32(Svc.Localization.Language.EntityID),
                Term = new
                {
                    Expression = String.Format("{0}%", term)
                },
                Count = 10
            });
            var data = new List<Object>();

            foreach(var i in m.Items)
            {
                data.Add(new
                {
                    id = JDM.String(i.Key),
                    value = JDM.String(i.Value)
                });
            }
            return Json(data.ToArray());
        }

        #endregion

        #region Child Actions

        [ChildActionOnly]
        public ActionResult _Ratings(int itemID)
        {
            dynamic m = new ExpandoObject();

            m.EntityID = itemID;
            m.Ratings = Svc.ProcessOperation("SearchCatalogItemRatings", new
            {
                HolderID = itemID
            }, new
            {
                DisclosureLevel = "Extended"
            }).Items;

            return PartialView(m);
        }

        [ChildActionOnly]
        public ActionResult _Comments(int itemID)
        {
            dynamic m = new ExpandoObject();

            m.EntityID = itemID;
            m.Comments = Svc.ProcessOperation("SearchCatalogItemComments", new
            {
                HolderID = itemID
            }, new
            {
                DisclosureLevel = "Extended"
            }).Items;

            return PartialView(m);
        }

        #endregion
    }
}