﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;
using Xenta.Utils;
using Xenta.Attributes;
using Xenta.Web.Controllers;

namespace Xenta.Web.Areas.Commerce.Controllers
{
    [Authenticate, InRole("ADMIN")]
    public sealed class PaymentController : ExtController
    {
        #region GET/POST Actions

        [HttpGet, RestoreState]
        public ActionResult Index()
        {
            return View();
        }

        [HttpGet]
        public ActionResult GoTo(int? id, string guid)
        {
            if(!id.HasValue)
            {
                dynamic m = null;
                if(!String.IsNullOrWhiteSpace(guid))
                {
                    m = Svc.ProcessOperation("GetPaymentByGuid", new
                    {
                        Guid = guid.ParseAs<Guid>()
                    });
                }
                if(m == null)
                    return HttpNotFound();
                id = JDM.Int32(m.EntityID);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id
            }));
        }

        [HttpGet, RestoreState]
        public ActionResult Create()
        {
            return View();
        }

        [HttpPost, PersistState]
        public ActionResult Place(int invoiceID,
            int customerID,
            Guid processorID,
            string amount)
        {
            try
            {
                var m = Svc.ProcessOperation("PlacePayment", new
                {
                    InvoiceID = invoiceID,
                    CustomerID = customerID,
                    ProcessorID = processorID,
                    Amount = Svc.Price(amount)
                });
                return RedirectToAction(Loc.Act("Edit").Set(new
                {
                    id = JDM.Int32(m.EntityID)
                }));
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Create"));
        }

        [HttpGet, RestoreState]
        public ActionResult Edit(int id)
        {
            var m = Svc.ProcessOperation("GetPayment", new
            {
                EntityID = id,
                Fetch = "Entire"
            }, new
            {
                DisclosureLevel = "Entire"
            });
            if(m == null)
                return HttpNotFound();
            return View(m);
        }

        [HttpPost, PersistState]
        public ActionResult Update(int id,
            string amount,
            string handlingFee,
            string[] flags)
        {
            try
            {
                Svc.ProcessOperation("UpdatePayment", new
                {
                    EntityID = id,
                    Amount = Svc.Price(amount),
                    HandlingFee = Svc.Price(handlingFee),
                    Flags = flags.Merge(",")
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id
            }));
        }

        [HttpGet, PersistState]
        public ActionResult Delete(int id)
        {
            try
            {
                Svc.ProcessOperation("DeletePayment", new
                {
                    EntityID = id
                });
                return RedirectToAction(Loc.Act("Index"));
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id
            }));
        }

        [HttpPost, PersistState]
        public ActionResult BulkAttributeUpdate(int id, IDictionary<String, String> attributes)
        {
            try
            {
                Svc.ProcessOperation("BulkUpdatePaymentAttributes", new
                {
                    HolderID = id,
                    Attributes = attributes
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }

            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id
            }));
        }

        [HttpGet, PersistState]
        public ActionResult Void(int id)
        {
            try
            {
                Svc.ProcessOperation("VoidPayment", new
                {
                    EntityID = id
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id
            }));
        }

        [HttpGet, PersistState]
        public ActionResult Capture(int id)
        {
            try
            {
                Svc.ProcessOperation("CapturePayment", new
                {
                    EntityID = id
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id
            }));
        }

        [HttpGet, PersistState]
        public ActionResult Refund(int id)
        {
            try
            {
                Svc.ProcessOperation("RefundPayment", new
                {
                    EntityID = id
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id
            }));
        }

        [HttpGet, PersistState]
        public ActionResult Authorize(int id)
        {
            try
            {
                Svc.ProcessOperation("AuthorizePayment", new
                {
                    EntityID = id
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id
            }));
        }

        [HttpPost, PersistState]
        public ActionResult HandleIPN(Guid handlerID, IDictionary<String, String> ipn)
        {
            try
            {
                Svc.ProcessOperation("HandleInstantPaymentNotification", new
                {
                    HandlerID = handlerID,
                    Attributes = ipn
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Index"));
        }

        #endregion

        #region AJAX Actions

        [HttpPost]
        public ActionResult Grid(int? page,
            int? rows, 
            string sord, 
            string sidx,
            int? invoiceID,
            int? customerID,
            string createdFrom,
            string createdTo,
            string flagMask)
        {
            page = page.HasValue ? page.Value - 1 : 0;
            rows = rows.HasValue ? rows.Value : 30;
            sord = sord.Equals("desc") ? "Descending" : "Ascending";

            var m = Svc.ProcessOperation("SearchPayments", new 
            {
                InvoiceID = invoiceID,
                CustomerID = customerID,
                FlagMask = flagMask,
                CreatedFrom = String.IsNullOrWhiteSpace(createdFrom)
                    ? (DateTime?)null
                    : Svc.DateTime(createdFrom),
                CreatedTo = String.IsNullOrWhiteSpace(createdTo)
                    ? (DateTime?)null
                    : Svc.DateTime(createdTo),
                Index = page * rows,
                Count = rows,
                SortBy = new[] 
                {
                    sidx
                },
                SortDir = sord
            }, new
            {
                DisclosureLevel = "Base"
            });
            var data = new List<Object>();

            foreach(var i in m.Items)
            {
                string status = String.Empty;

                if(JDM.HasFlags(i.Flags, "Voided"))
                    status = Svc.Translate(Res.In("API").Of("Enum").Of("PaymentFlags").Name("Voided"));
                else if(JDM.HasFlags(i.Flags, "Refunded"))
                    status = Svc.Translate(Res.In("API").Of("Enum").Of("PaymentFlags").Name("Refunded"));
                else if(JDM.HasFlags(i.Flags, "Pending"))
                    status = Svc.Translate(Res.In("API").Of("Enum").Of("PaymentFlags").Name("Pending"));
                else if(JDM.HasFlags(i.Flags, "Authorized"))
                    status = Svc.Translate(Res.In("API").Of("Enum").Of("PaymentFlags").Name("Authorized"));
                else if(JDM.HasFlags(i.Flags, "Captured"))
                    status = Svc.Translate(Res.In("API").Of("Enum").Of("PaymentFlags").Name("Captured"));

                data.Add(new[] { 
                    JDM.Int32(i.EntityID),
                    JDM.Int32(i.Invoice.EntityID),
                    JDM.String(i.Customer.Account.Email),
                    Svc.Price((decimal)JDM.Decimal(i.Amount)),
                    status,
                    Svc.DateTime((DateTime)JDM.DateTime(i.CreatedOn))
                });
            }

            return Json(new
            {
                pageIndex = page + 1,
                pageCount = Math.Ceiling(JDM.Double(m.TotalCount) / rows),
                totalCount = JDM.Int64(m.TotalCount),
                data = data.ToArray()
            });
        }

        [HttpPost]
        public ActionResult Processors()
        {
            dynamic m = Svc.ProcessOperation("ListPaymentProcessors");
            var data = new List<Object>();

            foreach(var i in m)
            {
                data.Add(new
                {
                    id = JDM.Guid(i.ID),
                    value = JDM.String(i.Name)
                });
            }
            return Json(data.ToArray());
        }

        [HttpPost]
        public ActionResult IPNHandlers()
        {
            dynamic m = Svc.ProcessOperation("ListIPNHandlers");
            var data = new List<Object>();

            foreach(var i in m)
            {
                data.Add(new
                {
                    id = JDM.Guid(i.ID),
                    value = JDM.String(i.Name)
                });
            }
            return Json(data.ToArray());
        }

        #endregion
    }
}
