﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using Xenta.Utils;
using Xenta.Attributes;
using Xenta.Web.Controllers;

namespace Xenta.Web.Areas.Forum.Controllers
{
    [Authenticate, InRole("ADMIN")]
    public sealed class ForumController : ExtController
    {
        #region GET/POST Actions

        [HttpGet]
        public ActionResult Index()
        {
            return View();
        }

        [HttpGet, RestoreState]
        public ActionResult Create()
        {
            return View();
        }

        [HttpPost, PersistState]
        public ActionResult Create(int parentID, int displayOrder, string[] flags)
        {
            try
            {
                var m = Svc.ProcessOperation("CreateForum", new
                {
                    ParentID = parentID,
                    DisplayOrder = displayOrder,
                    Flags = flags.Merge(",")
                });
                return RedirectToAction(Loc.Act("Edit").Set(new
                {
                    id = JDM.Int32(m.EntityID)
                }));
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Create"));
        }

        [HttpGet, RestoreState]
        public ActionResult Edit(int id)
        {
            var m = Svc.ProcessOperation("GetForum", new
            {
                EntityID = id
            }, new
            {
                DisclosureLevel = "Entire"
            });
            if(m == null)
                return HttpNotFound();
            return View(m);
        }

        [HttpPost, PersistState]
        public ActionResult Update(int id, int parentID, int displayOrder, string[] flags)
        {
            try
            {
                Svc.ProcessOperation("UpdateForum", new
                {
                    EntityID = id,
                    ParentID = parentID,
                    DisplayOrder = displayOrder,
                    Flags = flags.Merge(",")
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }

            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id
            }));
        }

        [HttpGet, PersistState]
        public ActionResult Delete(int id)
        {
            try
            {
                Svc.ProcessOperation("DeleteForum", new
                {
                    EntityID = id
                });
                return RedirectToAction(Loc.Act("Index"));
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id
            }));
        }

        [HttpPost, PersistState]
        public ActionResult CreateTranslation(int forumID,
            int languageID, 
            string name,
            string description)
        {
            try
            {
                Svc.ProcessOperation("CreateForumTranslation", new
                {
                    HolderID = forumID,
                    LanguageID = languageID,
                    Name = name,
                    Description = description
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id = forumID
            }));
        }

        [HttpPost, PersistState]
        public ActionResult UpdateTranslation(int id, 
            int forumID,
            int languageID, 
            string name, 
            string description)
        {
            try
            {
                Svc.ProcessOperation("UpdateForumTranslation", new
                {
                    EntityID = id,
                    LanguageID = languageID,
                    Name = name,
                    Description = description
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id = forumID
            }));
        }

        [HttpGet, PersistState]
        public ActionResult DeleteTranslation(int id, int forumID)
        {
            try
            {
                Svc.ProcessOperation("DeleteForumTranslation", new
                {
                    EntityID = id
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id = forumID
            }));
        }

        [HttpPost, PersistState]
        public ActionResult SetAccessRule(int forumID, int roleID, string accessRights)
        {
            try
            {
                Svc.ProcessOperation("SetForumAccessRule", new
                {
                    ForumID = forumID,
                    RoleID = roleID,
                    AccessRights = accessRights
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id = forumID
            }));
        }

        [HttpGet, PersistState]
        public ActionResult RemoveAccessRule(int id, int forumID)
        {
            try
            {
                Svc.ProcessOperation("DeleteForumAccessRule", new
                {
                    EntityID = id
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id = forumID
            }));
        }

        #endregion

        #region AJAX Actions

        [HttpPost]
        public ActionResult Grid(int? page, int? rows, string sord, string sidx)
        {
            page = page.HasValue ? page.Value - 1 : 0;
            rows = rows.HasValue ? rows.Value : 30;
            sord = sord.Equals("desc") ? "Descending" : "Ascending";

            var m = Svc.ProcessOperation("SearchForums", new 
            {
                Index = page * rows,
                Count = rows,
                SortBy = new[] 
                {
                    sidx
                },
                SortDir = sord
            }, new
            {
                DisclosureLevel = "Base",
                DisclosureOptions = new Dictionary<String, String>
                {
                    { "TranslatedTo", JDM.String(Svc.Localization.Language.Code) }
                }.Unload()
            });
            var data = new List<Object>();

            foreach(var i in m.Items)
            {
                var t = (i.Translations as IEnumerable<dynamic>)
                    .SingleOrDefault(x => JDM.Int32(x.Language.EntityID) ==
                        JDM.Int32(Svc.Localization.Language.EntityID));

                data.Add(new[] { 
                    JDM.Int32(i.EntityID),
                    i.Parent != null ? JDM.Int32(i.Parent.EntityID) : 0,
                    t != null ? JDM.String(t.Name) : String.Empty,
                    JDM.Int32(i.TopicCount),
                    JDM.Int32(i.PostCount)
                });
            }

            return Json(new
            {
                pageIndex = page + 1,
                pageCount = Math.Ceiling(JDM.Double(m.TotalCount) / rows),
                totalCount = JDM.Int64(m.TotalCount),
                data = data.ToArray()
            });
        }

        #endregion
    }
}
