﻿using System;
using System.Collections.Generic;
using System.Linq;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;
using Xenta.Utils;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the item entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  | - Tags(of the Extended level)
    ///  | - Categories(of the Extended level)
    ///  | - Attributes(of the Extended level)
    ///  | - Groups(of the Extended level)
    ///  | - Specifications(of the Extended level)
    ///  | - RelatedItems(of the Extended level)
    ///  + Extended
    ///   + Base
    ///    | - Supplier(of the Primary level)
    ///    | - FileGroup(of the Primary level)
    ///    | - RatingCount
    ///    | - CommentCount
    ///    | - RatingAverage
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    | - Translations(of the same level)
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class CatalogItemEntity : AuditableEntity,
        IRelationHolder<CatalogCategoryEntity>,
        IRelationHolder<LookupAttributeEntity>,
        IRelationHolder<CatalogTagEntity>,
        IRelationHolder<CatalogItemGroupEntity>,
        IRelationHolder<CatalogItemEntity>
    {
        #region Fields

        private FileGroupEntity _fileGroup;
        private ICollection<CatalogCategoryEntity> _categories;
        private ICollection<LookupAttributeEntity> _attributes;
        private ICollection<CatalogTagEntity> _tags;
        private ICollection<CatalogItemGroupEntity> _groups;
        private ICollection<CatalogItemEntity> _relatedItems;
        private ICollection<TCatalogItemEntity> _translations;
        private ICollection<CatalogItemCommentEntity> _comments;
        private ICollection<CatalogItemRatingEntity> _ratings;
        private ICollection<CatalogItemSpecEntity> _specifications;

        #endregion

        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public CatalogItemEntity()
        {
            _fileGroup = null;
            _categories = new List<CatalogCategoryEntity>();
            _attributes = new List<LookupAttributeEntity>();
            _tags = new List<CatalogTagEntity>();
            _groups = new List<CatalogItemGroupEntity>();
            _relatedItems = new List<CatalogItemEntity>();
            _translations = new List<TCatalogItemEntity>();
            _comments = new List<CatalogItemCommentEntity>();
            _ratings = new List<CatalogItemRatingEntity>();
            _specifications = new List<CatalogItemSpecEntity>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the supplier.
        /// </summary>
        public virtual CatalogSupplierEntity Supplier
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the file group identifier.
        /// </summary>
        public virtual int FileGroupID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the file group.
        /// </summary>
        public virtual FileGroupEntity FileGroup
        {
            get
            {
                return FileGroupID == 0 ? null : _fileGroup;
            }
            set
            {
                _fileGroup = value;
                FileGroupID = value == null ? 0 : value.EntityID;
            }
        }

        /// <summary>
        /// Gets or sets the numer of ratings.
        /// </summary>
        public virtual int RatingCount
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the numer of comments.
        /// </summary>
        public virtual int CommentCount
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the average rating.
        /// </summary>
        public virtual byte RatingAverage
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual CatalogItemFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Collections
        
        /// <summary>
        /// Gets or sets the category collection.
        /// </summary>
        public virtual ICollection<CatalogCategoryEntity> Categories
        {
            get
            {
                return _categories;
            }
            set
            {
                _categories = value;
            }
        }

        /// <summary>
        /// Gets or sets the attribute collection.
        /// </summary>
        public virtual ICollection<LookupAttributeEntity> Attributes
        {
            get
            {
                return _attributes;
            }
            set
            {
                _attributes = value;
            }
        }

        /// <summary>
        /// Gets or sets the tag collection.
        /// </summary>
        public virtual ICollection<CatalogTagEntity> Tags
        {
            get
            {
                return _tags;
            }
            set
            {
                _tags = value;
            }
        }

        /// <summary>
        /// Gets or sets the group collection.
        /// </summary>
        public virtual ICollection<CatalogItemGroupEntity> Groups
        {
            get
            {
                return _groups;
            }
            set
            {
                _groups = value;
            }
        }

        /// <summary>
        /// Gets or sets the related item collection.
        /// </summary>
        public virtual ICollection<CatalogItemEntity> RelatedItems
        {
            get
            {
                return _relatedItems;
            }
            set
            {
                _relatedItems = value;
            }
        }

        /// <summary>
        /// Gets or sets the translation collection.
        /// </summary>
        public virtual ICollection<TCatalogItemEntity> Translations
        {
            get
            {
                return _translations;
            }
            set
            {
                _translations = value;
            }
        }

        /// <summary>
        /// Gets or sets the comment collection.
        /// </summary>
        public virtual ICollection<CatalogItemCommentEntity> Comments
        {
            get
            {
                return _comments;
            }
            set
            {
                _comments = value;
            }
        }

        /// <summary>
        /// Gets or sets the rating collection.
        /// </summary>
        public virtual ICollection<CatalogItemRatingEntity> Ratings
        {
            get
            {
                return _ratings;
            }
            set
            {
                _ratings = value;
            }
        }

        /// <summary>
        /// Gets or sets the specification collection.
        /// </summary>
        public virtual ICollection<CatalogItemSpecEntity> Specifications
        {
            get
            {
                return _specifications;
            }
            set
            {
                _specifications = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the item translation.
        /// </summary>
        /// <param name="languageID">The language identifier.</param>
        /// <returns>The item translation.</returns>
        public virtual TCatalogItemEntity Translation(int languageID)
        {
            return _translations.SingleOrDefault(x => x.Language.EntityID == languageID);
        }

        /// <summary>
        /// Maps a category to the item.
        /// </summary>
        /// <param name="child">The category to map.</param>
        public virtual void Map(CatalogCategoryEntity child)
        {
            if(Categories.Contains(child))
                return;
            Categories.Add(child);
        }

        /// <summary>
        /// Unmaps the category from the item.
        /// </summary>
        /// <param name="child">The category to unmap.</param>
        public virtual void Unmap(CatalogCategoryEntity child)
        {
            if(!Categories.Contains(child))
                return;
            Categories.Remove(child);
        }

        /// <summary>
        /// Maps an attribute to the item.
        /// </summary>
        /// <param name="child">The attribute to map.</param>
        public virtual void Map(LookupAttributeEntity child)
        {
            if(Attributes.Contains(child))
                return;
            Attributes.Add(child);
        }

        /// <summary>
        /// Unmaps the attribute from the item.
        /// </summary>
        /// <param name="child">The attribute to unmap.</param>
        public virtual void Unmap(LookupAttributeEntity child)
        {
            if(!Attributes.Contains(child))
                return;
            Attributes.Remove(child);
        }

        /// <summary>
        /// Maps a tag to the item.
        /// </summary>
        /// <param name="child">The tag to map.</param>
        public virtual void Map(CatalogTagEntity child)
        {
            if(Tags.Contains(child))
                return;
            Tags.Add(child);
        }

        /// <summary>
        /// Unmaps the tag from the item.
        /// </summary>
        /// <param name="child">The tag to unmap.</param>
        public virtual void Unmap(CatalogTagEntity child)
        {
            if(!Tags.Contains(child))
                return;
            Tags.Remove(child);
        }

        /// <summary>
        /// Maps a group to the item.
        /// </summary>
        /// <param name="child">The group to map.</param>
        public virtual void Map(CatalogItemGroupEntity child)
        {
            if(Groups.Contains(child))
                return;
            Groups.Add(child);
        }

        /// <summary>
        /// Unmaps the group from the item.
        /// </summary>
        /// <param name="child">The group to unmap.</param>
        public virtual void Unmap(CatalogItemGroupEntity child)
        {
            if(!Groups.Contains(child))
                return;
            Groups.Remove(child);
        }

        /// <summary>
        /// Maps an item to the item.
        /// </summary>
        /// <param name="child">The item to map.</param>
        public virtual void Map(CatalogItemEntity child)
        {
            if(RelatedItems.Contains(child))
                return;
            RelatedItems.Add(child);
        }

        /// <summary>
        /// Unmaps the item from the item.
        /// </summary>
        /// <param name="child">The item to unmap.</param>
        public virtual void Unmap(CatalogItemEntity child)
        {
            if(!RelatedItems.Contains(child))
                return;
            RelatedItems.Remove(child);
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
            {
                obj["Supplier"] = Supplier.Disclose(DisclosureLevel.Primary, options);
                obj["FileGroup"] = FileGroup != null 
                    ? FileGroup.Disclose(DisclosureLevel.Primary, options)
                    : null;
                obj["RatingCount"] = RatingCount;
                obj["CommentCount"] = CommentCount;
                obj["RatingAverage"] = RatingAverage;
                obj["Flags"] = Flags.ToString();
                obj["Translations"] = (options.ContainsKey("TranslatedTo")
                    ? Translations.Where(x => options["TranslatedTo"].Contains(x.Language.Code))
                    : Translations)
                    .Map(x => x.Disclose(level, options));
            }
            if(level >= DisclosureLevel.Entire)
            {
                obj["Tags"] = Tags.Map(x => x.Disclose(DisclosureLevel.Extended, options));
                obj["Categories"] = Categories.Map(x => x.Disclose(DisclosureLevel.Extended, options));
                obj["Attributes"] = Attributes.Map(x => x.Disclose(DisclosureLevel.Extended, options));
                obj["Groups"] = Groups.Map(x => x.Disclose(DisclosureLevel.Extended, options));
                obj["Specifications"] = Specifications.Map(x => x.Disclose(DisclosureLevel.Extended, options));
                obj["RelatedItems"] = RelatedItems.Map(x => x.Disclose(DisclosureLevel.Extended, options));
            }
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class CatalogItemMapping : EntityMapping<CatalogItemEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public CatalogItemMapping()
        {
            Table("fwk_CatalogItems");
            Id(x => x.EntityID, m =>
            {
                m.Column("ItemID");
                m.Generator(Generators.Identity);
            });
            ManyToOne(x => x.Supplier, m =>
            {
                m.Column("SupplierID");
                m.ForeignKey("FK_fwk_CatalogItems_fwk_CatalogSuppliers");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            Property(x => x.FileGroupID, m =>
            {
                m.Column("FileGroupID");
                m.NotNullable(true);
            });
            ManyToOne(x => x.FileGroup, m =>
            {
                m.Column("FileGroupID");
                m.NotFound(NotFoundMode.Ignore);
                m.Insert(false);
                m.Update(false);
            });
            Property(x => x.RatingCount, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.CommentCount, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.RatingAverage, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
            Bag(x => x.Attributes, m =>
            {
                m.Table("fwk_MCatalogItemAttributes");
                m.Key(k =>
                {
                    k.Column("ItemID");
                    k.ForeignKey("FK_fwk_MCatalogItemAttributes_fwk_CatalogItems");
                    k.NotNullable(true);
                });
            },
            a =>
            {
                a.ManyToMany(m =>
                {
                    m.Column("AttributeID");
                    m.ForeignKey("FK_fwk_MCatalogItemAttributes_fwk_LookupAttributes");
                });
            });
            Bag(x => x.Categories, m =>
            {
                m.Table("fwk_MCatalogItemCategories");
                m.Key(k =>
                {
                    k.Column("ItemID");
                    k.ForeignKey("FK_fwk_MCatalogItemCategories_fwk_CatalogItems");
                    k.NotNullable(true);
                });
            },
            a =>
            {
                a.ManyToMany(m =>
                {
                    m.Column("CategoryID");
                    m.ForeignKey("FK_fwk_MCatalogItemCategories_fwk_CatalogCategories");
                });
            });
            Bag(x => x.Tags, m =>
            {
                m.Table("fwk_MCatalogItemTags");
                m.Key(k =>
                {
                    k.Column("ItemID");
                    k.ForeignKey("FK_fwk_MCatalogItemTags_fwk_CatalogItems");
                    k.NotNullable(true);
                });
            },
            a =>
            {
                a.ManyToMany(m =>
                {
                    m.Column("TagID");
                    m.ForeignKey("FK_fwk_MCatalogItemTags_fwk_CatalogTags");
                });
            });
            Bag(x => x.Groups, m =>
            {
                m.Table("fwk_MCatalogItemGroups");
                m.Key(k =>
                {
                    k.Column("ItemID");
                    k.ForeignKey("FK_fwk_MCatalogItemGroups_fwk_CatalogItems");
                    k.NotNullable(true);
                });
            },
            a =>
            {
                a.ManyToMany(m =>
                {
                    m.Column("GroupID");
                    m.ForeignKey("FK_fwk_MCatalogItemGroups_fwk_CatalogItemGroups");
                });
            });
            Bag(x => x.RelatedItems, m =>
            {
                m.Table("fwk_MCatalogItems");
                m.Key(k =>
                {
                    k.Column("ItemID1");
                    k.ForeignKey("FK_fwk_MCatalogItems_fwk_CatalogItems_1");
                    k.NotNullable(true);
                });
            },
            a =>
            {
                a.ManyToMany(m =>
                {
                    m.Column("ItemID2");
                    m.ForeignKey("FK_fwk_MCatalogItems_fwk_CatalogItems_2");
                });
            });
            Bag(x => x.Translations, m =>
            {
                m.Table("fwk_TCatalogItems");
                m.Key(k =>
                {
                    k.Column("ItemID");
                    k.ForeignKey("FK_fwk_TCatalogItems_fwk_CatalogItems");
                    k.NotNullable(true);
                });
                m.Inverse(true);
                m.BatchSize(10);
            },
            a =>
            {
                a.OneToMany();
            });
            Bag(x => x.Comments, m =>
            {
                m.Table("fwk_Comments");
                m.Key(k =>
                {
                    k.Column("HolderID");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.OneToMany();
            });
            Bag(x => x.Ratings, m =>
            {
                m.Table("fwk_Ratings");
                m.Key(k =>
                {
                    k.Column("HolderID");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.OneToMany();
            });
            Bag(x => x.Specifications, m =>
            {
                m.Table("fwk_CatalogItemSpecifications");
                m.Key(k =>
                {
                    k.Column("ItemID");
                    k.ForeignKey("FK_fwk_CatalogItemSpecifications_fwk_CatalogItems");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.OneToMany();
            });
        }
    }
}
