﻿using System;
using System.Collections.Generic;
using System.Linq;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;
using Xenta.Utils;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the item group entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   + Base
    ///    | - Supplier(of the Primary level)
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    | - Translations(of the same level)
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class CatalogItemGroupEntity : AuditableEntity
    {
        #region Fields

        private ICollection<CatalogItemEntity> _items;
        private ICollection<TCatalogItemGroupEntity> _translations;

        #endregion

        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public CatalogItemGroupEntity()
        {
            _items = new List<CatalogItemEntity>();
            _translations = new List<TCatalogItemGroupEntity>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the supplier.
        /// </summary>
        public virtual CatalogSupplierEntity Supplier
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual CatalogItemGroupFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Collections

        /// <summary>
        /// Gets or sets the item collection.
        /// </summary>
        public virtual ICollection<CatalogItemEntity> Items
        {
            get
            {
                return _items;
            }
            set
            {
                _items = value;
            }
        }

        /// <summary>
        /// Gets or sets the group translations.
        /// </summary>
        public virtual ICollection<TCatalogItemGroupEntity> Translations
        {
            get
            {
                return _translations;
            }
            set
            {
                _translations = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the group translation.
        /// </summary>
        /// <param name="languageID">The language identifier.</param>
        /// <returns>The group translation.</returns>
        public virtual TCatalogItemGroupEntity Translation(int languageID)
        {
            return _translations.SingleOrDefault(x => x.Language.EntityID == languageID);
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
            {
                obj["Supplier"] = Supplier.Disclose(DisclosureLevel.Primary, options);
                obj["Flags"] = Flags.ToString();
                obj["Translations"] = (options.ContainsKey("TranslatedTo")
                    ? Translations.Where(x => options["TranslatedTo"].Contains(x.Language.Code))
                    : Translations)
                    .Map(x => x.Disclose(level, options));
            }
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class CatalogItemGroupMapping : EntityMapping<CatalogItemGroupEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public CatalogItemGroupMapping()
        {
            Table("fwk_CatalogItemGroups");
            Id(x => x.EntityID, m =>
            {
                m.Column("GroupID");
                m.Generator(Generators.Identity);
            });
            ManyToOne(x => x.Supplier, m =>
            {
                m.Column("SupplierID");
                m.ForeignKey("FK_fwk_CatalogItemGroups_fwk_CatalogSuppliers");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
            Bag(x => x.Items, m =>
            {
                m.Table("fwk_MCatalogItemGroups");
                m.Key(k =>
                {
                    k.Column("GroupID");
                    k.ForeignKey("FK_fwk_MCatalogItemGroups_fwk_CatalogItemGroups");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.ManyToMany(m =>
                {
                    m.Column("ItemID");
                    m.ForeignKey("FK_fwk_MCatalogItemGroups_fwk_CatalogItems");
                });
            });
            Bag(x => x.Translations, m =>
            {
                m.Table("fwk_TCatalogItemGroups");
                m.Key(k =>
                {
                    k.Column("GroupID");
                    k.ForeignKey("FK_fwk_TCatalogItemGroups_fwk_CatalogItemGroups");
                    k.NotNullable(true);
                });
                m.Inverse(true);
                m.BatchSize(10);
            },
            a =>
            {
                a.OneToMany();
            });
        }
    }
}
