﻿using System;
using System.Collections.Generic;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the manager entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   + Base
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     | - Account(of the same level)
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class CatalogManagerEntity : AccountProfileEntity
    {
        #region Fields

        private ICollection<CatalogSupplierEntity> _suppliers;

        #endregion

        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public CatalogManagerEntity()
        {
            _suppliers = new List<CatalogSupplierEntity>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual CatalogManagerFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Collections

        /// <summary>
        /// Gets or sets the supplier collection.
        /// </summary>
        public virtual ICollection<CatalogSupplierEntity> Suppliers
        {
            get
            {
                return _suppliers;
            }
            set
            {
                _suppliers = value;
            }
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
                obj["Flags"] = Flags.ToString();
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class CatalogManagerMapping : EntityMapping<CatalogManagerEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public CatalogManagerMapping()
        {
            Table("fwk_CatalogManagers");
            Id(x => x.EntityID, m =>
            {
                m.Column("ManagerID");
                m.Generator(Generators.Identity);
            });
            ManyToOne(x => x.Account, m =>
            {
                m.Column("AccountID");
                m.ForeignKey("FK_fwk_CatalogManagers_fwk_Accounts");
                m.Index("IX_fwk_CatalogManagers");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
            Bag(x => x.Suppliers, m =>
            {
                m.Table("fwk_MCatalogSupplierManagers");
                m.Key(k =>
                {
                    k.Column("ManagerID");
                    k.ForeignKey("FK_fwk_MCatalogSupplierManagers_fwk_CatalogManagers");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.ManyToMany(m =>
                {
                    m.Column("SupplierID");
                    m.ForeignKey("FK_fwk_MCatalogSupplierManagers_fwk_CatalogSuppliers");
                });
            });
            Cache(m =>
            {
                m.Usage(CacheUsage.ReadWrite);
            });
        }
    }
}
