﻿using System;
using System.Collections.Generic;
using System.Linq;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;
using Xenta.Utils;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the tag entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   + Base
    ///    | - Weight
    ///    | - WeightByItem
    ///    | - WeightBySupplier
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    | - Translations(of the same level)
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class CatalogTagEntity : AuditableEntity
    {
        #region Fields

        private ICollection<CatalogItemEntity> _items;
        private ICollection<TCatalogTagEntity> _translations;
        private ICollection<CatalogSupplierEntity> _suppliers;

        #endregion

        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public CatalogTagEntity()
        {
            _items = new List<CatalogItemEntity>();
            _translations = new List<TCatalogTagEntity>();
            _suppliers = new List<CatalogSupplierEntity>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the weight summary.
        /// </summary>
        public virtual int Weight
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the weight by item.
        /// </summary>
        public virtual int WeightByItem
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the weight by item.
        /// </summary>
        public virtual int WeightBySupplier
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual CatalogTagFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Collections
        
        /// <summary>
        /// Gets or sets the supplier collection.
        /// </summary>
        public virtual ICollection<CatalogSupplierEntity> Suppliers
        {
            get
            {
                return _suppliers;
            }
            set
            {
                _suppliers = value;
            }
        }

        /// <summary>
        /// Gets or sets the item collection.
        /// </summary>
        public virtual ICollection<CatalogItemEntity> Items
        {
            get
            {
                return _items;
            }
            set
            {
                _items = value;
            }
        }

        /// <summary>
        /// Gets or sets the tag translations.
        /// </summary>
        public virtual ICollection<TCatalogTagEntity> Translations
        {
            get
            {
                return _translations;
            }
            set
            {
                _translations = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the tag translation.
        /// </summary>
        /// <param name="languageID">The language identifier.</param>
        /// <returns>The tag translation.</returns>
        public virtual TCatalogTagEntity Translation(int languageID)
        {
            return _translations.SingleOrDefault(x => x.Language.EntityID == languageID);
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
            {
                obj["Weight"] = Weight;
                obj["WeightByItem"] = WeightByItem;
                obj["WeightBySupplier"] = WeightBySupplier;
                obj["Flags"] = Flags.ToString();
                obj["Translations"] = (options.ContainsKey("TranslatedTo")
                    ? Translations.Where(x => options["TranslatedTo"].Contains(x.Language.Code))
                    : Translations)
                    .Map(x => x.Disclose(level, options));
            }
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class CatalogTagMapping : EntityMapping<CatalogTagEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public CatalogTagMapping()
        {
            Table("fwk_CatalogTags");
            Id(x => x.EntityID, m =>
            {
                m.Column("TagID");
                m.Generator(Generators.Identity);
            });
            Property(x => x.Weight, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.WeightByItem, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.WeightBySupplier, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
            Bag(x => x.Suppliers, m =>
            {
                m.Table("fwk_MCatalogSupplierTags");
                m.Key(k =>
                {
                    k.Column("TagID");
                    k.ForeignKey("FK_fwk_MCatalogSupplierTags_fwk_CatalogTags");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.ManyToMany(m =>
                {
                    m.Column("SupplierID");
                    m.ForeignKey("FK_fwk_MCatalogSupplierTags_fwk_CatalogSuppliers");
                });
            });
            Bag(x => x.Items, m =>
            {
                m.Table("fwk_MCatalogItemTags");
                m.Key(k =>
                {
                    k.Column("TagID");
                    k.ForeignKey("FK_fwk_MCatalogItemTags_fwk_CatalogTags");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.ManyToMany(m =>
                {
                    m.Column("ItemID");
                    m.ForeignKey("FK_fwk_MCatalogItemTags_fwk_CatalogItems");
                });
            });
            Bag(x => x.Translations, m =>
            {
                m.Table("fwk_TCatalogTags");
                m.Key(k =>
                {
                    k.Column("TagID");
                    k.ForeignKey("FK_fwk_TCatalogTags_fwk_CatalogTags");
                    k.NotNullable(true);
                });
                m.Inverse(true);
                m.BatchSize(10);
            },
            a =>
            {
                a.OneToMany();
            });
        }
    }
}
