﻿using System;
using System.Collections.Generic;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the item translation entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  | - Description
    ///  + Extended
    ///   | - ShortDescription
    ///   + Base
    ///    | - Holder(of the Primary level)
    ///    | - Language(of the Keys level)
    ///    | - Name
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class TCatalogItemEntity : TranslationEntity<CatalogItemEntity>
    {
        #region Properties

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        public virtual string Name
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the short description.
        /// </summary>
        public virtual string ShortDescription
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the description.
        /// </summary>
        public virtual string Description
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
                obj["Name"] = Name;
            if(level >= DisclosureLevel.Extended)
                obj["ShortDescription"] = ShortDescription;
            if(level >= DisclosureLevel.Entire)
                obj["Description"] = Description;
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class TCatalogItemMapping : EntityMapping<TCatalogItemEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public TCatalogItemMapping()
        {
            Table("fwk_TCatalogItems");
            Id(x => x.EntityID, m =>
            {
                m.Column("TranslationID");
                m.Generator(Generators.Identity);
            });
            ManyToOne(x => x.Holder, m =>
            {
                m.Column("ItemID");
                m.ForeignKey("FK_fwk_TCatalogItems_fwk_CatalogItems");
                m.Index("IX_fwk_TCatalogItems");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            ManyToOne(x => x.Language, m =>
            {
                m.Column("LanguageID");
                m.ForeignKey("FK_fwk_TCatalogItems_fwk_Languages");
                m.Index("IX_fwk_TCatalogItems");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            Property(x => x.Name, m =>
            {
                m.NotNullable(true);
                m.Length(100);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(100)");
                });
            });
            Property(x => x.ShortDescription, m =>
            {
                m.NotNullable(true);
                m.Length(1000);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(1000)");
                });
            });
            Property(x => x.Description, m =>
            {
                m.NotNullable(true);
                m.Length(Int32.MaxValue);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(max)");
                });
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
        }
    }
}
