﻿using System;
using System.Collections.Generic;
using System.Linq;
using NHibernate.Linq;
using Xenta.Api;
using Xenta.Entities;
using Xenta.Enums;
using Xenta.Security;
using Xenta.Structs;
using Xenta.Utils;

namespace Xenta.Operations
{
    #region __General__

    /// <summary>
    /// Creates a catalog item.
    /// </summary>
    public class CreateCatalogItem : CreateAuditable<CatalogItemEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var supplier = uow.Execute(new GetCatalogSupplier
                        {
                            EntityID = SupplierID
                        }).Result;
                        if(supplier == null)
                            return false;
                        return supplier.Managers.Contains(manager);
                    });
            }
        }

        /// <summary>
        /// Gets or sets the supplier identifier.
        /// </summary>
        public int SupplierID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the file group identifier.
        /// </summary>
        public int FileGroupID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public CatalogItemFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow, out CatalogItemEntity result)
        {
            base.Work(uow, out result);
            uow.Execute(new UpdateCatalogSupplierStats
            {
                EntityID = result.Supplier.EntityID
            });
        }

        protected override void Construct(IUnitOfWork uow, CatalogItemEntity entity)
        {
            base.Construct(uow, entity);

            entity.Supplier = uow.Execute(new GetCatalogSupplier
            {
                ThrowsException = true,
                EntityID = SupplierID
            }).Result;
            entity.FileGroup = FileGroupID == 0
                ? null
                : uow.Execute(new GetFileGroup
                {
                    ThrowsException = true,
                    EntityID = FileGroupID
                }).Result;
            entity.Flags = Flags;
        }

        #endregion
    }

    /// <summary>
    /// Updates the catalog item.
    /// </summary>
    public class UpdateCatalogItem : UpdateAuditable<CatalogItemEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var item = uow.Execute(new GetCatalogItem
                        {
                            EntityID = EntityID
                        }).Result;
                        if(item == null)
                            return false;
                        if(item.Supplier.EntityID != SupplierID)
                            return false;
                        return item.Supplier.Managers
                            .Contains(manager);
                    });
            }
        }

        /// <summary>
        /// Gets or sets the supplier identifier.
        /// </summary>
        public int SupplierID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the file group identifier.
        /// </summary>
        public int FileGroupID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public CatalogItemFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow, out CatalogItemEntity result)
        {
            var oldSupplier = uow.Execute(new GetCatalogItem
            {
                ThrowsException = true,
                EntityID = EntityID
            }).Result.Supplier;

            base.Work(uow, out result);

            uow.Execute(new UpdateCatalogSupplierStats
            {
                EntityID = result.Supplier.EntityID
            });
            if(!oldSupplier.Equals(result.Supplier))
            {
                uow.Execute(new UpdateCatalogSupplierStats
                {
                    EntityID = oldSupplier.EntityID
                });
            }
        }

        protected override void Revise(IUnitOfWork uow, CatalogItemEntity entity)
        {
            base.Revise(uow, entity);

            entity.Supplier = uow.Execute(new GetCatalogSupplier
            {
                ThrowsException = true,
                EntityID = SupplierID
            }).Result;
            entity.FileGroup = FileGroupID == 0 ? null : uow.Execute(new GetFileGroup
            {
                ThrowsException = true,
                EntityID = FileGroupID
            }).Result;
            entity.Flags = Flags;
        }

        #endregion
    }

    /// <summary>
    /// Updates the catalog item statistics.
    /// </summary>
    public class UpdateCatalogItemStats : OperateEntityByID<CatalogItemEntity>
    {
        #region Utilities

        protected override void Work(IUnitOfWork uow, out CatalogItemEntity result)
        {
            var item = uow.Execute(new GetCatalogItem
            {
                ThrowsException = true,
                EntityID = EntityID
            }).Result;

            uow.Flush();
            uow.SqlQuery("exec fwk_CatalogItems_UpdateStats @ItemID=:ItemID, @UpdatedOn=:UpdatedOn")
                .SetInt32("ItemID", item.EntityID)
                .SetDateTime("UpdatedOn", DateTime.UtcNow)
                .ExecuteUpdate();
            uow.Flush();
            
            result = uow.Execute(new RefreshEntity<CatalogItemEntity>
            {
                Entity = item
            }).Result;
        }

        #endregion
    }

    /// <summary>
    /// Deletes the catalog item.
    /// </summary>
    public class DeleteCatalogItem : DeleteEntity<CatalogItemEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var item = uow.Execute(new GetCatalogItem
                        {
                            EntityID = EntityID
                        }).Result;
                        if(item == null)
                            return false;
                        return item.Supplier.Managers
                            .Contains(manager);
                    });
            }
        }

        /// <summary>
        /// Gets or sets a value indicates, whether 
        /// delete the associated file group.
        /// </summary>
        public bool DeleteFileGroup
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow)
        {
            var supplier = uow.Execute(new GetCatalogItem
            {
                ThrowsException = true,
                EntityID = EntityID
            }).Result.Supplier;

            base.Work(uow);

            uow.Execute(new UpdateCatalogSupplierStats
            {
                EntityID = supplier.EntityID
            });
        }

        protected override void Destruct(IUnitOfWork uow, CatalogItemEntity entity)
        {
            foreach(var i in entity.Translations)
            {
                uow.Execute(new DeleteCatalogItemTranslation
                {
                    EntityID = i.EntityID
                });
            }
            foreach(var i in entity.Comments)
            {
                uow.Execute(new DeleteCatalogItemComment
                {
                    EntityID = i.EntityID
                });
            }
            foreach(var i in entity.Ratings)
            {
                uow.Execute(new DeleteCatalogItemRating
                {
                    EntityID = i.EntityID
                });
            }
            foreach(var i in entity.Groups.ToList())
            {
                uow.Execute(new UnmapCatalogItemGroupFromCatalogItem
                {
                    EntityID = entity.EntityID,
                    ChildID = i.EntityID
                });
            }
            foreach(var i in entity.Categories.ToList())
            {
                uow.Execute(new UnmapCatalogCategoryFromCatalogItem
                {
                    EntityID = entity.EntityID,
                    ChildID = i.EntityID
                });
            }
            foreach(var i in entity.Attributes.ToList())
            {
                uow.Execute(new UnmapLookupAttributeFromCatalogItem
                {
                    EntityID = entity.EntityID,
                    ChildID = i.EntityID
                });
            }
            foreach(var i in entity.Tags.ToList())
            {
                uow.Execute(new UnmapCatalogTagFromCatalogItem
                {
                    ChildID = i.EntityID,
                    EntityID = entity.EntityID
                });
            }
            foreach(var i in entity.Specifications)
            {
                uow.Execute(new DeleteCatalogItemSpec
                {
                    EntityID = i.EntityID
                });
            }
            if(DeleteFileGroup && entity.FileGroup != null)
            {
                uow.Execute(new DeleteFileGroup
                {
                    EntityID = entity.FileGroup.EntityID,
                    DeleteFiles = true
                });
            }
            base.Destruct(uow, entity);
        }

        #endregion
    }

    /// <summary>
    /// Gets the catalog item by identifier.
    /// </summary>
    public class GetCatalogItem : GetEntity<CatalogItemEntity>,
        IApiOperation
    {
    }

    #endregion

    #region __Translation__

    /// <summary>
    /// Creates an item translation.
    /// </summary>
    public class CreateCatalogItemTranslation : CreateTranslation
        <TCatalogItemEntity, CatalogItemEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var item = uow.Execute(new GetCatalogItem
                        {
                            EntityID = HolderID
                        }).Result;
                        if(item == null)
                            return false;
                        return item.Supplier.Managers
                            .Contains(manager);
                    });
            }
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        public string Name
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the short description.
        /// </summary>
        public string ShortDescription
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the description.
        /// </summary>
        public string Description
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Construct(IUnitOfWork uow, TCatalogItemEntity entity)
        {
            base.Construct(uow, entity);

            entity.Name = StringHelper.EnsureNotNull(Name).TrimToLen(100);
            entity.ShortDescription = StringHelper.EnsureNotNull(ShortDescription).TrimToLen(1000);
            entity.Description = StringHelper.EnsureNotNull(Description);
        }

        #endregion
    }

    /// <summary>
    /// Updates the item translation.
    /// </summary>
    public class UpdateCatalogItemTranslation : UpdateTranslation
        <TCatalogItemEntity, CatalogItemEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var translation = uow.Execute(new GetCatalogItemTranslation
                        {
                            EntityID = EntityID
                        }).Result;
                        if(translation == null)
                            return false;
                        return translation.Holder.Supplier
                            .Managers.Contains(manager);
                    });
            }
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        public string Name
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the short description.
        /// </summary>
        public string ShortDescription
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the description.
        /// </summary>
        public string Description
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Revise(IUnitOfWork uow, TCatalogItemEntity entity)
        {
            base.Revise(uow, entity);

            entity.Name = StringHelper.EnsureNotNull(Name).TrimToLen(100);
            entity.ShortDescription = StringHelper.EnsureNotNull(ShortDescription).TrimToLen(1000);
            entity.Description = StringHelper.EnsureNotNull(Description);
        }

        #endregion
    }

    /// <summary>
    /// Saves an item translation.
    /// </summary>
    public class SaveCatalogItemTranslation : SaveTranslation
        <TCatalogItemEntity, CatalogItemEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var item = uow.Execute(new GetCatalogItem
                        {
                            EntityID = HolderID
                        }).Result;
                        if(item == null)
                            return false;
                        return item.Supplier.Managers
                            .Contains(manager);
                    });
            }
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        public string Name
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the short description.
        /// </summary>
        public string ShortDescription
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the description.
        /// </summary>
        public string Description
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow, out TCatalogItemEntity result)
        {
            var translation = uow.Execute(new GetCatalogItemTranslationByLanguage
            {
                HolderID = HolderID,
                LanguageID = LanguageID
            }).Result;
            if(translation == null)
            {
                translation = uow.Execute(new CreateCatalogItemTranslation
                {
                    HolderID = HolderID,
                    LanguageID = LanguageID,
                    Name = Name,
                    ShortDescription = ShortDescription,
                    Description = Description
                }).Result;
            }
            else
            {
                translation = uow.Execute(new UpdateCatalogItemTranslation
                {
                    EntityID = translation.EntityID,
                    LanguageID = LanguageID,
                    Name = Name,
                    ShortDescription = ShortDescription,
                    Description = Description
                }).Result;
            }
            result = translation;
        }

        #endregion
    }

    /// <summary>
    /// Deletes the item translation.
    /// </summary>
    public class DeleteCatalogItemTranslation : DeleteTranlsation
        <TCatalogItemEntity, CatalogItemEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var translation = uow.Execute(new GetCatalogItemTranslation
                        {
                            EntityID = EntityID
                        }).Result;
                        if(translation == null)
                            return false;
                        return translation.Holder.Supplier
                            .Managers.Contains(manager);
                    });
            }
        }

        #endregion
    }

    /// <summary>
    /// Gets the item translation.
    /// </summary>
    public class GetCatalogItemTranslation : GetTranlsation
        <TCatalogItemEntity, CatalogItemEntity>,
        IApiOperation
    {
    }

    /// <summary>
    /// Gets the item translation.
    /// </summary>
    public class GetCatalogItemTranslationByLanguage : GetTranlsationByLanguage
        <TCatalogItemEntity, CatalogItemEntity>,
        IApiOperation
    {
    }

    #endregion

    #region __Search__

    /// <summary>
    /// Search over catalog item collection.
    /// </summary>
    public class SearchCatalogItems : SearchOperation<CatalogItemEntity>,
        IApiOperation
    {
        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public SearchCatalogItems()
        {
            Term = SearchTerm.Empty;
            SortDir = SortDirection.Ascending;
            SortBy = new []
            {
                CatalogItemSortBy.ID
            };
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the search term.
        /// </summary>
        public SearchTerm Term
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the language identifier.
        /// </summary>
        public int? LanguageID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the supplier identifier.
        /// </summary>
        public int? SupplierID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the category identifier.
        /// </summary>
        public int? CategoryID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the group identifier.
        /// </summary>
        public int? GroupID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the attribute identifiers.
        /// </summary>
        public int[] AttributeIDs
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the tag identifier.
        /// </summary>
        public int? TagID
        {
            get;
            set;
        }

        /// <summary>
        ///   Gets or sets the creation date 'from'.
        /// </summary>
        public DateTime? CreatedFrom
        {
            get;
            set;
        }

        /// <summary>
        ///   Gets or sets the creation date 'to'.
        /// </summary>
        public DateTime? CreatedTo
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flag mask.
        /// </summary>
        public CatalogItemFlags FlagMask
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sort direction.
        /// </summary>
        public SortDirection SortDir
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sort by.
        /// </summary>
        public CatalogItemSortBy[] SortBy
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override IQueryable<CatalogItemEntity> Build(IUnitOfWork uow, IQueryable<CatalogItemEntity> query)
        {
            query = base.Build(uow, query);
            query = query.Fetch(x => x.Supplier);
            query = query.FilterByFlags(x => (int)x.Flags, (int)FlagMask);
            query = query.FilterByDateRange(x => x.CreatedOn, CreatedFrom, CreatedTo);
            switch(Term.Mode)
            {
                case SearchTerm.SearchMode.StartsWith:
                    if(LanguageID.HasValue)
                        query = query.Where(x => x.Translations.Any(i =>
                            i.Language.EntityID == LanguageID.Value &&
                            i.Name.StartsWith(Term.Keyword)));
                    else
                        query = query.Where(x => x.Translations.Any(i =>
                            i.Name.StartsWith(Term.Keyword)));
                    break;
                case SearchTerm.SearchMode.EndsWith:
                    if(LanguageID.HasValue)
                        query = query.Where(x => x.Translations.Any(i =>
                            i.Language.EntityID == LanguageID.Value &&
                            i.Name.EndsWith(Term.Keyword)));
                    else
                        query = query.Where(x => x.Translations.Any(i =>
                            i.Name.EndsWith(Term.Keyword)));
                    break;
                case SearchTerm.SearchMode.Contains:
                    if(LanguageID.HasValue)
                        query = query.Where(x => x.Translations.Any(i =>
                            i.Language.EntityID == LanguageID.Value &&
                            (i.Name.Contains(Term.Keyword) ||
                            i.ShortDescription.Contains(Term.Keyword))));
                    else
                        query = query.Where(x => x.Translations.Any(i =>
                            i.Name.Contains(Term.Keyword) ||
                            i.ShortDescription.Contains(Term.Keyword)));
                    break;
                case SearchTerm.SearchMode.Equals:
                    if(LanguageID.HasValue)
                        query = query.Where(x => x.Translations.Any(i =>
                            i.Language.EntityID == LanguageID.Value &&
                            i.Name.Equals(Term.Keyword)));
                    else
                        query = query.Where(x => x.Translations.Any(i =>
                            i.Name.Equals(Term.Keyword)));
                    break;
                default:
                    if(LanguageID.HasValue)
                        query = query.Where(x => x.Translations.Any(i => i.Language.EntityID == LanguageID.Value));
                    break;
            }
            if(SupplierID.HasValue)
                query = query.Where(x => x.Supplier.EntityID == SupplierID.Value);
            if(CategoryID.HasValue)
            {
                query = CategoryID.Value != 0
                    ? query.Where(x => x.Categories.Any(i => i.EntityID == CategoryID.Value))
                    : query.Where(x => !x.Categories.Any());
            }
            if(TagID.HasValue)
                query = query.Where(x => x.Tags.Any(i => i.EntityID == TagID.Value));
            if(AttributeIDs != null && AttributeIDs.Any())
                query = query.Where(x => x.Attributes.Count(i => AttributeIDs.Contains(i.EntityID)) == AttributeIDs.Count());
            if(GroupID.HasValue)
            {
                query = GroupID.Value != 0
                    ? query.Where(x => x.Groups.Any(i => i.EntityID == GroupID.Value))
                    : query.Where(x => !x.Groups.Any());
            }
            if(SortBy != null)
            {
                for(int i = 0; i < SortBy.Length; i++)
                {
                    switch(SortBy[i])
                    {
                        case CatalogItemSortBy.CommentCount:
                            query = query.OrderBy(x => x.CommentCount, SortDir, i != 0);
                            break;
                        case CatalogItemSortBy.RatingCount:
                            query = query.OrderBy(x => x.RatingCount, SortDir, i != 0);
                            break;
                        case CatalogItemSortBy.RatingAverage:
                            query = query.OrderBy(x => x.RatingAverage, SortDir, i != 0);
                            break;
                        case CatalogItemSortBy.CreatedOn:
                            query = query.OrderBy(x => x.CreatedOn, SortDir, i != 0);
                            break;
                        default:
                            query = query.OrderBy(x => x.EntityID, SortDir, i != 0);
                            break;
                    }
                }
            }
            return query;
        }

        #endregion
    }

    /// <summary>
    /// Gets the collection of items as a dictionary, where 
    /// key - item identifier and value - item name.
    /// </summary>
    public class GetCatalogItemDictionary : SearchOperation<TCatalogItemEntity, Int32, String>,
        IApiOperation
    {
        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public GetCatalogItemDictionary()
        {
            Term = SearchTerm.Empty;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the language identifier.
        /// </summary>
        public int LanguageID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the supplier identifier.
        /// </summary>
        public int? SupplierID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the search term.
        /// </summary>
        public SearchTerm Term
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flag mask.
        /// </summary>
        public CatalogItemFlags FlagMask
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override IQueryable<TCatalogItemEntity> Build(IUnitOfWork uow, IQueryable<TCatalogItemEntity> query)
        {
            query = base.Build(uow, query);
            query = query.FilterByFlags(x => (int)x.Holder.Flags, (int)FlagMask);
            query = query.Where(x => x.Language.EntityID == LanguageID);
            if(SupplierID.HasValue)
                query = query.Where(x => x.Holder.Supplier.EntityID == SupplierID.Value);
            switch(Term.Mode)
            {
                case SearchTerm.SearchMode.StartsWith:
                    query = query.Where(x => x.Name.StartsWith(Term.Keyword));
                    break;
                case SearchTerm.SearchMode.EndsWith:
                    query = query.Where(x => x.Name.EndsWith(Term.Keyword));
                    break;
                case SearchTerm.SearchMode.Contains:
                    query = query.Where(x => x.Name.Contains(Term.Keyword));
                    break;
                case SearchTerm.SearchMode.Equals:
                    query = query.Where(x => x.Name.Equals(Term.Keyword));
                    break;
            }
            query = query.OrderBy(x => x.Name);
            return query;
        }

        protected override IQueryable<KeyValuePair<Int32, String>> Map(IQueryable<TCatalogItemEntity> query)
        {
            return query.Select(x => new KeyValuePair<Int32, String>(x.Holder.EntityID, x.Name));
        }

        #endregion
    }

    #endregion

    #region __Map__

    /// <summary>
    /// Maps the catalog category to catalog item.
    /// </summary>
    public class MapCatalogCategoryToCatalogItem : MapOperation
        <CatalogItemEntity, CatalogCategoryEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var item = uow.Execute(new GetCatalogItem
                        {
                            EntityID = EntityID
                        }).Result;
                        if(item == null)
                            return false;
                        return item.Supplier.Managers
                            .Contains(manager);
                    });
            }
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow, out CatalogItemEntity result)
        {
            base.Work(uow, out result);
            uow.Execute(new UpdateCatalogCategoryStats
            {
                EntityID = ChildID
            });
        }

        #endregion
    }

    /// <summary>
    /// Unmaps the catalog category from catalog item.
    /// </summary>
    public class UnmapCatalogCategoryFromCatalogItem : UnmapOperation
        <CatalogItemEntity, CatalogCategoryEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var item = uow.Execute(new GetCatalogItem
                        {
                            EntityID = EntityID
                        }).Result;
                        if(item == null)
                            return false;
                        return item.Supplier.Managers
                            .Contains(manager);
                    });
            }
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow, out CatalogItemEntity result)
        {
            base.Work(uow, out result);
            uow.Execute(new UpdateCatalogCategoryStats
            {
                EntityID = ChildID
            });
        }

        #endregion
    }

    /// <summary>
    /// Maps the lookup attribute to catalog item.
    /// </summary>
    public class MapLookupAttributeToCatalogItem : MapOperation
        <CatalogItemEntity, LookupAttributeEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var item = uow.Execute(new GetCatalogItem
                        {
                            EntityID = EntityID
                        }).Result;
                        if(item == null)
                            return false;
                        return item.Supplier.Managers
                            .Contains(manager);
                    });
            }
        }

        #endregion

        #region Utilities

        /// <summary>
        /// Performs the operation validation.
        /// </summary>
        /// <param name="uow">The unit of work.</param>
        /// <returns>true, whether the operation is allowed to be executed; otherwise false.</returns>
        protected override Err Validate(IUnitOfWork uow)
        {
            var attribute = uow.Execute(new GetLookupAttribute
            {
                EntityID = ChildID,
                ThrowsException = true
            }).Result;
            if(!attribute.Flags.HasFlag(LookupAttributeFlags.Common) &&
                !attribute.Flags.HasFlag(LookupAttributeFlags.CatalogItem))
            {
                return Err.Of(this).NotAllowed();
            }
            return base.Validate(uow);
        }

        #endregion
    }

    /// <summary>
    /// Unmaps the lookup attribute from catalog item.
    /// </summary>
    public class UnmapLookupAttributeFromCatalogItem : UnmapOperation
        <CatalogItemEntity, LookupAttributeEntity>,
        ISecuredOperation, IApiOperation
    {
        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var item = uow.Execute(new GetCatalogItem
                        {
                            EntityID = EntityID
                        }).Result;
                        if(item == null)
                            return false;
                        return item.Supplier.Managers
                            .Contains(manager);
                    });
            }
        }
    }

    /// <summary>
    /// Maps the catalog item to catalog item group.
    /// </summary>
    public class MapCatalogItemGroupToCatalogItem : MapOperation
        <CatalogItemEntity, CatalogItemGroupEntity>,
        ISecuredOperation, IApiOperation
    {
        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var item = uow.Execute(new GetCatalogItem
                        {
                            EntityID = EntityID
                        }).Result;
                        if(item == null)
                            return false;
                        var group = uow.Execute(new GetCatalogItemGroup
                        {
                            EntityID = ChildID
                        }).Result;
                        if(group == null)
                            return false;
                        return item.Supplier.EntityID == group.Supplier.EntityID &&
                            item.Supplier.Managers.Contains(manager);
                    });
            }
        }
    }

    /// <summary>
    /// Unmaps the catalog item from catalog item group.
    /// </summary>
    public class UnmapCatalogItemGroupFromCatalogItem : UnmapOperation
        <CatalogItemEntity, CatalogItemGroupEntity>,
        ISecuredOperation, IApiOperation
    {
        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var item = uow.Execute(new GetCatalogItem
                        {
                            EntityID = EntityID
                        }).Result;
                        if(item == null)
                            return false;
                        var group = uow.Execute(new GetCatalogItemGroup
                        {
                            EntityID = ChildID
                        }).Result;
                        if(group == null)
                            return false;
                        return item.Supplier.EntityID == group.Supplier.EntityID &&
                            item.Supplier.Managers.Contains(manager);
                    });
            }
        }
    }

    /// <summary>
    /// Maps the catalog tag to catalog item.
    /// </summary>
    public class MapCatalogTagToCatalogItem : MapOperation
        <CatalogItemEntity, CatalogTagEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var item = uow.Execute(new GetCatalogItem
                        {
                            EntityID = EntityID
                        }).Result;
                        if(item == null)
                            return false;
                        return item.Supplier.Managers
                            .Contains(manager);
                    });
            }
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow, out CatalogItemEntity result)
        {
            base.Work(uow, out result);
            uow.Execute(new UpdateCatalogTagStats
            {
                EntityID = ChildID
            });
        }

        #endregion
    }

    /// <summary>
    /// Unmaps the catalog tag from catalog item.
    /// </summary>
    public class UnmapCatalogTagFromCatalogItem : UnmapOperation
        <CatalogItemEntity, CatalogTagEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var item = uow.Execute(new GetCatalogItem
                        {
                            EntityID = EntityID
                        }).Result;
                        if(item == null)
                            return false;
                        return item.Supplier.Managers
                            .Contains(manager);
                    });
            }
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow, out CatalogItemEntity result)
        {
            base.Work(uow, out result);
            uow.Execute(new UpdateCatalogTagStats
            {
                EntityID = ChildID
            });
        }

        #endregion
    }

    /// <summary>
    /// Maps the catalog item to catalog item.
    /// </summary>
    public class MapCatalogItemToCatalogItem : MapOperation
        <CatalogItemEntity, CatalogItemEntity>,
        ISecuredOperation, IApiOperation
    {
        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var item1 = uow.Execute(new GetCatalogItem
                        {
                            EntityID = EntityID
                        }).Result;
                        if(item1 == null)
                            return false;
                        var item2 = uow.Execute(new GetCatalogItem
                        {
                            EntityID = ChildID
                        }).Result;
                        if(item2 == null)
                            return false;
                        return item1.Supplier.EntityID == item2.Supplier.EntityID &&
                            item1.Supplier.Managers.Contains(manager);
                    });
            }
        }
    }

    /// <summary>
    /// Unmaps the catalog item from catalog item.
    /// </summary>
    public class UnmapCatalogItemFromCatalogItem : UnmapOperation
        <CatalogItemEntity, CatalogItemEntity>,
        ISecuredOperation, IApiOperation
    {
        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var manager = ctx.CatalogManager(uow);
                        if(manager == null || !manager.Flags.HasFlag(CatalogManagerFlags.Active))
                            return false;
                        var item1 = uow.Execute(new GetCatalogItem
                        {
                            EntityID = EntityID
                        }).Result;
                        if(item1 == null)
                            return false;
                        var item2 = uow.Execute(new GetCatalogItem
                        {
                            EntityID = ChildID
                        }).Result;
                        if(item2 == null)
                            return false;
                        return item1.Supplier.EntityID == item2.Supplier.EntityID &&
                            item1.Supplier.Managers.Contains(manager);
                    });
            }
        }
    }

    #endregion
}
