﻿using System;
using System.Collections.Generic;
using System.Linq;
using Xenta.Api;
using Xenta.Entities;
using Xenta.Enums;
using Xenta.Security;
using Xenta.Utils;

namespace Xenta.Operations
{
    #region __General__

    /// <summary>
    /// Creates a catalog item comment.
    /// </summary>
    public class CreateCatalogItemComment : CreateComment<CatalogItemCommentEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) => ctx.Account(uow).EntityID == AccountID);
                yield return new SecurityPolicy()
                    .Rule((ctx, uow) => AccountID == 0);
            }
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public CatalogItemCommentFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override Err Validate(IUnitOfWork uow)
        {
            var item = uow.Execute(new GetCatalogItem
            {
                ThrowsException = true,
                EntityID = HolderID
            }).Result;
            if(!item.Flags.HasFlag(CatalogItemFlags.CommentAllowed))
                return Err.Of(this).NotAllowed();
            if(!uow.Execute(new GetConfigParamValueByKey
            {
                Key = Par.In("Catalog").Of<CatalogItemCommentEntity>().Name("Allowed")
            }).ResultAs<Boolean>())
            {
                return Err.Of(this).NotAllowed();
            }

            var account = AccountID != 0
                ? uow.Execute(new GetAccount
                {
                    ThrowsException = true,
                    EntityID = AccountID
                }).Result
                : null;
            if(account == null)
            {
                if(!item.Flags.HasFlag(CatalogItemFlags.AnonCommentAllowed))
                    return Err.Of(this).NotAllowed().Reason("Anon");
                if(!uow.Execute(new GetConfigParamValueByKey
                {
                    Key = Par.In("Catalog").Of<CatalogItemCommentEntity>().Name("AnonAllowed")
                }).ResultAs<Boolean>())
                {
                    return Err.Of(this).NotAllowed().Reason("Anon");
                }
            }

            return base.Validate(uow);
        }

        protected override void Work(IUnitOfWork uow, out CatalogItemCommentEntity result)
        {
            base.Work(uow, out result);

            uow.Execute(new UpdateCatalogItemStats
            {
                EntityID = result.HolderID
            });
            foreach(var i in uow.Execute(new GetCatalogItem
            {
                ThrowsException = true,
                EntityID = HolderID
            }).Result.Supplier.Managers)
            {
                try
                {
                    uow.Execute(new EnqueueCatalogItemCommentNotification
                    {
                        CommentID = result.EntityID,
                        RecipientID = i.Account.EntityID
                    });
                }
                catch(Exception ex)
                {
                    uow.Execute(new CreateLogRecordFromException
                    {
                        Ex = ex
                    });
                }
            }
        }

        protected override void Construct(IUnitOfWork uow, CatalogItemCommentEntity entity)
        {
            base.Construct(uow, entity);

            entity.Flags = Flags;
        }

        #endregion
    }

    /// <summary>
    /// Updates the catalog item comment.
    /// </summary>
    public class UpdateCatalogItemComment : UpdateComment<CatalogItemCommentEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
            }
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public CatalogItemCommentFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override Err Validate(IUnitOfWork uow)
        {
            if(String.IsNullOrEmpty(Title))
                return Err.Of<CatalogItemCommentEntity>().NotValid("Title");
            return base.Validate(uow);
        }

        protected override void Work(IUnitOfWork uow, out CatalogItemCommentEntity result)
        {
            base.Work(uow, out result);
            uow.Execute(new UpdateCatalogItemStats
            {
                EntityID = result.HolderID
            });
        }

        protected override void Revise(IUnitOfWork uow, CatalogItemCommentEntity entity)
        {
            base.Revise(uow, entity);

            entity.Flags = Flags;
        }

        #endregion
    }

    /// <summary>
    /// Deletes the catalog item comment.
    /// </summary>
    public class DeleteCatalogItemComment : DeleteComment<CatalogItemCommentEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
            }
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow)
        {
            int holderID = uow.Execute(new GetCatalogItemComment
            {
                ThrowsException = true,
                EntityID = EntityID
            }).Result.HolderID;

            base.Work(uow);

            uow.Execute(new UpdateCatalogItemStats
            {
                EntityID = holderID
            });
        }

        #endregion
    }

    /// <summary>
    /// Gets the catalog item comment.
    /// </summary>
    public class GetCatalogItemComment : GetComment<CatalogItemCommentEntity>,
        IApiOperation
    {
    }

    #endregion

    #region __Search__

    /// <summary>
    /// Search over catalog item comment collection.
    /// </summary>
    public class SearchCatalogItemComments : SearchComments<CatalogItemCommentEntity>,
        IApiOperation
    {
        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public SearchCatalogItemComments()
        {
            SortDir = SortDirection.Ascending;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the flag mask.
        /// </summary>
        public CatalogItemCommentFlags FlagMask
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sort direction.
        /// </summary>
        public SortDirection SortDir
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override IQueryable<CatalogItemCommentEntity> Build(IUnitOfWork uow, IQueryable<CatalogItemCommentEntity> query)
        {
            query = base.Build(uow, query);
            query = query.FilterByFlags(x => x.FlagsValue, (int)FlagMask);
            query = query.OrderBy(x => x.CreatedOn, SortDir);
            return query;
        }

        #endregion
    }

    #endregion

    #region __Notification__

    /// <summary>
    /// Enqueues a catalog item comment notification message.
    /// </summary>
    /// <remarks>
    /// Message template code: NCATITMCMT
    /// Allowed tokens: [T:Recipient.EntityID], [T:Recipient.Guid] [T:Recipient.Email], [T:Recipient.Username], 
    /// [T:CatalogItemComment.Item.EntityID],  [T:CatalogItemComment.Item.RatingAverage], [T:CatalogItemComment.Item.CommentCount],
    /// [T:CatalogItemComment.Item.Name], [T:CatalogItemComment.Item.ShortDescription], [T:CatalogItemComment.Account.EntityID], 
    /// [T:CatalogItemComment.Account.Username], [T:CatalogItemComment.Account.Email], [T:CatalogItemComment.Account.Guid], 
    /// [T:CatalogItemComment.EntityID], [T:CatalogItemComment.RatingCount], [T:CatalogItemComment.Title], [T:CatalogItemComment.Text], 
    /// [T:CatalogItemComment.CreatedOn]
    /// </remarks>
    public class EnqueueCatalogItemCommentNotification : Operation<MessageEntity>
    {
        #region Properties

        /// <summary>
        /// Gets or sets the review identifier.
        /// </summary>
        public int CommentID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the recipient account identifier.
        /// </summary>
        public int RecipientID
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow, out MessageEntity result)
        {
            var comment = uow.Execute(new GetCatalogItemComment
            {
                ThrowsException = true,
                EntityID = CommentID
            }).Result;
            var recipient = uow.Execute(new GetAccount
            {
                ThrowsException = true,
                EntityID = RecipientID
            }).Result;
            var usrLocale = recipient.LocalizationProfile;
            int utcOffset = usrLocale.TimeZone.UtcOffset;
            string dtFmt = usrLocale.Country.DateTimeFormat;
            var item = uow.Execute(new GetCatalogItem
            {
                ThrowsException = true,
                EntityID = comment.HolderID
            }).Result;
            var t_item = item.Translation(usrLocale.Language.EntityID);

            result = uow.Execute(new CreateTemplatedMessageByCode
            {
                TemplateCode = "NCATITMCMT",
                LanguageID = usrLocale.Language.EntityID,
                Replacements = new Dictionary<String, String>
                {
                    {"Recipient.EntityID", recipient.EntityID.ToString()},
                    {"Recipient.Email", recipient.Email},
                    {"Recipient.Username", recipient.Username},
                    {"Recipient.Guid", recipient.Guid.ToString()},
                    {"CatalogItemComment.Item.EntityID", item.EntityID.ToString()},
                    {"CatalogItemComment.Item.RatingAverage", item.RatingAverage.ToString()},
                    {"CatalogItemComment.Item.CommentCount", item.CommentCount.ToString()},
                    {"CatalogItemComment.Item.RatingCount", item.RatingCount.ToString()},
                    {"CatalogItemComment.Item.Name", t_item != null 
                        ? t_item.Name 
                        : String.Empty},
                    {"CatalogItemComment.Item.ShortDescription", t_item != null 
                        ? t_item.ShortDescription 
                        : String.Empty},
                    {"CatalogItemComment.Account.EntityID", comment.Account != null 
                        ? comment.Account.EntityID.ToString() 
                        : String.Empty},
                    {"CatalogItemComment.Account.Username", comment.Account != null 
                        ? comment.Account.Username 
                        : String.Empty},
                    {"CatalogItemComment.Account.Email", comment.Account != null 
                        ? comment.Account.Email 
                        : String.Empty},
                    {"CatalogItemComment.Account.Guid", comment.Account != null 
                        ? comment.Account.Guid.ToString() 
                        : String.Empty},
                    {"CatalogItemComment.EntityID", comment.EntityID.ToString()},
                    {"CatalogItemComment.Title", comment.Title},
                    {"CatalogItemComment.Text", comment.Text},
                    {"CatalogItemComment.CreatedOn", comment.CreatedOn.ToLocal(utcOffset).ToStringByFmt(dtFmt)}
                },
                To = recipient.Email,
                Flags = MessageFlags.Active
            }).Result;
        }

        #endregion
    }

    #endregion
}
