﻿using System;
using System.Collections.Generic;
using System.Linq;
using NHibernate.Linq;
using Xenta.Api;
using Xenta.Entities;
using Xenta.Enums;
using Xenta.Security;
using Xenta.Structs;
using Xenta.Utils;

namespace Xenta.Operations
{
    #region __General__

    /// <summary>
    /// Creates a catalog supplier rating.
    /// </summary>
    public class CreateCatalogSupplierRating : CreateRating
        <CatalogSupplierRatingEntity, CatalogSupplierCommentEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) => ctx.Account(uow).EntityID == AccountID);
                yield return new SecurityPolicy()
                    .Rule((ctx, uow) => AccountID == 0);
            }
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public CatalogSupplierRatingFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override Err Validate(IUnitOfWork uow)
        {
            var supplier = uow.Execute(new GetCatalogSupplier
            {
                ThrowsException = true,
                EntityID = HolderID
            }).Result;
            if(!supplier.Flags.HasFlag(CatalogSupplierFlags.RatingAllowed))
                return Err.Of(this).NotAllowed();
            if(!uow.Execute(new GetConfigParamValueByKey
            {
                Key = Par.In("Catalog").Of<CatalogSupplierRatingEntity>().Name("Allowed")
            }).ResultAs<Boolean>())
            {
                return Err.Of(this).NotAllowed();
            }

            var account = AccountID == 0 ? null : uow.Execute(new GetAccount
            {
                ThrowsException = true,
                EntityID = AccountID
            }).Result;
            if(account == null)
            {
                if(!supplier.Flags.HasFlag(CatalogSupplierFlags.AnonRatingAllowed))
                    return Err.Of(this).NotAllowed().Reason("Anon");
                if(!uow.Execute(new GetConfigParamValueByKey
                {
                    Key = Par.In("Catalog").Of<CatalogSupplierRatingEntity>().Name("AnonAllowed")
                }).ResultAs<Boolean>())
                {
                    return Err.Of(this).NotAllowed().Reason("Anon");
                }
            }

            byte valueMin = uow.Execute(new GetConfigParamValueByKey
            {
                Key = Par.In("Catalog").Of<CatalogSupplierRatingEntity>().Name("Min")
            }).ResultAs<Byte>();
            if(Value < valueMin)
                return Err.Of<CatalogSupplierRatingEntity>().NotValid("Value");
            byte valueMax = uow.Execute(new GetConfigParamValueByKey
            {
                Key = Par.In("Catalog").Of<CatalogSupplierRatingEntity>().Name("Max")
            }).ResultAs<Byte>();
            if(Value > valueMax)
                return Err.Of<CatalogSupplierRatingEntity>().NotValid("Value");

            return base.Validate(uow);
        }

        protected override void Work(IUnitOfWork uow, out CatalogSupplierRatingEntity result)
        {
            base.Work(uow, out result);
            uow.Execute(new UpdateCatalogSupplierStats
            {
                EntityID = result.HolderID
            });

            foreach(var i in uow.Execute(new GetCatalogSupplier
            {
                ThrowsException = true,
                EntityID = result.HolderID
            }).Result.Managers)
            {
                try
                {
                    uow.Execute(new EnqueueCatalogSupplierRatingNotification
                    {
                        RatingID = result.EntityID,
                        RecipientID = i.Account.EntityID
                    });
                }
                catch(Exception ex)
                {
                    uow.Execute(new CreateLogRecordFromException
                    {
                        Ex = ex
                    });
                }
            }
        }

        protected override void Construct(IUnitOfWork uow, CatalogSupplierRatingEntity entity)
        {
            base.Construct(uow, entity);

            entity.Flags = Flags;
        }

        #endregion
    }

    /// <summary>
    /// Updates the catalog supplier rating.
    /// </summary>
    public class UpdateCatalogSupplierRating : UpdateRating<CatalogSupplierRatingEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
            }
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public CatalogSupplierRatingFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override Err Validate(IUnitOfWork uow)
        {
            byte valueMin = uow.Execute(new GetConfigParamValueByKey
            {
                Key = Par.In("Catalog").Of<CatalogSupplierRatingEntity>().Name("Min")
            }).ResultAs<Byte>();
            if(Value < valueMin)
                return Err.Of<CatalogSupplierRatingEntity>().NotValid("Value");
            byte valueMax = uow.Execute(new GetConfigParamValueByKey
            {
                Key = Par.In("Catalog").Of<CatalogSupplierRatingEntity>().Name("Max")
            }).ResultAs<Byte>();
            if(Value > valueMax)
                return Err.Of<CatalogSupplierRatingEntity>().NotValid("Value");
            return base.Validate(uow);
        }

        protected override void Work(IUnitOfWork uow, out CatalogSupplierRatingEntity result)
        {
            base.Work(uow, out result);
            uow.Execute(new UpdateCatalogSupplierStats
            {
                EntityID = result.HolderID
            });
        }

        protected override void Revise(IUnitOfWork uow, CatalogSupplierRatingEntity entity)
        {
            base.Revise(uow, entity);

            entity.Flags = Flags;
        }

        #endregion
    }

    /// <summary>
    /// Deletes the catalog supplier rating.
    /// </summary>
    public class DeleteCatalogSupplierRating : DeleteRating<CatalogSupplierRatingEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("CTLG");
            }
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow)
        {
            int holderID = uow.Execute(new GetCatalogSupplierRating
            {
                ThrowsException =  true,
                EntityID = EntityID
            }).Result.HolderID;

            base.Work(uow);

            uow.Execute(new UpdateCatalogSupplierStats
            {
                EntityID = holderID
            });
        }

        #endregion
    }

    /// <summary>
    /// Gets the catalog supplier rating.
    /// </summary>
    public class GetCatalogSupplierRating : GetRating<CatalogSupplierRatingEntity>,
        IApiOperation
    {
    }

    #endregion

    #region __Search__

    /// <summary>
    /// Search over catalog supplier rating collection.
    /// </summary>
    public class SearchCatalogSupplierRatings : SearchRatings<CatalogSupplierRatingEntity>,
        IApiOperation
    {
        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public SearchCatalogSupplierRatings()
        {
            SortDir = SortDirection.Ascending;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the flag mask.
        /// </summary>
        public CatalogSupplierRatingFlags FlagMask
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sort direction.
        /// </summary>
        public SortDirection SortDir
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override IQueryable<CatalogSupplierRatingEntity> Build(IUnitOfWork uow, IQueryable<CatalogSupplierRatingEntity> query)
        {
            query = base.Build(uow, query);
            query = query.FilterByFlags(x => x.FlagsValue, (int)FlagMask);
            query = query.OrderBy(x => x.CreatedOn, SortDir);
            return query;
        }

        #endregion
    }

    #endregion

    #region __Notification__

    /// <summary>
    /// Enqueues a catalog supplier rating notification message.
    /// </summary>
    /// <remarks>
    /// Message template code: NCATSUPRTG
    /// Allowed tokens: [T:Recipient.EntityID], [T:Recipient.Guid], [T:Recipient.Email], [T:Recipient.Username],
    /// [T:CatalogSupplierRating.Supplier.EntityID], [T:CatalogSupplierRating.Supplier.RatingAverage],
    /// [T:CatalogSupplierRating.Supplier.RatingCount], [T:CatalogSupplierRating.Supplier.Name],
    /// [T:CatalogSupplierRating.Supplier.ShortDescription], [T:CatalogSupplierRating.Account.EntityID],
    /// [T:CatalogSupplierRating.Account.Username], [T:CatalogSupplierRating.Account.Email],
    /// [T:CatalogSupplierRating.Account.Guid], [T:CatalogSupplierRating.EntityID],
    /// [T:CatalogSupplierRating.Value], [T:CatalogSupplierRating.CommentCount], [T:CatalogSupplierRating.Text],
    /// [T:CatalogSupplierRating.CreatedOn]
    /// </remarks>
    public class EnqueueCatalogSupplierRatingNotification : Operation<MessageEntity>
    {
        #region Properties

        /// <summary>
        /// Gets or sets the rating identifier.
        /// </summary>
        public int RatingID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the recipient account identifier.
        /// </summary>
        public int RecipientID
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow, out MessageEntity result)
        {
            var rating = uow.Execute(new GetCatalogSupplierRating
            {
                ThrowsException = true,
                EntityID = RatingID
            }).Result;
            var recipient = uow.Execute(new GetAccount
            {
                ThrowsException = true,
                EntityID = RecipientID
            }).Result;
            var usrLocale = recipient.LocalizationProfile;
            int utcOffset = usrLocale.TimeZone.UtcOffset;
            string dtFmt = usrLocale.Country.DateTimeFormat;
            var supplier = uow.Execute(new GetCatalogSupplier
            {
                ThrowsException = true,
                EntityID = rating.HolderID
            }).Result;
            var t_supplier = supplier.Translation(usrLocale.Language.EntityID);

            result = uow.Execute(new CreateTemplatedMessageByCode
            {
                TemplateCode = "NCATSUPRTG",
                LanguageID = usrLocale.Language.EntityID,
                Replacements = new Dictionary<String, String>
                {
                    {"Recipient.EntityID", recipient.EntityID.ToString()},
                    {"Recipient.Email", recipient.Email},
                    {"Recipient.Username", recipient.Username},
                    {"Recipient.Guid", recipient.Guid.ToString()},
                    {"CatalogSupplierRating.Supplier.EntityID", supplier.EntityID.ToString()},
                    {"CatalogSupplierRating.Supplier.RatingAverage", supplier.RatingAverage.ToString()},
                    {"CatalogSupplierRating.Supplier.RatingCount", supplier.RatingCount.ToString()},
                    {"CatalogSupplierRating.Supplier.Name", t_supplier != null 
                        ? t_supplier.Name 
                        : String.Empty},
                    {"CatalogSupplierRating.Supplier.ShortDescription", t_supplier != null 
                        ? t_supplier.ShortDescription 
                        : String.Empty},
                    {"CatalogSupplierRating.Account.EntityID", rating.Account != null 
                        ? rating.Account.EntityID.ToString() 
                        : String.Empty},
                    {"CatalogSupplierRating.Account.Username", rating.Account != null 
                        ? rating.Account.Username 
                        : String.Empty},
                    {"CatalogSupplierRating.Account.Email", rating.Account != null 
                        ? rating.Account.Email 
                        : String.Empty},
                    {"CatalogSupplierRating.Account.Guid", rating.Account != null 
                        ? rating.Account.Guid.ToString() 
                        : String.Empty},
                    {"CatalogSupplierRating.EntityID", rating.EntityID.ToString()},
                    {"CatalogSupplierRating.Value", rating.Value.ToString()},
                    {"CatalogSupplierRating.CommentCount", supplier.CommentCount.ToString()},
                    {"CatalogSupplierRating.CreatedOn", rating.CreatedOn.ToLocal(utcOffset).ToStringByFmt(dtFmt)}
                },
                To = recipient.Email,
                Flags = MessageFlags.Active
            }).Result;
        }

        #endregion
    }

    #endregion
}
