﻿using System;
using System.Collections.Generic;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the currency entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   | - Rate
    ///   | - PriceFormat
    ///   + Base
    ///    | - Name
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     | - Code
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class CurrencyEntity : AuditableEntity
    {
        #region Properties

        /// <summary>
        /// Gets or sets the code.
        /// </summary>
        /// <remarks>
        /// ISO 4217
        /// </remarks>
        public virtual string Code
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the name
        /// </summary>
        public virtual string Name
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the rate.
        /// </summary>
        public virtual decimal Rate
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the price format.
        /// </summary>
        public virtual string PriceFormat
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual CurrencyFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Keys)
                obj["Code"] = Code;
            if(level >= DisclosureLevel.Base)
            {
                obj["Name"] = Name;
                obj["Flags"] = Flags.ToString();
            }
            if(level >= DisclosureLevel.Extended)
            {
                obj["Rate"] = Rate;
                obj["PriceFormat"] = PriceFormat;
            }
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapper.
    /// </summary>
    public class CurrencyMapping : EntityMapping<CurrencyEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public CurrencyMapping()
        {
            Table("fwk_Currencies");
            Id(x => x.EntityID, m =>
            {
                m.Column("CurrencyID");
                m.Generator(Generators.Identity);
            });
            Property(x => x.Code, m =>
            {
                m.NotNullable(true);
                m.Length(3);
                m.Unique(true);
                m.Index("IX_fwk_Currencies");
                m.Column(c =>
                {
                    c.SqlType("nchar(3)");
                });
            });
            Property(x => x.Name, m =>
            {
                m.NotNullable(true);
                m.Length(100);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(100)");
                });
            });
            Property(x => x.Rate, m =>
            {
                m.NotNullable(true);
                m.Precision(19);
                m.Scale(4);
            });
            Property(x => x.PriceFormat, m =>
            {
                m.NotNullable(true);
                m.Length(100);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(100)");
                });
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
            Cache(m =>
            {
                m.Usage(CacheUsage.ReadWrite);
            });
        }
    }
}
