﻿using System;
using System.Collections.Generic;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the language pack entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   + Base
    ///    | - Language(of Base level)
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     | - Code
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class LanguagePackEntity : AuditableEntity
    {
        #region Properties

        /// <summary>
        /// Gets or sets the language.
        /// </summary>
        public virtual LanguageEntity Language
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the code.
        /// </summary>
        public virtual string Code
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the language pack content.
        /// </summary>
        public virtual string Content
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual int Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Keys)
                obj["Code"] = Code;
            if(level >= DisclosureLevel.Base)
            {
                obj["Language"] = Language.Disclose(DisclosureLevel.Base, options);
                obj["Flags"] = Flags.ToString();
            }
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapper.
    /// </summary>
    public class LanguagePackMapping : EntityMapping<LanguagePackEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public LanguagePackMapping()
        {
            Table("fwk_LanguagePacks");
            Id(x => x.EntityID, m =>
            {
                m.Column("PackID");
                m.Generator(Generators.Identity);
            });
            ManyToOne(x => x.Language, m =>
            {
                m.Column("LanguageID");
                m.ForeignKey("FK_fwk_LanguagePacks_fwk_Languages");
                m.NotNullable(true);
                m.Index("IX_fwk_LanguagePacks");
                m.Fetch(FetchKind.Join);
            });
            Property(x => x.Code, m =>
            {
                m.NotNullable(true);
                m.Length(10);
                m.Index("IX_fwk_LanguagePacks");
                m.Column(c =>
                {
                    c.SqlType("nvarchar(10)");
                });
            });
            Property(x => x.Content, m =>
            {
                m.NotNullable(true);
                m.Length(Int32.MaxValue);
                m.Column(c =>
                {
                    c.Name("[Content]");
                    c.SqlType("xml");
                });
                m.Lazy(true);
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
            Cache(m =>
            {
                m.Usage(CacheUsage.ReadWrite);
            });
        }
    }
}
