﻿using System;
using System.Collections.Generic;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the localization profile entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   + Base
    ///    | - Country(of same level)
    ///    | - Region(of same level)
    ///    | - TimeZone(of same level)
    ///    | - Currency(of same level)
    ///    | - WeightUnit(of same level)
    ///    | - DimensionUnit(of same level)
    ///    | - InformationUnit(of same level)
    ///    | - Name
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     | - Code
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class LocalizationProfileEntity : AuditableEntity
    {
        #region Properties

        /// <summary>
        /// Gets or sets the code.
        /// </summary>
        public virtual string Code
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        public virtual string Name
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the language.
        /// </summary>
        public virtual LanguageEntity Language
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the time zone.
        /// </summary>
        public virtual TimeZoneEntity TimeZone
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the country.
        /// </summary>
        public virtual CountryEntity Country
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the region.
        /// </summary>
        public virtual RegionEntity Region
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the currency.
        /// </summary>
        public virtual CurrencyEntity Currency
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the dimension unit.
        /// </summary>
        public virtual MeasureUnitEntity DimensionUnit
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the weight unit.
        /// </summary>
        public virtual MeasureUnitEntity WeightUnit
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the information unit.
        /// </summary>
        public virtual MeasureUnitEntity InformationUnit
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual LocalizationProfileFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Keys)
                obj["Code"] = Code;
            if(level >= DisclosureLevel.Base)
            {
                obj["Country"] = Country.Disclose(level, options);
                obj["Region"] = Region.Disclose(level, options);
                obj["TimeZone"] = TimeZone.Disclose(level, options);
                obj["Language"] = Language.Disclose(level, options);
                obj["Currency"] = Currency.Disclose(level, options);
                obj["WeightUnit"] = WeightUnit.Disclose(level, options);
                obj["DimensionUnit"] = DimensionUnit.Disclose(level, options);
                obj["InformationUnit"] = InformationUnit.Disclose(level, options);
                obj["Name"] = Name;
                obj["Flags"] = Flags.ToString();
            }
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapper.
    /// </summary>
    public class LocalizationProfileMapping : EntityMapping<LocalizationProfileEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public LocalizationProfileMapping()
        {
            Table("fwk_LocalizationProfiles");
            Id(x => x.EntityID, m =>
            {
                m.Column("ProfileID");
                m.Generator(Generators.Identity);
            });
            Property(x => x.Code, m =>
            {
                m.NotNullable(true);
                m.Length(10);
                m.Unique(true);
                m.Index("IX_fwk_LocalizationProfiles");
                m.Column(c =>
                {
                    c.SqlType("nvarchar(10)");
                });
            });
            Property(x => x.Name, m =>
            {
                m.NotNullable(true);
                m.Length(100);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(100)");
                });
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
            ManyToOne(x => x.Language, m =>
            {
                m.Column("LanguageID");
                m.ForeignKey("FK_fwk_LocalizationProfiles_fwk_Languages");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            ManyToOne(x => x.Currency, m =>
            {
                m.Column("CurrencyID");
                m.ForeignKey("FK_fwk_LocalizationProfiles_fwk_Currencies");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            ManyToOne(x => x.TimeZone, m =>
            {
                m.Column("TimeZoneID");
                m.ForeignKey("FK_fwk_LocalizationProfiles_fwk_TimeZones");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            ManyToOne(x => x.Country, m =>
            {
                m.Column("CountryID");
                m.ForeignKey("FK_fwk_LocalizationProfiles_fwk_Countries");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            ManyToOne(x => x.Region, m =>
            {
                m.Column("RegionID");
                m.ForeignKey("FK_fwk_LocalizationProfiles_fwk_Regions");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            ManyToOne(x => x.DimensionUnit, m =>
            {
                m.Column("DimensionUnitID");
                m.ForeignKey("FK_fwk_LocalizationProfiles_fwk_MeasureUnits");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            ManyToOne(x => x.WeightUnit, m =>
            {
                m.Column("WeightUnitID");
                m.ForeignKey("FK_fwk_LocalizationProfiles_fwk_MeasureUnits1");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            ManyToOne(x => x.InformationUnit, m =>
            {
                m.Column("InformationUnitID");
                m.ForeignKey("FK_fwk_LocalizationProfiles_fwk_MeasureUnits2");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            Cache(m =>
            {
                m.Usage(CacheUsage.ReadWrite);
            });
        }
    }
}
