﻿using System;
using System.Collections.Generic;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the message entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  | - Text
    ///  + Extended
    ///   | - Subject
    ///   + Base
    ///    | - SenderID
    ///    | - From
    ///    | - To
    ///    | - BccTo
    ///    | - ReplyTo
    ///    | - SendAttemptCount
    ///    | - Flags
    ///    | - SentOn
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class MessageEntity : AuditableEntity
    {
        #region Properties

        /// <summary>
        /// Gets or sets the sender unique identifier.
        /// </summary>
        public virtual Guid SenderID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sender address.
        /// </summary>
        public virtual string From
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the recipient address.
        /// </summary>
        public virtual string To
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the BCC recipient address.
        /// </summary>
        public virtual string BccTo
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the address where to reply to.
        /// </summary>
        public virtual string ReplyTo
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the subject.
        /// </summary>
        public virtual string Subject
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the text.
        /// </summary>
        public virtual string Text
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the number of send attempts.
        /// </summary>
        public virtual int SendAttemptCount
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual MessageFlags Flags
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sending date.
        /// </summary>
        public virtual DateTime? SentOn
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
            {
                obj["SenderID"] = SenderID;
                obj["From"] = From;
                obj["To"] = To;
                obj["BccTo"] = BccTo;
                obj["ReplyTo"] = ReplyTo;
                obj["SendAttemptCount"] = SendAttemptCount;
                obj["Flags"] = Flags.ToString();
                obj["SentOn"] = SentOn;
            }
            if(level >= DisclosureLevel.Extended)
                obj["Subject"] = Subject;
            if(level >= DisclosureLevel.Entire)
                obj["Text"] = Text;
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class MessageMapping : EntityMapping<MessageEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public MessageMapping()
        {
            Table("fwk_Messages");
            Id(x => x.EntityID, m =>
            {
                m.Column("MessageID");
                m.Generator(Generators.Identity);
            });
            Property(x => x.SenderID, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.From, m =>
            {
                m.NotNullable(true);
                m.Length(100);
                m.Column(c =>
                {
                    c.Name("[From]");
                    c.SqlType("nvarchar(10)");
                });
            });
            Property(x => x.To, m =>
            {
                m.NotNullable(true);
                m.Length(100);
                m.Column(c =>
                {
                    c.Name("[To]");
                    c.SqlType("nvarchar(100)");
                });
            });
            Property(x => x.BccTo, m =>
            {
                m.NotNullable(true);
                m.Length(100);
            });
            Property(x => x.ReplyTo, m =>
            {
                m.NotNullable(true);
                m.Length(100);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(100)");
                });
            });
            Property(x => x.Subject, m =>
            {
                m.NotNullable(true);
                m.Length(100);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(100)");
                });
            });
            Property(x => x.Text, m =>
            {
                m.NotNullable(true);
                m.Length(Int32.MaxValue);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(max)");
                });
            });
            Property(x => x.SendAttemptCount, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.SentOn, m =>
            {
                m.NotNullable(false);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
        }
    }
}
