﻿using System;
using System.Collections.Generic;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;
using Xenta.Utils;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the setting entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   + Base
    ///    | - Value
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     | - Key
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class SettingEntity : AuditableEntity
    {
        #region Properties

        /// <summary>
        /// Gets or sets the key.
        /// </summary>
        public virtual string Key
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the value.
        /// </summary>
        public virtual string Value
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual int Flags
        {
            get;
            set;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the setting value converted into type T.
        /// </summary>
        /// <typeparam name="T">The target type to convert.</typeparam>
        /// <param name="defaultValue">The default value.</param>
        /// <returns>The value as type T.</returns>
        public virtual T ValueAs<T>(T defaultValue = default(T))
        {
            return Value.ParseAs(defaultValue);
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Keys)
                obj["Key"] = Key;
            if(level >= DisclosureLevel.Base)
                obj["Value"] = Value;
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapper.
    /// </summary>
    public class SettingMapping : EntityMapping<SettingEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public SettingMapping()
        {
            Table("fwk_Settings");
            Id(x => x.EntityID, m =>
            {
                m.Column("SettingID");
                m.Generator(Generators.Identity);
            });
            Property(x => x.Key, m =>
            {
                m.NotNullable(true);
                m.Length(100);
                m.Index("IX_fwk_Settings");
                m.Column(c =>
                {
                    c.Name("[Key]");
                    c.SqlType("nvarchar(100)");
                });
            });
            Property(x => x.Value, m =>
            {
                m.NotNullable(true);
                m.Length(1000);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(1000)");
                });
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
            Cache(m =>
            {
                m.Usage(CacheUsage.ReadWrite);
            });
        }
    }
}
