﻿using System;
using System.Collections.Generic;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the message template translation entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  | - Text
    ///  + Extended
    ///   + Base
    ///    | - Holder(of the Primary level)
    ///    | - Language(of the Keys level)
    ///    | - Subject
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class TMessageTemplateEntity : TranslationEntity<MessageTemplateEntity>
    {
        #region Properties

        /// <summary>
        /// Gets or sets the subject.
        /// </summary>
        public virtual string Subject
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the text.
        /// </summary>
        public virtual string Text
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
                obj["Subject"] = Subject;
            if(level >= DisclosureLevel.Entire)
                obj["Text"] = Text;
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class TMessageTemplateMapping : EntityMapping<TMessageTemplateEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public TMessageTemplateMapping()
        {
            Table("fwk_TMessageTemplates");
            Id(x => x.EntityID, m =>
            {
                m.Column("TranslationID");
                m.Generator(Generators.Identity);
            });
            ManyToOne(x => x.Holder, m =>
            {
                m.Column("TemplateID");
                m.ForeignKey("FK_fwk_TMessageTemplates_fwk_MessageTemplates");
                m.Index("IX_fwk_TMessageTemplates");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            ManyToOne(x => x.Language, m =>
            {
                m.Column("LanguageID");
                m.ForeignKey("FK_fwk_TMessageTemplates_fwk_Languages");
                m.Index("IX_fwk_TMessageTemplates");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            Property(x => x.Subject, m =>
            {
                m.NotNullable(true);
                m.Length(100);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(100)");
                });
            });
            Property(x => x.Text, m =>
            {
                m.NotNullable(true);
                m.Length(Int32.MaxValue);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(max)");
                });
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
        }
    }
}
