﻿using System;
using System.Collections.Generic;
using System.Linq;
using Xenta.Api;
using Xenta.Entities;
using Xenta.Enums;
using Xenta.Security;
using Xenta.Structs;
using Xenta.Utils;

namespace Xenta.Operations
{
    #region __General__

    /// <summary>
    /// Creates a country.
    /// </summary>
    public class CreateCountry : CreateAuditable<CountryEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public CreateCountry()
        {
            NumericFormat = NumericFormat.Default;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("GLBLZ");
            }
        }

        /// <summary>
        /// Gets or sets the code.
        /// </summary>
        public string Code
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        public string Name
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the date format.
        /// </summary>
        public string DateFormat
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the time format.
        /// </summary>
        public string TimeFormat
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the numeric format.
        /// </summary>
        public NumericFormat NumericFormat
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public CountryFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override Err Validate(IUnitOfWork uow)
        {
            if(String.IsNullOrWhiteSpace(Code))
                return Err.Of<CountryEntity>().NotValid("Code");
            return base.Validate(uow);
        }

        protected override void Construct(IUnitOfWork uow, CountryEntity entity)
        {
            base.Construct(uow, entity);

            entity.Code = Code.TrimToLen(2);
            entity.Name = StringHelper.EnsureNotNull(Name).TrimToLen(100);
            entity.DateFormat = StringHelper.EnsureNotNull(DateFormat).TrimToLen(100);
            entity.TimeFormat = StringHelper.EnsureNotNull(TimeFormat).TrimToLen(100);
            entity.RawNumericFormat = NumericFormat.ToString().TrimToLen(100);
            entity.Flags = Flags;
        }

        #endregion
    }

    /// <summary>
    /// Updates the country.
    /// </summary>
    public class UpdateCountry : UpdateAuditable<CountryEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public UpdateCountry()
        {
            NumericFormat = NumericFormat.Default;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("GLBLZ");
            }
        }

        /// <summary>
        /// Gets or sets the code.
        /// </summary>
        public string Code
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        public string Name
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the date format.
        /// </summary>
        public string DateFormat
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the time format.
        /// </summary>
        public string TimeFormat
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the numeric format.
        /// </summary>
        public NumericFormat NumericFormat
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public CountryFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override Err Validate(IUnitOfWork uow)
        {
            if(String.IsNullOrWhiteSpace(Code))
                return Err.Of<CountryEntity>().NotValid("Code");
            return base.Validate(uow);
        }

        protected override void Revise(IUnitOfWork uow, CountryEntity entity)
        {
            base.Revise(uow, entity);

            entity.Code = Code.TrimToLen(2);
            entity.Name = StringHelper.EnsureNotNull(Name).TrimToLen(100);
            entity.DateFormat = StringHelper.EnsureNotNull(DateFormat).TrimToLen(100);
            entity.TimeFormat = StringHelper.EnsureNotNull(TimeFormat).TrimToLen(100);
            entity.RawNumericFormat = NumericFormat.ToString().TrimToLen(100);
            entity.Flags = Flags;
        }

        #endregion
    }

    /// <summary>
    /// Deletes the country.
    /// </summary>
    public class DeleteCountry : DeleteEntity<CountryEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("GLBLZ");
            }
        }

        #endregion

        #region Utilities

        protected override void Destruct(IUnitOfWork uow, CountryEntity entity)
        {
            foreach(var i in entity.Regions)
            {
                uow.Execute(new DeleteRegion
                {
                    EntityID = i.EntityID
                });
            }
            base.Destruct(uow, entity);
        }

        #endregion
    }

    /// <summary>
    /// Gets the country by identifier.
    /// </summary>
    public class GetCountry : GetEntity<CountryEntity>,
        IApiOperation
    {
    }

    /// <summary>
    /// Gets the country by code.
    /// </summary>
    public class GetCountryByCode : QueryEntity<CountryEntity>,
        IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets or sets the country code.
        /// </summary>
        public string Code
        {
            get;
            set;
        }

        #endregion

        #region Options

        protected override bool Cacheable
        {
            get
            {
                return true;
            }
        }

        #endregion

        #region Utilities

        protected override IQueryable<CountryEntity> Build(IUnitOfWork uow, IQueryable<CountryEntity> query)
        {
            string code = StringHelper.EnsureNotNull(Code).TrimToLen(2);
            return base.Build(uow, query).Where(x => x.Code.Equals(code));
        }

        #endregion
    }

    #endregion

    #region __Search__

    /// <summary>
    /// Search over country collection.
    /// </summary>
    public class SearchCountries : SearchOperation<CountryEntity>,
        IApiOperation
    {
        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public SearchCountries()
        {
            SortDir = SortDirection.Ascending;
            SortBy = new []
            {
                CountrySortBy.ID
            };
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the flag mask.
        /// </summary>
        public CountryFlags FlagMask
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sort direction.
        /// </summary>
        public SortDirection SortDir
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sort by.
        /// </summary>
        public CountrySortBy[] SortBy
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override IQueryable<CountryEntity> Build(IUnitOfWork uow, IQueryable<CountryEntity> query)
        {
            query = base.Build(uow, query);
            query = query.FilterByFlags(x => (int)x.Flags, (int)FlagMask);
            if(SortBy != null)
            {
                for(int i = 0; i < SortBy.Length; i++)
                {
                    switch(SortBy[i])
                    {
                        case CountrySortBy.Code:
                            query = query.OrderBy(x => x.Code, SortDir, i != 0);
                            break;
                        case CountrySortBy.Name:
                            query = query.OrderBy(x => x.Name, SortDir, i != 0);
                            break;
                        default:
                            query = query.OrderBy(x => x.EntityID, SortDir, i != 0);
                            break;
                    }
                }
            }
            return query;
        }

        #endregion
    }

    /// <summary>
    /// Gets the collection of countries as a dictionary, where 
    /// key - country identifier and value - country code.
    /// </summary>
    public class GetCountryDictionary : SearchOperation<CountryEntity, Int32, String>,
        IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets or sets the flag mask.
        /// </summary>
        public CountryFlags FlagMask
        {
            get;
            set;
        }

        #endregion

        #region Options

        protected override bool Cacheable
        {
            get
            {
                return true;
            }
        }

        #endregion

        #region Utilities

        protected override IQueryable<CountryEntity> Build(IUnitOfWork uow, IQueryable<CountryEntity> query)
        {
            return base.Build(uow, query)
                .FilterByFlags(x => (int)x.Flags, (int)FlagMask)
                .OrderBy(x => x.Code);
        }

        protected override IQueryable<KeyValuePair<Int32, String>> Map(IQueryable<CountryEntity> query)
        {
            return query.Select(x => new KeyValuePair<Int32, String>(x.EntityID, x.Name));
        }

        #endregion
    }

    #endregion
}
