﻿using System;
using System.Collections.Generic;
using System.Linq;
using Xenta.Api;
using Xenta.Entities;
using Xenta.Enums;
using Xenta.Security;
using Xenta.Utils;

namespace Xenta.Operations
{
    #region __General__

    /// <summary>
    /// Creates a measure unit.
    /// </summary>
    public class CreateMeasureUnit : CreateAuditable<MeasureUnitEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("GLBLZ");
            }
        }

        /// <summary>
        /// Gets or sets the code.
        /// </summary>
        public string Code
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        public string Name
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the ratio.
        /// </summary>
        public decimal Ratio
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the value format.
        /// </summary>
        public string ValueFormat
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public MeasureUnitFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override Err Validate(IUnitOfWork uow)
        {
            if(String.IsNullOrWhiteSpace(Code))
                return Err.Of<MeasureUnitEntity>().NotValid("Code");
            if(Ratio <= Decimal.Zero)
                return Err.Of<MeasureUnitEntity>().NotValid("Ratio");
            return base.Validate(uow);
        }

        protected override void Construct(IUnitOfWork uow, MeasureUnitEntity entity)
        {
            base.Construct(uow, entity);

            entity.Code = Code.TrimToLen(10);
            entity.Name = StringHelper.EnsureNotNull(Name).TrimToLen(100);
            entity.Ratio = Ratio;
            entity.ValueFormat = StringHelper.EnsureNotNull(ValueFormat).TrimToLen(100);
            entity.Flags = Flags;
        }

        #endregion
    }

    /// <summary>
    /// Updates a measure unit.
    /// </summary>
    public class UpdateMeasureUnit : UpdateAuditable<MeasureUnitEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("GLBLZ");
            }
        }

        /// <summary>
        /// Gets or sets the code.
        /// </summary>
        public string Code
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        public string Name
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the ratio.
        /// </summary>
        public decimal Ratio
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the value format.
        /// </summary>
        public string ValueFormat
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public MeasureUnitFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override Err Validate(IUnitOfWork uow)
        {
            if(String.IsNullOrWhiteSpace(Code))
                return Err.Of<MeasureUnitEntity>().NotValid("Code");
            if(Ratio <= Decimal.Zero)
                return Err.Of<MeasureUnitEntity>().NotValid("Ratio");
            return base.Validate(uow);
        }

        protected override void Revise(IUnitOfWork uow, MeasureUnitEntity entity)
        {
            base.Revise(uow, entity);

            entity.Code = Code.TrimToLen(10);
            entity.Name = StringHelper.EnsureNotNull(Name).TrimToLen(100);
            entity.Ratio = Ratio;
            entity.ValueFormat = StringHelper.EnsureNotNull(ValueFormat).TrimToLen(100);
            entity.Flags = Flags;
        }

        #endregion
    }

    /// <summary>
    /// Deletes a measure unit.
    /// </summary>
    public class DeleteMeasureUnit : DeleteEntity<MeasureUnitEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("GLBLZ");
            }
        }

        #endregion
    }

    /// <summary>
    /// Gets the measure unit by identifier.
    /// </summary>
    public class GetMeasureUnit : GetEntity<MeasureUnitEntity>,
        IApiOperation
    {
    }

    /// <summary>
    /// Gets the measure unit by code.
    /// </summary>
    public class GetMeasureUnitByCode : QueryEntity<MeasureUnitEntity>,
        IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets or sets the country code.
        /// </summary>
        public string Code
        {
            get;
            set;
        }

        #endregion

        #region Options

        protected override bool Cacheable
        {
            get
            {
                return true;
            }
        }

        #endregion

        #region Utilities

        protected override IQueryable<MeasureUnitEntity> Build(IUnitOfWork uow, IQueryable<MeasureUnitEntity> query)
        {
            string code = StringHelper.EnsureNotNull(Code).TrimToLen(10);
            return base.Build(uow, query).Where(x => x.Code.Equals(code));
        }

        #endregion
    }

    #endregion

    #region __Search__

    /// <summary>
    /// Search over measure unit collection.
    /// </summary>
    public class SearchMeasureUnits : SearchOperation<MeasureUnitEntity>,
        IApiOperation
    {
        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public SearchMeasureUnits()
        {
            SortDir = SortDirection.Ascending;
            SortBy = new []
            {
                MeasureUnitSortBy.ID
            };
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the flag mask.
        /// </summary>
        public MeasureUnitFlags FlagMask
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sort direction.
        /// </summary>
        public SortDirection SortDir
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sort by.
        /// </summary>
        public MeasureUnitSortBy[] SortBy
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override IQueryable<MeasureUnitEntity> Build(IUnitOfWork uow, IQueryable<MeasureUnitEntity> query)
        {
            query = base.Build(uow, query);
            query = query.FilterByFlags(x => (int)x.Flags, (int)FlagMask);
            if(SortBy != null)
            {
                for(int i = 0; i < SortBy.Length; i++)
                {
                    switch(SortBy[i])
                    {
                        case MeasureUnitSortBy.Code:
                            query = query.OrderBy(x => x.Code, SortDir, i != 0);
                            break;
                        case MeasureUnitSortBy.Name:
                            query = query.OrderBy(x => x.Name, SortDir, i != 0);
                            break;
                        case MeasureUnitSortBy.Ratio:
                            query = query.OrderBy(x => x.Ratio, SortDir, i != 0);
                            break;
                        default:
                            query = query.OrderBy(x => x.EntityID, SortDir, i != 0);
                            break;
                    }
                }
            }
            return query;
        }

        #endregion
    }

    /// <summary>
    /// Gets the collection of units as a dictionary, where 
    /// key - unit identifier and value - time zone code.
    /// </summary>
    public class GetMeasureUnitDictionary : SearchOperation<MeasureUnitEntity, Int32, String>,
        IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets or sets the flag mask.
        /// </summary>
        public MeasureUnitFlags FlagMask
        {
            get;
            set;
        }

        #endregion

        #region Options

        protected override bool Cacheable
        {
            get
            {
                return true;
            }
        }

        #endregion

        #region Utilities

        protected override IQueryable<MeasureUnitEntity> Build(IUnitOfWork uow, IQueryable<MeasureUnitEntity> query)
        {
            return base.Build(uow, query)
                .FilterByFlags(x => (int)x.Flags, (int)FlagMask)
                .OrderBy(x => x.Code);
        }

        protected override IQueryable<KeyValuePair<Int32, String>> Map(IQueryable<MeasureUnitEntity> query)
        {
            return query.Select(x => new KeyValuePair<Int32, String>(x.EntityID, x.Name));
        }

        #endregion
    }

    #endregion  
}
