﻿using System;
using System.Collections.Generic;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the comment entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///  | - Text
    ///   + Base
    ///    | - HolderID
    ///    | - Account(of the Keys level)
    ///    | - Title
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public abstract class CommentEntity : AuditableEntity
    {
        #region Fields

        private AccountEntity _account;

        #endregion

        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        protected CommentEntity()
        {
            _account = null;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the holder identifier.
        /// </summary>
        public virtual int HolderID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the account identifier.
        /// </summary>
        public virtual int AccountID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the account.
        /// </summary>
        public virtual AccountEntity Account
        {
            get
            {
                return AccountID != 0 ? _account : null;
            }
            set
            {
                _account = value;
                AccountID = value != null ? value.EntityID : 0;
            }
        }

        /// <summary>
        /// Gets or sets the title.
        /// </summary>
        public virtual string Title
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the text.
        /// </summary>
        public virtual string Text
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags value.
        /// </summary>
        public virtual int FlagsValue
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
            {
                obj["HolderID"] = HolderID;
                obj["Account"] = Account != null
                    ? Account.Disclose(DisclosureLevel.Keys, options)
                    : null;
                obj["Title"] = Title;
            }
            if(level >= DisclosureLevel.Extended)
                obj["Text"] = Text;
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class CommentMapping : EntityMapping<CommentEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public CommentMapping()
        {
            Table("fwk_Comments");
            Id(x => x.EntityID, m =>
            {
                m.Column("CommentID");
                m.Generator(Generators.Identity);
            });
            Discriminator(m =>
            {
                m.Column("Discriminator");
                m.Length(10);
                m.Force(true);
                m.NotNullable(true);
            });
            Property(x => x.HolderID, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.AccountID, m =>
            {
                m.Column("AccountID");
                m.NotNullable(true);
            });
            ManyToOne(x => x.Account, m =>
            {
                m.Column("AccountID");
                m.NotFound(NotFoundMode.Ignore);
                m.Insert(false);
                m.Update(false);
            });
            Property(x => x.Title, m =>
            {
                m.NotNullable(true);
                m.Length(100);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(1000)");
                });
            });
            Property(x => x.Text, m =>
            {
                m.NotNullable(true);
                m.Length(1000);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(4000)");
                });
            });
            Property(x => x.FlagsValue, m =>
            {
                m.Column("Flags");
                m.NotNullable(true);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
        }
    }
}
