﻿using System;
using System.Collections.Generic;
using System.Linq;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;
using Xenta.Utils;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the rank entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   + Base
    ///    | - Threshold
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    | - Translations(of the same level)
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class ForumRankEntity : AuditableEntity
    {
        #region Fields

        private ICollection<ForumMemberEntity> _members;
        private ICollection<TForumRankEntity> _translations;

        #endregion

        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public ForumRankEntity()
        {
            _members = new List<ForumMemberEntity>();
            _translations = new List<TForumRankEntity>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the threshold.
        /// </summary>
        public virtual int Threshold
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual ForumRankFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Collections

        /// <summary>
        /// Gets or sets the member collection.
        /// </summary>
        public virtual ICollection<ForumMemberEntity> Members
        {
            get
            {
                return _members;
            }
            set
            {
                _members = value;
            }
        }

        /// <summary>
        /// Gets or sets the rank translations.
        /// </summary>
        public virtual ICollection<TForumRankEntity> Translations
        {
            get
            {
                return _translations;
            }
            set
            {
                _translations = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the rank translation.
        /// </summary>
        /// <param name="languageID">The language identifier.</param>
        /// <returns>The rank translation.</returns>
        public virtual TForumRankEntity Translation(int languageID)
        {
            return _translations.SingleOrDefault(x => x.Language.EntityID == languageID);
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
            {
                obj["Threshold"] = Threshold;
                obj["Flags"] = Flags.ToString();
                obj["Translations"] = (options.ContainsKey("TranslatedTo")
                    ? Translations.Where(x => options["TranslatedTo"].Contains(x.Language.Code))
                    : Translations)
                    .Map(x => x.Disclose(level, options));
            }
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class ForumRankMapping : EntityMapping<ForumRankEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public ForumRankMapping()
        {
            Table("fwk_ForumRanks");
            Id(x => x.EntityID, m =>
            {
                m.Column("RankID");
                m.Generator(Generators.Identity);
            });
            Property(x => x.Threshold, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
            Bag(x => x.Members, m =>
            {
                m.Table("fwk_ForumMembers");
                m.Key(k =>
                {
                    k.Column("RankID");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.OneToMany();
            });
            Bag(x => x.Translations, m =>
            {
                m.Table("fwk_TForumRanks");
                m.Key(k =>
                {
                    k.Column("RankID");
                    k.ForeignKey("FK_fwk_TForumRanks_fwk_ForumRanks");
                    k.NotNullable(true);
                });
                m.Inverse(true);
                m.BatchSize(10);
            },
            a =>
            {
                a.OneToMany();
            });
            Cache(m =>
            {
                m.Usage(CacheUsage.ReadWrite);
            });
        }
    }
}
