﻿using System;
using System.Collections.Generic;
using System.Linq;
using NHibernate.Linq;
using Xenta.Api;
using Xenta.Entities;
using Xenta.Enums;
using Xenta.Security;
using Xenta.Structs;
using Xenta.Utils;

namespace Xenta.Operations
{
    #region __General__

    /// <summary>
    /// Creates a forum post.
    /// </summary>
    public class CreateForumPost : CreateAuditable<ForumPostEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("FORUM");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var member = ctx.ForumMember(uow);
                        if(member == null ||
                            !member.Flags.HasFlag(ForumMemberFlags.Active) || 
                            member.EntityID != MemberID)
                        {
                            return false;
                        }
                        var topic = uow.Execute(new GetForumTopic
                        {
                            EntityID = TopicID
                        }).Result;
                        if(topic == null)
                            return false;
                        var acl = topic.Forum.AccessRules;
                        return !acl.Any() || acl.Any(x => member.Account.Roles.Contains(x.Role) &&
                                x.AccessRights >= ForumAccessRights.ReadWrite);
                    });
            }
        }

        /// <summary>
        /// Gets or sets the member identifier.
        /// </summary>
        public int MemberID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the topic identifier.
        /// </summary>
        public int TopicID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the parent post identifier.
        /// </summary>
        public int ParentID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the text.
        /// </summary>
        public string Text
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public ForumPostFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override Err Validate(IUnitOfWork uow)
        {
            if(String.IsNullOrEmpty(Text))
                return Err.Of<ForumPostEntity>().NotValid("Text");
            return base.Validate(uow);
        }

        protected override void Construct(IUnitOfWork uow, ForumPostEntity entity)
        {
            base.Construct(uow, entity);

            entity.Member = uow.Execute(new GetForumMember
            {
                ThrowsException = true,
                EntityID = MemberID
            }).Result;
            entity.Topic = uow.Execute(new GetForumTopic
            {
                ThrowsException = true,
                EntityID = TopicID
            }).Result;
            entity.Parent = ParentID != 0 
                ? uow.Execute(new GetForumPost
                {
                    ThrowsException = true,
                    EntityID = ParentID
                }).Result 
                : null;
            entity.Text = Text;
            entity.Flags = Flags;
        }

        protected override void Work(IUnitOfWork uow, out ForumPostEntity result)
        {
            base.Work(uow, out result);
            uow.Execute(new UpdateForumTopicStats
            {
                EntityID = result.Topic.EntityID
            });
            uow.Execute(new UpdateForumMemberStats
            {
                EntityID = result.Member.EntityID
            });
        }

        #endregion
    }

    /// <summary>
    /// Updates the forum post.
    /// </summary>
    public class UpdateForumPost : UpdateAuditable<ForumPostEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("FORUM");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var member = ctx.ForumMember(uow);
                        if(member == null || !member.Flags.HasFlag(ForumMemberFlags.Active))
                            return false;
                        var post = uow.Execute(new GetForumPost
                        {
                            EntityID = EntityID
                        }).Result;
                        if(post == null)
                            return false;
                        if(post.Member.EntityID == member.EntityID)
                            return true;
                        var acl = post.Topic.Forum.AccessRules;
                        return acl.Any(x => member.Account.Roles.Contains(x.Role) &&
                            x.AccessRights >= ForumAccessRights.Moderate);
                    });
            }
        }

        /// <summary>
        /// Gets or sets the text.
        /// </summary>
        public string Text
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public ForumPostFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override Err Validate(IUnitOfWork uow)
        {
            if(String.IsNullOrEmpty(Text))
                return Err.Of<ForumPostEntity>().NotValid("Text");
            return base.Validate(uow);
        }

        protected override void Revise(IUnitOfWork uow, ForumPostEntity entity)
        {
            base.Revise(uow, entity);

            entity.Text = Text;
            entity.Flags = Flags;
        }

        protected override void Work(IUnitOfWork uow, out ForumPostEntity result)
        {
            base.Work(uow, out result);
            uow.Execute(new UpdateForumMemberStats
            {
                EntityID = result.Member.EntityID
            });
        }

        #endregion
    }

    /// <summary>
    /// Deletes the forum post.
    /// </summary>
    public class DeleteForumPost : DeleteEntity<ForumPostEntity>,
        IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("FORUM");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var member = ctx.ForumMember(uow);
                        if(member == null || !member.Flags.HasFlag(ForumMemberFlags.Active))
                            return false;
                        var post = uow.Execute(new GetForumPost
                        {
                            EntityID = EntityID
                        }).Result;
                        if(post == null)
                            return false;
                        var acl = post.Topic.Forum.AccessRules;
                        return acl.Any(x => member.Account.Roles.Contains(x.Role) &&
                            x.AccessRights >= ForumAccessRights.Moderate);
                    });
            }
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow)
        {
            var post = uow.Execute(new GetForumPost
            {
                ThrowsException = true,
                EntityID = EntityID
            }).Result;
            var topic = post.Topic;
            var member = post.Member;

            base.Work(uow);

            uow.Execute(new UpdateForumTopicStats
            {
                EntityID = topic.EntityID
            });
            uow.Execute(new UpdateForumMemberStats
            {
                EntityID = member.EntityID
            });
        }

        #endregion
    }

    /// <summary>
    /// Gets the forum post by identifier.
    /// </summary>
    public class GetForumPost : QueryEntityByID<ForumPostEntity>,
        IApiOperation
    {
        #region Utilities

        protected override IQueryable<ForumPostEntity> Build(IUnitOfWork uow, IQueryable<ForumPostEntity> query)
        {
            query = base.Build(uow, query); 
            if(Fetch > FetchLevel.None)
                query = query.Fetch(x => x.Member)
                    .ThenFetch(x => x.Account)
                    .Fetch(x => x.Topic);
            return query;
        }

        #endregion
    }

    #endregion

    #region __Search__

    /// <summary>
    /// Search over forum post collection.
    /// </summary>
    public class SearchForumPosts : SearchOperation<ForumPostEntity>,
        IApiOperation
    {
        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public SearchForumPosts()
        {
            Term = SearchTerm.Empty;
            SortDir = SortDirection.Ascending;
            SortBy = new []
            {
                ForumPostSortBy.ID
            };
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the search term.
        /// </summary>
        public SearchTerm Term
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the topic identifier.
        /// </summary>
        public int? TopicID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the member identifier.
        /// </summary>
        public int? MemberID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flag mask.
        /// </summary>
        public ForumPostFlags FlagMask
        {
            get;
            set;
        }

        /// <summary>
        ///   Gets or sets the creation date 'from'.
        /// </summary>
        public DateTime? CreatedFrom
        {
            get;
            set;
        }

        /// <summary>
        ///   Gets or sets the creation date 'to'.
        /// </summary>
        public DateTime? CreatedTo
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sort direction.
        /// </summary>
        public SortDirection SortDir
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sort by.
        /// </summary>
        public ForumPostSortBy[] SortBy
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override IQueryable<ForumPostEntity> Build(IUnitOfWork uow, IQueryable<ForumPostEntity> query)
        {
            query = base.Build(uow, query);
            query = query.Fetch(x => x.Member)
                .ThenFetch(x => x.Account)
                .Fetch(x => x.Topic);
            query = query.FilterByFlags(x => (int)x.Flags, (int)FlagMask);
            query = query.FilterByDateRange(x => x.CreatedOn, CreatedFrom, CreatedTo);
            switch(Term.Mode)
            {
                case SearchTerm.SearchMode.StartsWith:
                    query = query.Where(x => x.Text.StartsWith(Term.Keyword));
                    break;
                case SearchTerm.SearchMode.EndsWith:
                    query = query.Where(x => x.Text.EndsWith(Term.Keyword));
                    break;
                case SearchTerm.SearchMode.Contains:
                    query = query.Where(x => x.Text.Contains(Term.Keyword));
                    break;
                case SearchTerm.SearchMode.Equals:
                    query = query.Where(x => x.Text.Equals(Term.Keyword));
                    break;
            }
            if(TopicID.HasValue)
                query = query.Where(x => x.Topic.EntityID == TopicID.Value);
            if(MemberID.HasValue)
                query = query.Where(x => x.Member.EntityID == MemberID.Value);
            if(SortBy != null)
            {
                for(int i = 0; i < SortBy.Length; i++)
                {
                    switch(SortBy[i])
                    {
                        case ForumPostSortBy.CreatedOn:
                            query = query.OrderBy(x => x.CreatedOn, SortDir, i != 0);
                            break;
                        default:
                            query = query.OrderBy(x => x.EntityID, SortDir, i != 0);
                            break;
                    }
                }
            }
            return query;
        }

        #endregion
    }

    #endregion

    #region __Misc__

    /// <summary>
    /// Marks the forum post as useful.
    /// </summary>
    public class MarkForumPostAsUseful : UpdateAuditable<ForumPostEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("FORUM");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var member = ctx.ForumMember(uow);
                        if(member == null || !member.Flags.HasFlag(ForumMemberFlags.Active))
                            return false;
                        var post = uow.Execute(new GetForumPost
                        {
                            EntityID = EntityID
                        }).Result;
                        if(post == null)
                            return false;
                        if(post.Topic.Member.EntityID == member.EntityID)
                            return true;
                        var acl = post.Topic.Forum.AccessRules;
                        return acl.Any(x => member.Account.Roles.Contains(x.Role) &&
                            x.AccessRights >= ForumAccessRights.Moderate);
                    });
            }
        }

        #endregion

        #region Utilities

        protected override void Revise(IUnitOfWork uow, ForumPostEntity entity)
        {
            base.Revise(uow, entity);

            entity.Flags = (entity.Flags & (~ForumPostFlags.Useless)) | ForumPostFlags.Useful;
        }

        protected override void Work(IUnitOfWork uow, out ForumPostEntity result)
        {
            base.Work(uow, out result);
            uow.Execute(new UpdateForumMemberStats
            {
                EntityID = result.Member.EntityID
            });
        }

        #endregion
    }

    /// <summary>
    /// Marks the forum post as useless.
    /// </summary>
    public class MarkForumPostAsUseless : UpdateAuditable<ForumPostEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("FORUM");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) =>
                    {
                        var member = ctx.ForumMember(uow);
                        if(member == null || !member.Flags.HasFlag(ForumMemberFlags.Active))
                            return false;
                        var post = uow.Execute(new GetForumPost
                        {
                            EntityID = EntityID
                        }).Result;
                        if(post == null)
                            return false;
                        if(post.Topic.Member.EntityID == member.EntityID)
                            return true;
                        var acl = post.Topic.Forum.AccessRules;
                        return acl.Any(x => member.Account.Roles.Contains(x.Role) &&
                            x.AccessRights >= ForumAccessRights.Moderate);
                    });
            }
        }

        #endregion

        #region Utilities

        protected override void Revise(IUnitOfWork uow, ForumPostEntity entity)
        {
            base.Revise(uow, entity);

            entity.Flags = (entity.Flags & (~ForumPostFlags.Useful)) | ForumPostFlags.Useless;
        }

        protected override void Work(IUnitOfWork uow, out ForumPostEntity result)
        {
            base.Work(uow, out result);
            uow.Execute(new UpdateForumMemberStats
            {
                EntityID = result.Member.EntityID
            });
        }

        #endregion
    }

    #endregion
}
