﻿using System;
using System.Collections.Generic;
using System.Linq;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;
using Xenta.Utils;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the tag entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   + Base
    ///    | - Weight
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    | - Translations(of the same level)
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class PublicationTagEntity : AuditableEntity
    {
        #region Fields

        private ICollection<PublicationEntity> _publications;
        private ICollection<TPublicationTagEntity> _translations;

        #endregion

        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public PublicationTagEntity()
        {
            _publications = new List<PublicationEntity>();
            _translations = new List<TPublicationTagEntity>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the weight.
        /// </summary>
        public virtual int Weight
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual PublicationTagFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Collections

        /// <summary>
        /// Gets or sets the publication collection.
        /// </summary>
        public virtual ICollection<PublicationEntity> Publications
        {
            get
            {
                return _publications;
            }
            set
            {
                _publications = value;
            }
        }

        /// <summary>
        /// Gets or sets the tag translations.
        /// </summary>
        public virtual ICollection<TPublicationTagEntity> Translations
        {
            get
            {
                return _translations;
            }
            set
            {
                _translations = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the tag translation.
        /// </summary>
        /// <param name="languageID">The language identifier.</param>
        /// <returns>The tag translation.</returns>
        public virtual TPublicationTagEntity Translation(int languageID)
        {
            return _translations.SingleOrDefault(x => x.Language.EntityID == languageID);
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
            {
                obj["Weight"] = Weight;
                obj["Flags"] = Flags.ToString();
                obj["Translations"] = (options.ContainsKey("TranslatedTo")
                    ? Translations.Where(x => options["TranslatedTo"].Contains(x.Language.Code))
                    : Translations)
                    .Map(x => x.Disclose(level, options));
            }
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class PublicationTagMapping : EntityMapping<PublicationTagEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public PublicationTagMapping()
        {
            Table("fwk_PublicationTags");
            Id(x => x.EntityID, m =>
            {
                m.Column("TagID");
                m.Generator(Generators.Identity);
            });
            Property(x => x.Weight, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
            Bag(x => x.Publications, m =>
            {
                m.Table("fwk_MPublicationTags");
                m.Key(k =>
                {
                    k.Column("TagID");
                    k.ForeignKey("FK_fwk_MPublicationTags_fwk_PublicationTags");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.ManyToMany(m =>
                {
                    m.Column("PublicationID");
                    m.ForeignKey("FK_fwk_MPublicationTags_fwk_Publications");
                });
            });
            Bag(x => x.Translations, m =>
            {
                m.Table("fwk_TPublicationTags");
                m.Key(k =>
                {
                    k.Column("TagID");
                    k.ForeignKey("FK_fwk_TPublicationTags_fwk_PublicationTags");
                    k.NotNullable(true);
                });
                m.Inverse(true);
                m.BatchSize(10);
            },
            a =>
            {
                a.OneToMany();
            });
        }
    }
}
