﻿using System;
using System.Collections.Generic;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the publication translation entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  | - Text
    ///  + Extended
    ///   | - Description
    ///   + Base
    ///    | - Holder(of the Primary level)
    ///    | - Language(of the Keys level)
    ///    | - Title
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class TPublicationEntity : TranslationEntity<PublicationEntity>
    {
        #region Properties

        /// <summary>
        /// Gets or sets the title.
        /// </summary>
        public virtual string Title
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the description.
        /// </summary>
        public virtual string Description
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the text.
        /// </summary>
        public virtual string Text
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
                obj["Title"] = Title;
            if(level >= DisclosureLevel.Extended)
                obj["Description"] = Description;
            if(level >= DisclosureLevel.Entire)
                obj["Text"] = Text;
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class TPublicationMapping : EntityMapping<TPublicationEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public TPublicationMapping()
        {
            Table("fwk_TPublications");
            Id(x => x.EntityID, m =>
            {
                m.Column("TranslationID");
                m.Generator(Generators.Identity);
            });
            ManyToOne(x => x.Holder, m =>
            {
                m.Column("PublicationID");
                m.ForeignKey("FK_fwk_TPublications_fwk_Publications");
                m.Index("IX_fwk_TPublications");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            ManyToOne(x => x.Language, m =>
            {
                m.Column("LanguageID");
                m.ForeignKey("FK_fwk_TPublications_fwk_Languages");
                m.Index("IX_fwk_TPublications");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            Property(x => x.Title, m =>
            {
                m.NotNullable(true);
                m.Length(100);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(100)");
                });
            });
            Property(x => x.Description, m =>
            {
                m.NotNullable(true);
                m.Length(1000);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(1000)");
                });
            });
            Property(x => x.Text, m =>
            {
                m.NotNullable(true);
                m.Length(Int32.MaxValue);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(max)");
                });
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
        }
    }
}
