﻿using System;
using System.Collections.Generic;
using System.Linq;
using NHibernate.Linq;
using Xenta.Api;
using Xenta.Entities;
using Xenta.Enums;
using Xenta.Security;
using Xenta.Structs;
using Xenta.Utils;

namespace Xenta.Operations
{
    #region __General__

    /// <summary>
    /// Creates a publication author.
    /// </summary>
    public class CreatePublicationAuthor : CreateAccountProfile<PublicationAuthorEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("PBLSH");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .Rule((ctx, uow) => ctx.Account(uow).EntityID == AccountID);
            }
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public PublicationAuthorFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Construct(IUnitOfWork uow, PublicationAuthorEntity entity)
        {
            base.Construct(uow, entity);

            entity.Flags = Flags;
        }

        protected override void Work(IUnitOfWork uow, out PublicationAuthorEntity result)
        {
            base.Work(uow, out result);
            try
            {
                uow.Execute(new EnqueuePublicationAuthorNotification
                {
                    AuthorID = result.EntityID,
                    RecipientID = result.Account.EntityID
                });
            }
            catch(Exception ex)
            {
                uow.Execute(new CreateLogRecordFromException
                {
                    Ex = ex
                });
            }
        }

        #endregion
    }

    /// <summary>
    /// Updates the publication author.
    /// </summary>
    public class UpdatePublicationAuthor : UpdateAccountProfile<PublicationAuthorEntity>,
         ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("PBLSH");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .Rule((ctx, uow) =>
                    {
                        var author = ctx.PublicationAuthor(uow);
                        return author != null && author.EntityID == EntityID;
                    });
            }
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public PublicationAuthorFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Revise(IUnitOfWork uow, PublicationAuthorEntity entity)
        {
            base.Revise(uow, entity);

            entity.Flags = Flags;
        }

        #endregion
    }

    /// <summary>
    /// Updates the publication author statistics.
    /// </summary>
    public class UpdatePublicationAuthorStats : OperateEntityByID<PublicationAuthorEntity>
    {
        #region Utilities

        protected override void Work(IUnitOfWork uow, out PublicationAuthorEntity result)
        {
            var author = uow.Execute(new GetPublicationAuthor
            {
                ThrowsException = true,
                EntityID = EntityID
            }).Result;

            uow.Flush();
            uow.SqlQuery("exec fwk_PublicationAuthors_UpdateStats @AuthorID=:AuthorID, @UpdatedOn=:UpdatedOn")
                .SetInt32("AuthorID", author.EntityID)
                .SetDateTime("UpdatedOn", DateTime.UtcNow)
                .ExecuteUpdate();
            uow.Flush();
            
            result = uow.Execute(new RefreshEntity<PublicationAuthorEntity>
            {
                Entity = author
            }).Result;
        }

        #endregion
    }

    /// <summary>
    /// Deletes the publication author.
    /// </summary>
    public class DeletePublicationAuthor : DeleteAccountProfile<PublicationAuthorEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("PBLSH");
            }
        }

        #endregion
    }

    /// <summary>
    /// Gets the publication author by identifier.
    /// </summary>
    public class GetPublicationAuthor : GetAccountProfile<PublicationAuthorEntity>,
        IApiOperation
    {
    }

    /// <summary>
    /// Gets the publication author by identifier.
    /// </summary>
    public class GetPublicationAuthorByAccount : GetAccountProfileByAccount<PublicationAuthorEntity>,
        IApiOperation
    {
        #region Options

        protected override bool Cacheable
        {
            get
            {
                return true;
            }
        }

        #endregion
    }

    #endregion

    #region __Search__

    /// <summary>
    /// Search over publication author collection.
    /// </summary>
    public class SearchPublicationAuthors : SearchOperation<PublicationAuthorEntity>,
        IApiOperation
    {
        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public SearchPublicationAuthors()
        {
            Term = SearchTerm.Empty;
            SortDir = SortDirection.Ascending;
            SortBy = new []
            {
                PublicationAuthorSortBy.ID
            };
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the search term.
        /// </summary>
        public SearchTerm Term
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flag mask.
        /// </summary>
        public PublicationAuthorFlags FlagMask
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sort direction.
        /// </summary>
        public SortDirection SortDir
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sort by.
        /// </summary>
        public PublicationAuthorSortBy[] SortBy
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override IQueryable<PublicationAuthorEntity> Build(IUnitOfWork uow,
            IQueryable<PublicationAuthorEntity> query)
        {
            query = base.Build(uow, query);
            query = query.Fetch(x => x.Account);
            query = query.FilterByFlags(x => (int)x.Flags, (int)FlagMask);
            switch(Term.Mode)
            {
                case SearchTerm.SearchMode.StartsWith:
                    query = query.Where(x => x.Account.Email.StartsWith(Term.Keyword) ||
                        x.Account.Username.StartsWith(Term.Keyword));
                    break;
                case SearchTerm.SearchMode.EndsWith:
                    query = query.Where(x => x.Account.Email.EndsWith(Term.Keyword) ||
                        x.Account.Username.EndsWith(Term.Keyword));
                    break;
                case SearchTerm.SearchMode.Contains:
                    query = query.Where(x => x.Account.Email.Contains(Term.Keyword) ||
                        x.Account.Username.Contains(Term.Keyword));
                    break;
                case SearchTerm.SearchMode.Equals:
                    query = query.Where(x => x.Account.Email.Equals(Term.Keyword) ||
                        x.Account.Username.Equals(Term.Keyword));
                    break;
            }
            if(SortBy != null)
            {
                for(int i = 0; i < SortBy.Length; i++)
                {
                    switch(SortBy[i])
                    {
                        case PublicationAuthorSortBy.AccountEmail:
                            query = query.OrderBy(x => x.Account.Email, SortDir, i != 0);
                            break;
                        case PublicationAuthorSortBy.AccountUsername:
                            query = query.OrderBy(x => x.Account.Username, SortDir, i != 0);
                            break;
                        case PublicationAuthorSortBy.RatingAverage:
                            query = query.OrderBy(x => x.RatingAverage, SortDir, i != 0);
                            break;
                        case PublicationAuthorSortBy.PublicationCount:
                            query = query.OrderBy(x => x.PublicationCount, SortDir, i != 0);
                            break;
                        case PublicationAuthorSortBy.CreatedOn:
                            query = query.OrderBy(x => x.CreatedOn, SortDir, i != 0);
                            break;
                        default:
                            query = query.OrderBy(x => x.EntityID, SortDir, i != 0);
                            break;
                    }
                }
            }
            return query;
        }

        #endregion
    }

    /// <summary>
    /// Gets the collection of authors as a dictionary, where 
    /// key - author identifier and value - account email.
    /// </summary>
    public class GetPublicationAuthorDictionary : SearchOperation<PublicationAuthorEntity, Int32, String>,
        IApiOperation
    {
        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public GetPublicationAuthorDictionary()
        {
            Term = SearchTerm.Empty;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the search term.
        /// </summary>
        public SearchTerm Term
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flag mask.
        /// </summary>
        public PublicationAuthorFlags FlagMask
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override IQueryable<PublicationAuthorEntity> Build(IUnitOfWork uow, IQueryable<PublicationAuthorEntity> query)
        {
            query = base.Build(uow, query);
            query = query.Fetch(x => x.Account);
            query = query.FilterByFlags(x => (int)x.Flags, (int)FlagMask);
            switch(Term.Mode)
            {
                case SearchTerm.SearchMode.StartsWith:
                    query = query.Where(x =>
                        x.Account.Email.StartsWith(Term.Keyword) ||
                        x.Account.Username.StartsWith(Term.Keyword));
                    break;
                case SearchTerm.SearchMode.EndsWith:
                    query = query.Where(x =>
                        x.Account.Email.EndsWith(Term.Keyword) ||
                        x.Account.Username.EndsWith(Term.Keyword));
                    break;
                case SearchTerm.SearchMode.Contains:
                    query = query.Where(x =>
                        x.Account.Email.Contains(Term.Keyword) ||
                        x.Account.Username.Contains(Term.Keyword));
                    break;
                case SearchTerm.SearchMode.Equals:
                    query = query.Where(x =>
                        x.Account.Email.Equals(Term.Keyword) ||
                        x.Account.Username.Equals(Term.Keyword));
                    break;
            }
            query = query.OrderBy(x => x.Account.Email);
            return query;
        }

        protected override IQueryable<KeyValuePair<Int32, String>> Map(IQueryable<PublicationAuthorEntity> query)
        {
            return query.Select(x => new KeyValuePair<Int32, String>(x.EntityID, x.Account.Email));
        }

        #endregion
    }

    #endregion

    #region __Notification__

    /// <summary>
    /// Enqueues a publication author notification message.
    /// </summary>
    /// <remarks>
    /// Message template code: IPBLATHR
    /// Allowed tokens: [T:Recipient.EntityID], [T:Recipient.Guid], [T:Recipient.Email], [T:Recipient.Username],
    /// [T:PublicationAuthor.Account.EntityID], [T:PublicationAuthor.Account.Guid], [T:PublicationAuthor.Account.Email],
    /// [T:PublicationAuthor.Account.Username], [T:PublicationAuthor.EntityID], [T:PublicationAuthor.PublicationCount],
    /// [T:PublicationAuthor.RatingAverage]
    /// </remarks>
    public class EnqueuePublicationAuthorNotification : Operation<MessageEntity>
    {
        #region Properties

        /// <summary>
        /// Gets or sets the author identifier.
        /// </summary>
        public int AuthorID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the recipient account identifier.
        /// </summary>
        public int RecipientID
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow, out MessageEntity result)
        {
            var author = uow.Execute(new GetPublicationAuthor
            {
                ThrowsException = true,
                EntityID = AuthorID
            }).Result;
            var recipient = author.Account.EntityID == RecipientID ? author.Account : uow.Execute(new GetAccount
            {
                ThrowsException = true,
                EntityID = RecipientID
            }).Result;
            var usrLocale = recipient.LocalizationProfile;

            result = uow.Execute(new CreateTemplatedMessageByCode
            {
                TemplateCode = "IPBLATHR",
                LanguageID = usrLocale.Language.EntityID,
                Replacements = new Dictionary<String, String>
                {
                    {"Recipient.EntityID", recipient.EntityID.ToString()},
                    {"Recipient.Email", recipient.Email},
                    {"Recipient.Username", recipient.Username},
                    {"Recipient.Guid", recipient.Guid.ToString()},
                    {"PublicationAuthor.Account.EntityID", author.Account.EntityID.ToString()},
                    {"PublicationAuthor.Account.Email", author.Account.Email},
                    {"PublicationAuthor.Account.Username", author.Account.Username},
                    {"PublicationAuthor.Account.Guid", author.Account.Guid.ToString()},
                    {"PublicationAuthor.EntityID", author.EntityID.ToString()},
                    {"PublicationAuthor.PublicationCount", author.PublicationCount.ToString()},
                    {"PublicationAuthor.RatingAverage", author.RatingAverage.ToString()}
                },
                To = recipient.Email,
                Flags = MessageFlags.Active
            }).Result;
        }

        #endregion
    }

    #endregion
}
