﻿using System;
using System.Collections.Generic;
using System.Linq;
using Xenta.Api;
using Xenta.Entities;
using Xenta.Enums;
using Xenta.Security;
using Xenta.Utils;

namespace Xenta.Operations
{
    #region __General__

    /// <summary>
    /// Creates a publication comment.
    /// </summary>
    public class CreatePublicationComment : CreateComment<PublicationCommentEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("PBLSH");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) => ctx.Account(uow).EntityID == AccountID);
                yield return new SecurityPolicy()
                    .Rule((ctx, uow) => AccountID == 0);
            }
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public PublicationCommentFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override Err Validate(IUnitOfWork uow)
        {
            var publication = uow.Execute(new GetPublication
            {
                ThrowsException = true,
                EntityID = HolderID
            }).Result;
            if(!publication.Flags.HasFlag(PublicationFlags.CommentAllowed))
                return Err.Of(this).NotAllowed();
            if(!uow.Execute(new GetConfigParamValueByKey
            {
                Key = Par.In("Publishing").Of<PublicationCommentEntity>().Name("Allowed")
            }).ResultAs<Boolean>())
            {
                return Err.Of(this).NotAllowed();
            }

            var account = AccountID != 0 
                ? uow.Execute(new GetAccount
                {
                    ThrowsException = true,
                    EntityID = AccountID
                }).Result 
                : null;
            if(account == null)
            {
                if(!publication.Flags.HasFlag(PublicationFlags.AnonCommentAllowed))
                    return Err.Of(this).NotAllowed().Reason("Anon");
                if(!uow.Execute(new GetConfigParamValueByKey
                {
                    Key = Par.In("Publishing").Of<PublicationCommentEntity>().Name("AnonAllowed")
                }).ResultAs<Boolean>())
                {
                    return Err.Of(this).NotAllowed().Reason("Anon");
                }
            }

            return base.Validate(uow);
        }

        protected override void Construct(IUnitOfWork uow, PublicationCommentEntity entity)
        {
            base.Construct(uow, entity);

            entity.Flags = Flags;
        }

        protected override void Work(IUnitOfWork uow, out PublicationCommentEntity result)
        {
            base.Work(uow, out result);

            uow.Execute(new UpdatePublicationStats
            {
                EntityID = result.HolderID
            });

            try
            {
                uow.Execute(new EnqueuePublicationCommentNotification
                {
                    CommentID = result.EntityID,
                    RecipientID = uow.Execute(new GetPublication
                    {
                        ThrowsException = true,
                        EntityID = result.HolderID
                    }).Result.Author.Account.EntityID
                });
            }
            catch(Exception ex)
            {
                uow.Execute(new CreateLogRecordFromException
                {
                    Ex = ex
                });
            }
        }

        #endregion
    }

    /// <summary>
    /// Updates the publication comment.
    /// </summary>
    public class UpdatePublicationComment : UpdateComment<PublicationCommentEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("PBLSH");
            }
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public PublicationCommentFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow, out PublicationCommentEntity result)
        {
            base.Work(uow, out result);
            uow.Execute(new UpdatePublicationStats
            {
                EntityID = result.HolderID
            });
        }

        protected override void Revise(IUnitOfWork uow, PublicationCommentEntity entity)
        {
            base.Revise(uow, entity);

            entity.Flags = Flags;
        }

        #endregion
    }

    /// <summary>
    /// Deletes the publication comment.
    /// </summary>
    public class DeletePublicationComment : DeleteComment<PublicationCommentEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("PBLSH");
            }
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow)
        {
            int holderID = uow.Execute(new GetPublicationComment
            {
                ThrowsException = true,
                EntityID = EntityID
            }).Result.HolderID;

            base.Work(uow);

            uow.Execute(new UpdatePublicationStats
            {
                EntityID = holderID
            });
        }

        #endregion
    }

    /// <summary>
    /// Gets the publication comment by identifier.
    /// </summary>
    public class GetPublicationComment : GetComment<PublicationCommentEntity>,
        IApiOperation
    {
    }

    #endregion

    #region __Search__

    /// <summary>
    /// Search over publication comment collection.
    /// </summary>
    public class SearchPublicationComments : SearchComments<PublicationCommentEntity>,
        IApiOperation
    {
        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public SearchPublicationComments()
        {
            SortDir = SortDirection.Ascending;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the flag mask.
        /// </summary>
        public PublicationCommentFlags FlagMask
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sort direction.
        /// </summary>
        public SortDirection SortDir
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override IQueryable<PublicationCommentEntity> Build(IUnitOfWork uow, IQueryable<PublicationCommentEntity> query)
        {
            query = base.Build(uow, query);
            query = query.FilterByFlags(x => x.FlagsValue, (int)FlagMask);
            query = query.OrderBy(x => x.CreatedOn, SortDir);
            return query;
        }

        #endregion
    }

    #endregion

    #region __Notification___

    /// <summary>
    /// Enqueues a publication author notification.
    /// </summary>
    /// <remarks>
    /// Message template code: NPBLCMNT
    /// Allowed tokens: [T:Recipient.EntityID], [T:Recipient.Guid], [T:Recipient.Email], [T:Recipient.Username],
    /// [T:PublicationComment.Publication.EntityID], [T:PublicationComment.Publication.RatingAverage],
    /// [T:PublicationComment.Publication.RatingCount], [T:PublicationComment.Publication.CommentCount],
    /// [T:PublicationComment.Publication.Title], [T:PublicationComment.Publication.Description],
    /// [T:PublicationComment.Account.EntityID], [T:PublicationComment.Account.Email],
    /// [T:PublicationComment.Account.Username], [T:PublicationComment.Account.Guid],
    /// [T:PublicationComment.EntityID], [T:PublicationComment.Title], [T:PublicationComment.Text],
    /// [T:PublicationComment.CreatedOn], [T:PublicationComment.Publication.Code]
    /// </remarks>
    public class EnqueuePublicationCommentNotification : Operation<MessageEntity>
    {
        #region Properties

        /// <summary>
        /// Gets or sets the comment identifier.
        /// </summary>
        public int CommentID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the recipient account identifier.
        /// </summary>
        public int RecipientID
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow, out MessageEntity result)
        {
            var comment = uow.Execute(new GetPublicationComment
            {
                ThrowsException = true,
                EntityID = CommentID
            }).Result;
            var recipient = uow.Execute(new GetAccount
            {
                ThrowsException = true,
                EntityID = RecipientID
            }).Result;
            var usrLocale = recipient.LocalizationProfile;
            int utcOffset = usrLocale.TimeZone.UtcOffset;
            string dtFmt = usrLocale.Country.DateTimeFormat;
            var publication = uow.Execute(new GetPublication
            {
                ThrowsException = true,
                EntityID = comment.HolderID
            }).Result;
            var t_publication = publication.Translation(usrLocale.Language.EntityID);

            result = uow.Execute(new CreateTemplatedMessageByCode
            {
                TemplateCode = "NPBLCMNT",
                LanguageID = usrLocale.Language.EntityID,
                Replacements = new Dictionary<String, String>
                {
                    {"Recipient.EntityID", recipient.EntityID.ToString()},
                    {"Recipient.Email", recipient.Email},
                    {"Recipient.Username", recipient.Username},
                    {"Recipient.Guid", recipient.Guid.ToString()},
                    {"PublicationComment.Publication.EntityID", publication.EntityID.ToString()},
                    {"PublicationComment.Publication.RatingAverage", publication.RatingAverage.ToString()},
                    {"PublicationComment.Publication.RatingCount", publication.RatingCount.ToString()},
                    {"PublicationComment.Publication.CommentCount", publication.CommentCount.ToString()},
                    {"PublicationComment.Publication.Code", publication.Code},
                    {"PublicationComment.Publication.Title", t_publication != null 
                        ? t_publication.Title 
                        : String.Empty},
                    {"PublicationComment.Publication.Description", t_publication != null 
                        ? t_publication.Description 
                        : String.Empty},
                    {"PublicationComment.Account.EntityID", comment.Account != null 
                        ? comment.Account.EntityID.ToString() 
                        : String.Empty},
                    {"PublicationComment.Account.Email", comment.Account != null 
                        ? comment.Account.Email 
                        : String.Empty},
                    {"PublicationComment.Account.Username", comment.Account != null 
                        ? comment.Account.Username 
                        : String.Empty},
                    {"PublicationComment.Account.Guid", comment.Account != null 
                        ? comment.Account.Guid.ToString() 
                        : String.Empty},
                    {"PublicationComment.EntityID", comment.EntityID.ToString()},
                    {"PublicationComment.Title", comment.Title},
                    {"PublicationComment.Text", comment.Text},
                    {"PublicationComment.CreatedOn", comment.CreatedOn.ToLocal(utcOffset).ToStringByFmt(dtFmt)}
                },
                To = recipient.Email,
                Flags = MessageFlags.Active
            }).Result;
        }

        #endregion
    }

    #endregion
}
