﻿using System;
using System.Collections.Generic;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;
using Xenta.Utils;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the customer entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  | - Addresses(of the Extended level)
    ///  + Extended
    ///   + Base
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     | - Account(of the same level)
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class CustomerEntity : AccountProfileEntity
    {
        #region Fields

        private ICollection<CustomerAddressEntity> _addresses;
        private ICollection<OrderEntity> _orders;
        private ICollection<PaymentEntity> _payments;

        #endregion

        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public CustomerEntity()
        {
            _addresses = new List<CustomerAddressEntity>();
            _orders = new List<OrderEntity>();
            _payments = new List<PaymentEntity>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual CustomerFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Collections

        /// <summary>
        /// Gets or sets the order collection.
        /// </summary>
        public virtual ICollection<OrderEntity> Orders
        {
            get
            {
                return _orders;
            }
            set
            {
                _orders = value;
            }
        }

        /// <summary>
        /// Gets or sets the payment collection.
        /// </summary>
        public virtual ICollection<PaymentEntity> Payments
        {
            get
            {
                return _payments;
            }
            set
            {
                _payments = value;
            }
        }

        /// <summary>
        /// Gets or sets the address collection.
        /// </summary>
        public virtual ICollection<CustomerAddressEntity> Addresses
        {
            get
            {
                return _addresses;
            }
            set
            {
                _addresses = value;
            }
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
                obj["Flags"] = Flags.ToString();
            if(level >= DisclosureLevel.Entire)
                obj["Addresses"] = Addresses.Map(x => x.Disclose(DisclosureLevel.Extended, options));
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class CustomerMapping : EntityMapping<CustomerEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public CustomerMapping()
        {
            Table("fwk_Customers");
            Id(x => x.EntityID, m =>
            {
                m.Column("CustomerID");
                m.Generator(Generators.Identity);
            });
            ManyToOne(x => x.Account, m =>
            {
                m.Column("AccountID");
                m.ForeignKey("FK_fwk_Customers_fwk_Accounts");
                m.Index("IX_fwk_Customers");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
            Bag(x => x.Orders, m =>
            {
                m.Table("fwk_Orders");
                m.Key(k =>
                {
                    k.Column("CustomerID");
                    k.ForeignKey("FK_fwk_Orders_fwk_Customers");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.OneToMany();
            });
            Bag(x => x.Payments, m =>
            {
                m.Table("fwk_Payments");
                m.Key(k =>
                {
                    k.Column("CustomerID");
                    k.ForeignKey("FK_fwk_Payments_fwk_Customers");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.OneToMany();
            });
            Bag(x => x.Addresses, m =>
            {
                m.Table("fwk_Addresses");
                m.Key(k =>
                {
                    k.Column("HolderID");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.OneToMany();
            });
            Cache(m =>
            {
                m.Usage(CacheUsage.ReadWrite);
            });
        }
    }
}
