﻿using System;
using System.Collections.Generic;
using System.Linq;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;
using Xenta.Utils;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the order entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  | - Items(of the Extended level)
    ///  + Extended
    ///   | - Attributes(of the Extended level)
    ///   + Base
    ///    | - Customer(of the Base level)
    ///    | - Seller(of the Base level)
    ///    | - BillingAddress(of the Base level)
    ///    | - ShippingAddress(of the Base level)
    ///    | - ShippingMethodID
    ///    | - SalesModelID
    ///    | - SellerAddress(of the Base level)
    ///    | - HandlingFee
    ///    | - TaxAmount
    ///    | - DiscountAmount
    ///    | - ShippingPrice
    ///    | - Subtotal
    ///    | - Total
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     | - Guid
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class OrderEntity : AuditableEntity
    {
        #region Fields

        private ICollection<OrderCommentEntity> _comments;
        private ICollection<OrderItemEntity> _items;
        private ICollection<InvoiceEntity> _invoices;
        private ICollection<OrderAttributeEntity> _attributes;
        private ICollection<SalesRuleHistoryEntity> _appliedRules;

        #endregion

        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public OrderEntity()
        {
            _comments = new List<OrderCommentEntity>();
            _items = new List<OrderItemEntity>();
            _invoices = new List<InvoiceEntity>();
            _attributes = new List<OrderAttributeEntity>();
            _appliedRules = new List<SalesRuleHistoryEntity>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or set the customer.
        /// </summary>
        public virtual CustomerEntity Customer
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or set the seller.
        /// </summary>
        public virtual SellerEntity Seller
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or set the billing address.
        /// </summary>
        public virtual CustomerAddressEntity BillingAddress
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or set the shipping address.
        /// </summary>
        public virtual CustomerAddressEntity ShippingAddress
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or set the seller address.
        /// </summary>
        public virtual SellerAddressEntity SellerAddress
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the order global unique identifier.
        /// </summary>
        public virtual Guid Guid
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the shipping method unique identifier.
        /// </summary>
        public virtual Guid ShippingMethodID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sales model unique identifier.
        /// </summary>
        public virtual Guid SalesModelID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the handling fee.
        /// </summary>
        public virtual decimal HandlingFee
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the tax amount.
        /// </summary>
        public virtual decimal TaxAmount
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the discount amount.
        /// </summary>
        public virtual decimal DiscountAmount
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the shipping price.
        /// </summary>
        public virtual decimal ShippingPrice
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the subtotal.
        /// </summary>
        public virtual decimal Subtotal
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the total.
        /// </summary>
        public virtual decimal Total
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual OrderFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Collections
        
        /// <summary>
        /// Gets or sets the item collection.
        /// </summary>
        public virtual ICollection<OrderItemEntity> Items
        {
            get
            {
                return _items;
            }
            set
            {
                _items = value;
            }
        }

        /// <summary>
        /// Gets or sets the comment collection.
        /// </summary>
        public virtual ICollection<OrderCommentEntity> Comments
        {
            get
            {
                return _comments;
            }
            set
            {
                _comments = value;
            }
        }

        /// <summary>
        /// Gets or sets the invoice collection.
        /// </summary>
        public virtual ICollection<InvoiceEntity> Invoices
        {
            get
            {
                return _invoices;
            }
            set
            {
                _invoices = value;
            }
        }

        /// <summary>
        /// Gets or sets the collection of attributes.
        /// </summary>
        public virtual ICollection<OrderAttributeEntity> Attributes
        {
            get
            {
                return _attributes;
            }
            set
            {
                _attributes = value;
            }
        }

        /// <summary>
        /// Gets or sets the collection of applied sales rules.
        /// </summary>
        public virtual ICollection<SalesRuleHistoryEntity> AppliedRules
        {
            get
            {
                return _appliedRules;
            }
            set
            {
                _appliedRules = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the attribute value.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns>The attribute value.</returns>
        public virtual string Attribute(string key)
        {
            if(String.IsNullOrWhiteSpace(key))
                return String.Empty;
            var attribute = _attributes.SingleOrDefault(x => x.Key.Equals(key));
            if(attribute == null)
                return String.Empty;
            return attribute.Value;
        }

        /// <summary>
        /// Gets the attribute value as the type T.
        /// </summary>
        /// <typeparam name="T">The type to return.</typeparam>
        /// <param name="key">The attribute key.</param>
        /// <returns>The attribute value as the type T.</returns>
        public virtual T Attribute<T>(string key)
        {
            return StringHelper.ParseAs<T>(Attribute(key));
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Keys)
                obj["Guid"] = Guid;
            if(level >= DisclosureLevel.Base)
            {
                obj["Customer"] = Customer.Disclose(DisclosureLevel.Base, options);
                obj["Seller"] = Seller.Disclose(DisclosureLevel.Base, options);
                obj["BillingAddress"] = BillingAddress.Disclose(DisclosureLevel.Base, options);
                obj["ShippingAddress"] = ShippingAddress.Disclose(DisclosureLevel.Base, options);
                obj["ShippingMethodID"] = ShippingMethodID;
                obj["SalesModelID"] = SalesModelID;
                obj["SellerAddress"] = SellerAddress.Disclose(DisclosureLevel.Base, options);
                obj["HandlingFee"] = HandlingFee;
                obj["TaxAmount"] = TaxAmount;
                obj["DiscountAmount"] = DiscountAmount;
                obj["ShippingPrice"] = ShippingPrice;
                obj["Subtotal"] = Subtotal;
                obj["Total"] = Total;
                obj["Flags"] = Flags.ToString();
            }
            if(level >= DisclosureLevel.Extended)
                obj["Attributes"] = Attributes.Map(x => x.Disclose(DisclosureLevel.Extended, options));
            if(level >= DisclosureLevel.Entire)
            {
                obj["AppliedRules"] = AppliedRules.Map(x => x.Disclose(DisclosureLevel.Extended, options));
                obj["Items"] = Items.Map(x => x.Disclose(DisclosureLevel.Extended, options));
            }
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class OrderMapping : EntityMapping<OrderEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public OrderMapping()
        {
            Table("fwk_Orders");
            Id(x => x.EntityID, m =>
            {
                m.Column("OrderID");
                m.Generator(Generators.Identity);
            });
            ManyToOne(x => x.Customer, m =>
            {
                m.Column("CustomerID");
                m.ForeignKey("FK_fwk_Orders_fwk_Customers");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            ManyToOne(x => x.Seller, m =>
            {
                m.Column("SellerID");
                m.ForeignKey("FK_fwk_Orders_fwk_Sellers");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            ManyToOne(x => x.BillingAddress, m =>
            {
                m.Column("BillingAddressID");
                m.ForeignKey("FK_fwk_Orders_fwk_Addresses_Billing");
                m.NotNullable(true);
            });
            ManyToOne(x => x.ShippingAddress, m =>
            {
                m.Column("ShippingAddressID");
                m.ForeignKey("FK_fwk_Orders_fwk_Addresses_Shipping");
                m.NotNullable(true);
            });
            ManyToOne(x => x.SellerAddress, m =>
            {
                m.Column("SellerAddressID");
                m.ForeignKey("FK_fwk_Orders_fwk_Addresses_Seller");
                m.NotNullable(true);
            });
            Property(x => x.ShippingMethodID, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.SalesModelID, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.Guid, m =>
            {
                m.Unique(true);
                m.Index("IX_fwk_Orders");
                m.NotNullable(true);
            });
            Property(x => x.HandlingFee, m =>
            {
                m.Precision(19);
                m.Scale(4);
                m.NotNullable(true);
            });
            Property(x => x.TaxAmount, m =>
            {
                m.Precision(19);
                m.Scale(4);
                m.NotNullable(true);
            });
            Property(x => x.DiscountAmount, m =>
            {
                m.Precision(19);
                m.Scale(4);
                m.NotNullable(true);
            });
            Property(x => x.ShippingPrice, m =>
            {
                m.Precision(19);
                m.Scale(4);
                m.NotNullable(true);
            });
            Property(x => x.Subtotal, m =>
            {
                m.Precision(19);
                m.Scale(4);
                m.NotNullable(true);
            });
            Property(x => x.Total, m =>
            {
                m.Precision(19);
                m.Scale(4);
                m.NotNullable(true);
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
            Bag(x => x.Items, m =>
            {
                m.Table("fwk_OrderItems");
                m.Key(k =>
                {
                    k.Column("OrderID");
                    k.ForeignKey("FK_fwk_OrderItems_fwk_Orders");
                    k.NotNullable(true);
                });
                m.Inverse(true);
                m.BatchSize(10);
            },
            a =>
            {
                a.OneToMany();
            });
            Bag(x => x.Comments, m =>
            {
                m.Table("fwk_Comments");
                m.Key(k =>
                {
                    k.Column("HolderID");
                    k.NotNullable(true);
                });
                m.Inverse(true);
                m.BatchSize(10);
            },
            a =>
            {
                a.OneToMany();
            });
            Bag(x => x.Invoices, m =>
            {
                m.Table("fwk_Invoices");
                m.Key(k =>
                {
                    k.Column("OrderID");
                    k.ForeignKey("FK_fwk_Invoices_fwk_Orders");
                    k.NotNullable(true);
                });
                m.Inverse(true);
                m.BatchSize(10);
            },
            a =>
            {
                a.OneToMany();
            });
            Bag(x => x.Attributes, m =>
            {
                m.Table("fwk_GenericAttributes");
                m.Key(k =>
                {
                    k.Column("HolderID");
                    k.NotNullable(true);
                });
                m.Inverse(true);
                m.BatchSize(10);
            },
            a =>
            {
                a.OneToMany();
            });
            Bag(x => x.AppliedRules, m =>
            {
                m.Table("fwk_SalesRuleHistory");
                m.Key(k =>
                {
                    k.Column("OrderID");
                    k.ForeignKey("FK_fwk_SalesRuleHistory_fwk_Orders");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.OneToMany();
            });
        }
    }
}
