﻿using System;
using System.Collections.Generic;
using System.Linq;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;
using Xenta.Utils;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the sales option entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   + Base
    ///    | - CodeName
    ///    | - CodeValue
    ///    | - Price
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    | - Translations(of the same level)
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class SalesOptionEntity : AuditableEntity
    {
        #region Fields

        private ICollection<TSalesOptionEntity> _translations;
        private ICollection<SalesItemEntity> _salesItems;
        private ICollection<ShoppingCartEntity> _shopingCarts;
        private ICollection<ShoppingCartItemEntity> _shopingCartItems;

        #endregion

        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public SalesOptionEntity()
        {
            _translations = new List<TSalesOptionEntity>();
            _salesItems = new List<SalesItemEntity>();
            _shopingCarts = new List<ShoppingCartEntity>();
            _shopingCartItems = new List<ShoppingCartItemEntity>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the code name.
        /// </summary>
        public virtual string CodeName
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the code value.
        /// </summary>
        public virtual string CodeValue
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the price.
        /// </summary>
        public virtual decimal Price
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual SalesOptionFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Collections

        /// <summary>
        /// Gets or sets the attribute translations.
        /// </summary>
        public virtual ICollection<TSalesOptionEntity> Translations
        {
            get
            {
                return _translations;
            }
            set
            {
                _translations = value;
            }
        }

        /// <summary>
        /// Gets or sets the sales item collection.
        /// </summary>
        public virtual ICollection<SalesItemEntity> SalesItems
        {
            get
            {
                return _salesItems;
            }
            set
            {
                _salesItems = value;
            }
        }

        /// <summary>
        /// Gets or sets the shopping cart collection.
        /// </summary>
        public virtual ICollection<ShoppingCartEntity> ShoppingCarts
        {
            get
            {
                return _shopingCarts;
            }
            set
            {
                _shopingCarts = value;
            }
        }

        /// <summary>
        /// Gets or sets the shopping cart item collection.
        /// </summary>
        public virtual ICollection<ShoppingCartItemEntity> ShoppingCartItems
        {
            get
            {
                return _shopingCartItems;
            }
            set
            {
                _shopingCartItems = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the attribute translation.
        /// </summary>
        /// <param name="languageID">The language identifier.</param>
        /// <returns>The attribute translation.</returns>
        public virtual TSalesOptionEntity Translation(int languageID)
        {
            return _translations.SingleOrDefault(x => x.Language.EntityID == languageID);
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
            {
                obj["CodeName"] = CodeName;
                obj["CodeValue"] = CodeValue;
                obj["Price"] = Price;
                obj["Flags"] = Flags.ToString();
                obj["Translations"] = (options.ContainsKey("TranslatedTo")
                    ? Translations.Where(x => options["TranslatedTo"].Contains(x.Language.Code))
                    : Translations)
                    .Map(x => x.Disclose(level, options));
            }
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class SalesOptionMapping : EntityMapping<SalesOptionEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public SalesOptionMapping()
        {
            Table("fwk_SalesOptions");
            Id(x => x.EntityID, m =>
            {
                m.Column("OptionID");
                m.Generator(Generators.Identity);
            });
            Property(x => x.CodeName, m =>
            {
                m.NotNullable(true);
                m.Length(10);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(10)");
                });
            });
            Property(x => x.CodeValue, m =>
            {
                m.NotNullable(true);
                m.Length(10);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(10)");
                });
            });
            Property(x => x.Price, m =>
            {
                m.Precision(19);
                m.Scale(4);
                m.NotNullable(true);
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
            Bag(x => x.Translations, m =>
            {
                m.Table("fwk_TSalesOptions");
                m.Key(k =>
                {
                    k.Column("OptionID");
                    k.ForeignKey("FK_fwk_TSalesOptions_fwk_SalesOptions");
                    k.NotNullable(true);
                });
                m.Inverse(true);
                m.BatchSize(10);
            },
            a =>
            {
                a.OneToMany();
            });
            Bag(x => x.SalesItems, m =>
            {
                m.Table("fwk_MSalesItemOptions");
                m.Key(k =>
                {
                    k.Column("OptionID");
                    k.ForeignKey("FK_fwk_MSalesItemOptions_fwk_SalesOptions");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.ManyToMany(m =>
                {
                    m.Column("ItemID");
                    m.ForeignKey("FK_fwk_MSalesItemOptions_fwk_SalesItems");
                });
            });
            Bag(x => x.ShoppingCarts, m =>
            {
                m.Table("fwk_MShoppingCartOptions");
                m.Key(k =>
                {
                    k.Column("OptionID");
                    k.ForeignKey("FK_fwk_MShoppingCartOptions_fwk_SalesOptions");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.ManyToMany(m =>
                {
                    m.Column("CartID");
                    m.ForeignKey("FK_fwk_MShoppingCartOptions_fwk_ShoppingCarts");
                });
            });
            Bag(x => x.ShoppingCartItems, m =>
            {
                m.Table("fwk_MShoppingCartItemOptions");
                m.Key(k =>
                {
                    k.Column("OptionID");
                    k.ForeignKey("FK_fwk_MShoppingCartItemOptions_fwk_SalesOptions");
                    k.NotNullable(true);
                });
                m.Inverse(true);
            },
            a =>
            {
                a.ManyToMany(m =>
                {
                    m.Column("ItemID");
                    m.ForeignKey("FK_fwk_MShoppingCartItemOptions_fwk_ShoppingCartItems");
                });
            });
        }
    }
}
