﻿using System;
using System.Collections.Generic;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;
using Xenta.Utils;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the shopping cart item entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   | - Options(of the Extended level)
    ///   + Base
    ///    | - Cart(of the Primary level)
    ///    | - SalesItem(of the same level)
    ///    | - Quantity
    ///    | - Price
    ///    | - Total
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class ShoppingCartItemEntity : AuditableEntity,
        IRelationHolder<SalesOptionEntity>
    {
        #region Fields

        private ICollection<SalesOptionEntity> _options;

        #endregion

        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public ShoppingCartItemEntity()
        {
            _options = new List<SalesOptionEntity>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the cart.
        /// </summary>
        public virtual ShoppingCartEntity Cart
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sales item.
        /// </summary>
        public virtual SalesItemEntity SalesItem
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the quantity.
        /// </summary>
        public virtual decimal Quantity
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the price.
        /// </summary>
        public virtual decimal Price
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the total.
        /// </summary>
        public virtual decimal Total
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual ShoppingCartItemFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Collections

        /// <summary>
        /// Gets or sets the option collection.
        /// </summary>
        public virtual ICollection<SalesOptionEntity> Options
        {
            get
            {
                return _options;
            }
            set
            {
                _options = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Maps an option to the item.
        /// </summary>
        /// <param name="child">The option to map.</param>
        public virtual void Map(SalesOptionEntity child)
        {
            if(Options.Contains(child))
                return;
            Options.Add(child);
        }

        /// <summary>
        /// Unmaps the option from the item.
        /// </summary>
        /// <param name="child">The option to unmap.</param>
        public virtual void Unmap(SalesOptionEntity child)
        {
            if(!Options.Contains(child))
                return;
            Options.Remove(child);
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
            {
                obj["Cart"] = Cart.Disclose(DisclosureLevel.Primary, options);
                obj["SalesItem"] = SalesItem.Disclose(level, options);
                obj["Quantity"] = Quantity;
                obj["Price"] = Price;
                obj["Total"] = Total;
                obj["Flags"] = Flags.ToString();
            }
            if(level >= DisclosureLevel.Extended)
                obj["Options"] = Options.Map(x => x.Disclose(DisclosureLevel.Extended, options));
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class ShoppingCartItemMapping : EntityMapping<ShoppingCartItemEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public ShoppingCartItemMapping()
        {
            Table("fwk_ShoppingCartItems");
            Id(x => x.EntityID, m =>
            {
                m.Column("ItemID");
                m.Generator(Generators.Identity);
            });
            ManyToOne(x => x.Cart, m =>
            {
                m.Column("CartID");
                m.ForeignKey("FK_fwk_ShoppingCartItems_fwk_ShoppingCarts");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            ManyToOne(x => x.SalesItem, m =>
            {
                m.Column("SalesItemID");
                m.ForeignKey("FK_fwk_ShoppingCartItems_fwk_SalesItems");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            Property(x => x.Quantity, m =>
            {
                m.Precision(19);
                m.Scale(4);
                m.NotNullable(true);
            });
            Property(x => x.Price, m =>
            {
                m.Precision(19);
                m.Scale(4);
                m.NotNullable(true);
            });
            Property(x => x.Total, m =>
            {
                m.Precision(19);
                m.Scale(4);
                m.NotNullable(true);
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
            Bag(x => x.Options, m =>
            {
                m.Table("fwk_MShoppingCartItemOptions");
                m.Key(k =>
                {
                    k.Column("ItemID");
                    k.ForeignKey("FK_fwk_MShoppingCartItemOptions_fwk_ShoppingCartItems");
                    k.NotNullable(true);
                });
            },
            a =>
            {
                a.ManyToMany(m =>
                {
                    m.Column("OptionID");
                    m.ForeignKey("FK_fwk_MShoppingCartItemOptions_fwk_SalesOptions");
                });
            });
        }
    }
}
