﻿using System;
using System.Collections.Generic;
using System.Linq;
using Xenta.Api;
using Xenta.Entities;
using Xenta.Enums;
using Xenta.Security;
using Xenta.Utils;

namespace Xenta.Operations
{
    #region __General__

    /// <summary>
    /// Creates a seller rating.
    /// </summary>
    public class CreateSellerRating : CreateRating<SellerRatingEntity, SellerCommentEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("SALES");
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .IsVerified()
                    .Rule((ctx, uow) => ctx.Account(uow).EntityID == AccountID);
                yield return new SecurityPolicy()
                    .Rule((ctx, uow) => AccountID == 0);
            }
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public SellerRatingFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override Err Validate(IUnitOfWork uow)
        {
            var seller = uow.Execute(new GetSeller
            {
                ThrowsException = true,
                EntityID = HolderID
            }).Result;
            if(!seller.Flags.HasFlag(SellerFlags.RatingAllowed))
                return Err.Of(this).NotAllowed();
            if(!uow.Execute(new GetConfigParamValueByKey
            {
                Key = Par.In("Sales").Of<SellerRatingEntity>().Name("Allowed")
            }).ResultAs<Boolean>())
            {
                return Err.Of(this).NotAllowed();
            }

            var account = AccountID != 0 
                ? uow.Execute(new GetAccount
                {
                    ThrowsException = true,
                    EntityID = AccountID
                }).Result 
                : null;
            if(account == null)
            {
                if(!seller.Flags.HasFlag(SellerFlags.AnonRatingAllowed))
                    return Err.Of(this).NotAllowed().Reason("Anon");
                if(!uow.Execute(new GetConfigParamValueByKey
                {
                    Key = Par.In("Sales").Of<SellerRatingEntity>().Name("AnonAllowed")
                }).ResultAs<Boolean>())
                {
                    return Err.Of(this).NotAllowed().Reason("Anon");
                }
            }

            byte valueMin = uow.Execute(new GetConfigParamValueByKey
            {
                Key = Par.In("Sales").Of<SellerRatingEntity>().Name("Min")
            }).ResultAs<Byte>();
            if(Value < valueMin)
                return Err.Of<SellerRatingEntity>().NotValid("Value");
            byte valueMax = uow.Execute(new GetConfigParamValueByKey
            {
                Key = Par.In("Sales").Of<SellerRatingEntity>().Name("Max")
            }).ResultAs<Byte>();
            if(Value > valueMax)
                return Err.Of<SellerRatingEntity>().NotValid("Value");

            return base.Validate(uow);
        }

        protected override void Work(IUnitOfWork uow, out SellerRatingEntity result)
        {
            base.Work(uow, out result);
            uow.Execute(new UpdateSellerStats
            {
                EntityID = result.HolderID
            });

            try
            {
                uow.Execute(new EnqueueSellerRatingNotification
                {
                    RatingID = result.EntityID,
                    RecipientID = uow.Execute(new GetSeller
                    {
                        ThrowsException = true,
                        EntityID = result.HolderID
                    }).Result.Account.EntityID
                });
            }
            catch(Exception ex)
            {
                uow.Execute(new CreateLogRecordFromException
                {
                    Ex = ex
                });
            }
        }

        protected override void Construct(IUnitOfWork uow, SellerRatingEntity entity)
        {
            base.Construct(uow, entity);

            entity.Flags = Flags;
        }

        #endregion
    }

    /// <summary>
    /// Updates the seller rating.
    /// </summary>
    public class UpdateSellerRating : UpdateRating<SellerRatingEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("SALES");
            }
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public SellerRatingFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override Err Validate(IUnitOfWork uow)
        {
            byte valueMin = uow.Execute(new GetConfigParamValueByKey
            {
                Key = Par.In("Sales").Of<SellerRatingEntity>().Name("Min")
            }).ResultAs<Byte>();
            if(Value < valueMin)
                return Err.Of<SellerRatingEntity>().NotValid("Value");
            byte valueMax = uow.Execute(new GetConfigParamValueByKey
            {
                Key = Par.In("Sales").Of<SellerRatingEntity>().Name("Max")
            }).ResultAs<Byte>();
            if(Value > valueMax)
                return Err.Of<SellerRatingEntity>().NotValid("Value");
            return base.Validate(uow);
        }

        protected override void Work(IUnitOfWork uow, out SellerRatingEntity result)
        {
            base.Work(uow, out result);
            uow.Execute(new UpdateSellerStats
            {
                EntityID = result.HolderID
            });
        }

        protected override void Revise(IUnitOfWork uow, SellerRatingEntity entity)
        {
            base.Revise(uow, entity);

            entity.Flags = Flags;
        }

        #endregion
    }

    /// <summary>
    /// Deletes the seller rating.
    /// </summary>
    public class DeleteSellerRating : DeleteRating<SellerRatingEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("SALES");
            }
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow)
        {
            int holderID = uow.Execute(new GetSellerRating
            {
                ThrowsException = true,
                EntityID = EntityID
            }).Result.HolderID;

            base.Work(uow);

            uow.Execute(new UpdateSellerStats
            {
                EntityID = holderID
            });
        }

        #endregion
    }

    /// <summary>
    /// Gets the seller rating.
    /// </summary>
    public class GetSellerRating : GetRating<SellerRatingEntity>,
        IApiOperation
    {
    }

    #endregion

    #region __Search__

    /// <summary>
    /// Search over seller rating collection.
    /// </summary>
    public class SearchSellerRatings : SearchRatings<SellerRatingEntity>,
        IApiOperation
    {
        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public SearchSellerRatings()
        {
            SortDir = SortDirection.Ascending;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the flag mask.
        /// </summary>
        public SellerRatingFlags FlagMask
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sort direction.
        /// </summary>
        public SortDirection SortDir
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override IQueryable<SellerRatingEntity> Build(IUnitOfWork uow, IQueryable<SellerRatingEntity> query)
        {
            query = base.Build(uow, query);
            query = query.FilterByFlags(x => x.FlagsValue, (int)FlagMask);
            query = query.OrderBy(x => x.CreatedOn, SortDir);
            return query;
        }

        #endregion
    }

    #endregion

    #region __Notification__

    /// <summary>
    /// Enqueues a seller rating notification message.
    /// </summary>
    /// <remarks>
    /// Message template code: NSLRRTG
    /// Allowed tokens: [T:Recipient.EntityID], [T:Recipient.Guid], [T:Recipient.Email], [T:Recipient.Username],
    /// [T:SellerRating.Seller.EntityID], [T:SellerRating.Seller.DisplayName], [T:SellerRating.Seller.RatingAverage],
    /// [T:SellerRating.Seller.RatingCount], [T:SellerRating.Seller.Account.EntityID], [T:SellerRating.Seller.Account.Email],
    /// [T:SellerRating.Seller.Account.Username], [T:SellerRating.Seller.Account.Guid], [T:SellerRating.Account.EntityID],
    /// [T:SellerRating.Account.Username], [T:SellerRating.Account.Email], [T:SellerRating.Account.Guid], [T:SellerRating.EntityID],
    /// [T:SellerRating.Rating], [T:SellerRating.Title], [T:SellerRating.Text], [T:SellerRating.CreatedOn]
    /// </remarks>
    public class EnqueueSellerRatingNotification : Operation<MessageEntity>
    {
        #region Properties

        /// <summary>
        /// Gets or sets the rating identifier.
        /// </summary>
        public int RatingID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the recipient account identifier.
        /// </summary>
        public int RecipientID
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Work(IUnitOfWork uow, out MessageEntity result)
        {
            var rating = uow.Execute(new GetSellerRating
            {
                ThrowsException = true,
                EntityID = RatingID
            }).Result;
            var recipient = uow.Execute(new GetAccount
            {
                ThrowsException = true,
                EntityID = RecipientID
            }).Result;
            var usrLocale = recipient.LocalizationProfile;
            int utcOffset = usrLocale.TimeZone.UtcOffset;
            string dtFmt = usrLocale.Country.DateTimeFormat;
            var seller = uow.Execute(new GetSeller
            {
                ThrowsException = true,
                EntityID = rating.HolderID
            }).Result;

            result = uow.Execute(new CreateTemplatedMessageByCode
            {
                TemplateCode = "NSLRRTG",
                LanguageID = usrLocale.Language.EntityID,
                Replacements = new Dictionary<String, String>
                {
                    {"Recipient.EntityID", recipient.EntityID.ToString()},
                    {"Recipient.Email", recipient.Email},
                    {"Recipient.Username", recipient.Username},
                    {"Recipient.Guid", recipient.Guid.ToString()},
                    {"SellerRating.Seller.EntityID", seller.EntityID.ToString()},
                    {"SellerRating.Seller.DisplayName", seller.DisplayName},
                    {"SellerRating.Seller.RatingAverage", seller.RatingAverage.ToString()},
                    {"SellerRating.Seller.RatingCount", seller.RatingCount.ToString()},
                    {"SellerRating.Seller.CommentCount", seller.CommentCount.ToString()},
                    {"SellerRating.Seller.Account.EntityID", seller.Account.EntityID.ToString()},
                    {"SellerRating.Seller.Account.Email", seller.Account.Email},
                    {"SellerRating.Seller.Account.Username", seller.Account.Username},
                    {"SellerRating.Seller.Account.Guid", seller.Account.Guid.ToString()},
                    {"SellerRating.Account.EntityID", rating.Account != null 
                        ? rating.Account.EntityID.ToString() 
                        : String.Empty},
                    {"SellerRating.Account.Username", rating.Account != null 
                        ? rating.Account.Username 
                        : String.Empty},
                    {"SellerRating.Account.Email", rating.Account != null 
                        ? rating.Account.Email 
                        : String.Empty},
                    {"SellerRating.Account.Guid", rating.Account != null 
                        ? rating.Account.Guid.ToString() 
                        : String.Empty},
                    {"SellerRating.EntityID", rating.EntityID.ToString()},
                    {"SellerRating.Value", rating.Value.ToString()},
                    {"SellerRating.CreatedOn", rating.CreatedOn.ToLocal(utcOffset).ToStringByFmt(dtFmt)}
                },
                To = recipient.Email,
                Flags = MessageFlags.Active
            }).Result;
        }

        #endregion
    }

    #endregion
}
