﻿using System;
using System.Collections.Generic;
using Xenta.Enums;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the fixed tax sales rule entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   | - CouponCode
    ///   | - UsageNumber
    ///   | - Amount
    ///   | - Flags
    ///   + Base
    ///    | - Name
    ///    | - CommonFlags
    ///    | - ValidFrom
    ///    | - ValidTo
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class FixedDiscountRuleEntity : SalesRuleEntity
    {
        #region Constants

        /// <summary>
        /// The specification system code.
        /// </summary>
        public const string CODE = "FIXED_DISCOUNT";

        #endregion

        #region Fields

        private int _usageNumber;

        #endregion

        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public FixedDiscountRuleEntity()
        {
            _usageNumber = -1;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the global unique identifier of a rule handler.
        /// </summary>
        public override Guid HandlerID
        {
            get
            {
                return Guid.Parse("dcb99a98-4c36-4f99-8084-610c25fbbec0");
            }
        }

        /// <summary>
        /// Gets the rule system code.
        /// </summary>
        public override string Code
        {
            get
            {
                return CODE;
            }
        }

        /// <summary>
        /// Gets or set the coupon code.
        /// </summary>
        public virtual string CouponCode
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the usage number.
        /// </summary>
        /// <remarks>
        /// Negative value means unlimited usage.
        /// </remarks>
        public virtual int UsageNumber
        {
            get
            {
                return _usageNumber;
            }
            set
            {
                _usageNumber = value;
            }
        }

        /// <summary>
        /// Gets or sets the amount.
        /// </summary>
        public virtual decimal Amount
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the common flags.
        /// </summary>
        public virtual FixedDiscountRuleFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Extended)
            {
                obj["CouponCode"] = CouponCode;
                obj["UsageNumber"] = UsageNumber;
                obj["Amount"] = Amount;
                obj["Flags"] = Flags.ToString();
            }
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class FixedDiscountRuleMapping : JoinedEntityMapping<FixedDiscountRuleEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public FixedDiscountRuleMapping()
        {
            Table("fwk_FixedDiscountRules");
            Key(m =>
            {
                m.NotNullable(true);
                m.Column("DiscountID");
                m.ForeignKey("FK_fwk_FixedDiscountRules_fwk_SalesRules");
            });
            Property(x => x.CouponCode, m =>
            {
                m.NotNullable(true);
                m.Length(10);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(10)");
                });
            });
            Property(x => x.UsageNumber, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.Amount, m =>
            {
                m.Precision(19);
                m.Scale(4);
                m.NotNullable(true);
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
        }
    }
}
