﻿using System;
using System.Collections.Generic;
using Xenta.Enums;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the volume tax sales rule entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   | - ThresholdLower
    ///   | - ThresholdUpper
    ///   | - Amount
    ///   | - Flags
    ///   + Base
    ///    | - Name
    ///    | - CommonFlags
    ///    | - ValidFrom
    ///    | - ValidTo
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class VolumeDiscountRuleEntity : SalesRuleEntity
    {
        #region Constants

        /// <summary>
        /// The specification system code.
        /// </summary>
        public const string CODE = "VOLUME_DISCOUNT";

        #endregion

        #region Properties

        /// <summary>
        /// Gets the global unique identifier of a rule handler.
        /// </summary>
        public override Guid HandlerID
        {
            get
            {
                return Guid.Parse("c5533617-972d-4fe9-9bd1-621e807b3288");
            }
        }

        /// <summary>
        /// Gets the rule system code.
        /// </summary>
        public override string Code
        {
            get
            {
                return CODE;
            }
        }

        /// <summary>
        /// Gets or sets the lower threshold.
        /// </summary>
        public virtual decimal ThresholdLower
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the upper threshold.
        /// </summary>
        public virtual decimal ThresholdUpper
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the amount.
        /// </summary>
        public virtual decimal Amount
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the common flags.
        /// </summary>
        public virtual VolumeDiscountRuleFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Extended)
            {
                obj["ThresholdLower"] = ThresholdLower;
                obj["ThresholdUpper"] = ThresholdUpper;
                obj["Amount"] = Amount;
                obj["Flags"] = Flags.ToString();
            }
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class VolumeDiscountRuleMapping : JoinedEntityMapping<VolumeDiscountRuleEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public VolumeDiscountRuleMapping()
        {
            Table("fwk_VolumeDiscountRules");
            Key(m =>
            {
                m.NotNullable(true);
                m.Column("DiscountID");
                m.ForeignKey("FK_fwk_VolumeDiscountRules_fwk_SalesRules");
            });
            Property(x => x.ThresholdLower, m =>
            {
                m.Precision(19);
                m.Scale(4);
                m.NotNullable(true);
            });
            Property(x => x.ThresholdUpper, m =>
            {
                m.Precision(19);
                m.Scale(4);
                m.NotNullable(true);
            });
            Property(x => x.Amount, m =>
            {
                m.Precision(19);
                m.Scale(4);
                m.NotNullable(true);
            });
            Property(x => x.Flags, m =>
            {
                m.NotNullable(true);
            });
        }
    }
}
