﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Enums;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class ForumPostTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("text")]
        public void Can_CRUD(string text)
        {
            #region Prepare

            var country = Processor.Execute(new CreateCountry
            {
                Code = "xy"
            }).Result;
            var localizationProfile = Processor.Execute(new CreateLocalizationProfile
            {
                Code = "xy",
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                TimeZoneID = Processor.Execute(new CreateTimeZone
                {
                    Code = "xy"
                }).Result.EntityID,
                CurrencyID = Processor.Execute(new CreateCurrency
                {
                    Code = "xy",
                    Rate = 1
                }).Result.EntityID,
                CountryID = country.EntityID,
                RegionID = Processor.Execute(new CreateRegion
                {
                    Code = "xy",
                    CountryID = country.EntityID
                }).Result.EntityID,
                DimensionUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-dim",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Dimension
                }).Result.EntityID,
                WeightUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-weight",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Weight
                }).Result.EntityID,
                InformationUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-info",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Information
                }).Result.EntityID
            }).Result;
            var account = Processor.Execute(new CreateAccount
            {
                Email = "email@example.com",
                Username = "username",
                Password = "password",
                LocalizationProfileID = localizationProfile.EntityID
            }).Result;
            var member = Processor.Execute(new CreateForumMember
            {
                AccountID = account.EntityID
            }).Result;
            var forum = Processor.Execute(new CreateForum
            {
                ParentID = 0,
                DisplayOrder = 0
            }).Result;
            var topic = Processor.Execute(new CreateForumTopic
            {
                ForumID = forum.EntityID,
                MemberID = member.EntityID,
                Title = "title"
            }).Result;

            #endregion

            var post = Processor.Execute(new CreateForumPost
            {
                TopicID = topic.EntityID,
                MemberID = member.EntityID,
                ParentID = 0,
                Text = text
            }).Result;

            Assert.NotNull(post);
            post.Text.AreEqualIgnoringCase(text);
            post.AreEqual(Processor.Execute(new GetForumPost
            {
                EntityID = post.EntityID
            }).Result);

            text = text.Reverse();
            var upd_post = Processor.Execute(new UpdateForumPost
            {
                EntityID = post.EntityID,
                Text = text,
                Flags = ForumPostFlags.Active
            }).Result;

            Assert.NotNull(upd_post);
            upd_post.AreEqual(post);
            upd_post.Text.AreEqualIgnoringCase(text);
            upd_post.Flags.AreEqual(ForumPostFlags.Active);

            Processor.Execute(new DeleteForumPost
            {
                EntityID = post.EntityID
            });
        }

        [Test]
        public void Can_MarkAs_UsefulUseless()
        {
            #region Prepare

            var country = Processor.Execute(new CreateCountry
            {
                Code = "xy"
            }).Result;
            var localizationProfile = Processor.Execute(new CreateLocalizationProfile
            {
                Code = "xy",
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                TimeZoneID = Processor.Execute(new CreateTimeZone
                {
                    Code = "xy"
                }).Result.EntityID,
                CurrencyID = Processor.Execute(new CreateCurrency
                {
                    Code = "xy",
                    Rate = 1
                }).Result.EntityID,
                CountryID = country.EntityID,
                RegionID = Processor.Execute(new CreateRegion
                {
                    Code = "xy",
                    CountryID = country.EntityID
                }).Result.EntityID,
                DimensionUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-dim",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Dimension
                }).Result.EntityID,
                WeightUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-weight",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Weight
                }).Result.EntityID,
                InformationUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-info",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Information
                }).Result.EntityID
            }).Result;
            var account = Processor.Execute(new CreateAccount
            {
                Email = "email@example.com",
                Username = "username",
                Password = "password",
                LocalizationProfileID = localizationProfile.EntityID
            }).Result;
            var member = Processor.Execute(new CreateForumMember
            {
                AccountID = account.EntityID
            }).Result;
            var forum = Processor.Execute(new CreateForum
            {
                ParentID = 0,
                DisplayOrder = 0
            }).Result;
            var topic = Processor.Execute(new CreateForumTopic
            {
                ForumID = forum.EntityID,
                MemberID = member.EntityID,
                Title = "title"
            }).Result;

            #endregion

            var post = Processor.Execute(new CreateForumPost
            {
                TopicID = topic.EntityID,
                MemberID = member.EntityID,
                ParentID = 0,
                Flags = ForumPostFlags.Active,
                Text = "text"
            }).Result;

            Assert.NotNull(post);
            post.Flags.AreEqual(ForumPostFlags.Active);

            var postUseful = Processor.Execute(new MarkForumPostAsUseful
            {
                EntityID = post.EntityID
            }).Result;

            Assert.NotNull(postUseful);
            postUseful.AreEqual(post);
            postUseful.Flags.AreEqual(ForumPostFlags.Active | ForumPostFlags.Useful);

            var postUseless = Processor.Execute(new MarkForumPostAsUseless
            {
                EntityID = post.EntityID
            }).Result;

            Assert.NotNull(postUseless);
            postUseless.AreEqual(postUseful);
            postUseless.Flags.AreEqual(ForumPostFlags.Active | ForumPostFlags.Useless);

            Processor.Execute(new DeleteForumPost
            {
                EntityID = post.EntityID
            });
            
        }

        [Test]
        public void Cannot_Create_If_Text_Empty()
        {
            Assert.Throws<ApplicationException>(() => Processor.Execute(new CreateForumPost
            {
                Text = String.Empty
            }));
        }

        [Test]
        public void Cannot_Update_If_Text_Empty()
        {
            Assert.Throws<ApplicationException>(() => Processor.Execute(new UpdateForumPost
            {
                Text = String.Empty
            }));
        }
    }
}
