﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Enums;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class ForumTopicTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("title", "description")]
        public void Can_CRUD(string title, string description)
        {
            #region Prepare

            var country = Processor.Execute(new CreateCountry
            {
                Code = "xy"
            }).Result;
            var localizationProfile = Processor.Execute(new CreateLocalizationProfile
            {
                Code = "xy",
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                TimeZoneID = Processor.Execute(new CreateTimeZone
                {
                    Code = "xy"
                }).Result.EntityID,
                CurrencyID = Processor.Execute(new CreateCurrency
                {
                    Code = "xy",
                    Rate = 1
                }).Result.EntityID,
                CountryID = country.EntityID,
                RegionID = Processor.Execute(new CreateRegion
                {
                    Code = "xy",
                    CountryID = country.EntityID
                }).Result.EntityID,
                DimensionUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-dim",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Dimension
                }).Result.EntityID,
                WeightUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-weight",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Weight
                }).Result.EntityID,
                InformationUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-info",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Information
                }).Result.EntityID
            }).Result;
            var account = Processor.Execute(new CreateAccount
            {
                Email = "email@example.com",
                Username = "username",
                Password = "password",
                LocalizationProfileID = localizationProfile.EntityID
            }).Result;
            var member = Processor.Execute(new CreateForumMember
            {
                AccountID = account.EntityID
            }).Result;
            var forum = Processor.Execute(new CreateForum
            {
                ParentID = 0,
                DisplayOrder = 0
            }).Result;

            #endregion

            var topic = Processor.Execute(new CreateForumTopic
            {
                ForumID = forum.EntityID,
                MemberID = member.EntityID,
                Title = title,
                Description = description
            }).Result;

            Assert.NotNull(topic);
            topic.Title.AreEqualIgnoringCase(title);
            topic.Description.AreEqualIgnoringCase(description);
            topic.AreEqual(Processor.Execute(new GetForumTopic
            {
                EntityID = topic.EntityID
            }).Result);

            title = title.Reverse();
            description = description.Reverse();
            var upd_topic = Processor.Execute(new UpdateForumTopic
            {
                EntityID = topic.EntityID,
                ForumID = forum.EntityID,
                Title = title,
                Description = description,
                Flags = ForumTopicFlags.Active
            }).Result;

            Assert.NotNull(upd_topic);
            upd_topic.AreEqual(topic);
            upd_topic.Title.AreEqualIgnoringCase(title);
            upd_topic.Description.AreEqualIgnoringCase(description);
            upd_topic.Flags.AreEqual(ForumTopicFlags.Active);

            Processor.Execute(new DeleteForumTopic
            {
                EntityID = topic.EntityID
            });
        }

        [Test]
        public void Cannot_Create_If_Title_Empty()
        {
            Assert.Throws<ApplicationException>(() => Processor.Execute(new CreateForumTopic
            {
                Title = String.Empty
            }));
        }

        [Test]
        public void Cannot_Update_If_Title_Empty()
        {
            Assert.Throws<ApplicationException>(() => Processor.Execute(new UpdateForumTopic
            {
                Title = String.Empty
            }));
        }
    }
}
