﻿using System;
using System.Collections.Generic;
using Moq;
using NUnit.Framework;
using Xenta.Entities;
using Xenta.Enums;
using Xenta.Operations;
using Xenta.Plugins;
using Xenta.Structs;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class PaymentTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        protected override IDictionary<DependencyInfo, IDependency> Dependencies
        {
            get
            {
                var components = new Dictionary<DependencyInfo, IDependency>();
                var paymentProcessor = new Mock<IPaymentProcessor>();
                paymentProcessor.Setup(x => x.CanApplyTo(It.IsAny<InvoiceEntity>()))
                    .Returns(true);
                components.Add(new DependencyInfo
                {
                    ID = "52fa95dc-61b3-47b8-af7d-9453d8269d30"
                }, paymentProcessor.Object);
                var salesModel = new Mock<ISalesModel>();
                components.Add(new DependencyInfo
                {
                    ID = "c616d39a-88bf-42b8-947a-d7a752abc176"
                }, salesModel.Object);
                var shippingMethod = new Mock<IShippingMethod>();
                shippingMethod.Setup(x => x.CanShipFrom(It.IsAny<IAddress>()))
                    .Returns(true);
                shippingMethod.Setup(x => x.CanShipTo(It.IsAny<IAddress>()))
                    .Returns(true);
                components.Add(new DependencyInfo
                {
                    ID = "68c82a8b-add4-4a1e-9e74-52da6a02b103"
                }, shippingMethod.Object);
                return components;
            }
        }

        [TestCase(1, 1)]
        public void Can_CRUD(decimal amount, decimal handlingFee)
        {
            #region Prepare

            var country = Processor.Execute(new CreateCountry
            {
                Code = "xy"
            }).Result;
            var localizationProfile = Processor.Execute(new CreateLocalizationProfile
            {
                Code = "xy",
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                TimeZoneID = Processor.Execute(new CreateTimeZone
                {
                    Code = "xy"
                }).Result.EntityID,
                CurrencyID = Processor.Execute(new CreateCurrency
                {
                    Code = "xy",
                    Rate = 1
                }).Result.EntityID,
                CountryID = country.EntityID,
                RegionID = Processor.Execute(new CreateRegion
                {
                    Code = "xy",
                    CountryID = country.EntityID
                }).Result.EntityID,
                DimensionUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-dim",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Dimension
                }).Result.EntityID,
                WeightUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-weight",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Weight
                }).Result.EntityID,
                InformationUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-info",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Information
                }).Result.EntityID
            }).Result;
            var account = Processor.Execute(new CreateAccount
            {
                Email = "email@example.com",
                Username = "username",
                Password = "password",
                LocalizationProfileID = localizationProfile.EntityID
            }).Result;
            var customer = Processor.Execute(new CreateCustomer
            {
                AccountID = account.EntityID
            }).Result;
            var customerAddr = Processor.Execute(new CreateCustomerAddress
            {
                HolderID = customer.EntityID,
                CountryID = country.EntityID,
                RegionID = localizationProfile.Region.EntityID
            }).Result;
            var seller = Processor.Execute(new CreateSeller
            {
                AccountID = account.EntityID
            }).Result;
            var sellerAddr = Processor.Execute(new CreateSellerAddress
            {
                HolderID = seller.EntityID,
                CountryID = country.EntityID,
                RegionID = localizationProfile.Region.EntityID
            }).Result;
            var order = Processor.Execute(new CreateOrder
            {
                CustomerID = customer.EntityID,
                SellerID = seller.EntityID,
                BillingAddressID = customerAddr.EntityID,
                ShippingAddressID = customerAddr.EntityID,
                SellerAddressID = sellerAddr.EntityID,
                SalesModelID = Guid.Parse("c616d39a-88bf-42b8-947a-d7a752abc176"),
                ShippingMethodID = Guid.Parse("68c82a8b-add4-4a1e-9e74-52da6a02b103"),
                HandlingFee = Decimal.Zero,
                DiscountAmount = Decimal.Zero,
                ShippingPrice = Decimal.Zero,
                Subtotal = Decimal.Zero,
                TaxAmount = Decimal.Zero,
                Total = Decimal.Zero,
                Flags = OrderFlags.Pending
            }).Result;
            var invoice = Processor.Execute(new CreateInvoice
            {
                OrderID = order.EntityID,
                Total = 1,
                PaidAmount = Decimal.Zero,
                RefundAmount = Decimal.Zero,
                IssuedOn = DateTime.UtcNow
            }).Result;

            #endregion

            var payment = Processor.Execute(new CreatePayment
            {
                InvoiceID = invoice.EntityID,
                CustomerID = customer.EntityID,
                ProcessorID = Guid.Parse("52fa95dc-61b3-47b8-af7d-9453d8269d30"),
                HandlingFee = handlingFee,
                Amount = amount,
                Flags = PaymentFlags.Pending
            }).Result;

            Assert.NotNull(payment);
            payment.Amount.AreEqual(amount);
            payment.HandlingFee.AreEqual(handlingFee);
            payment.Flags.AreEqual(PaymentFlags.Pending);
            payment.AreEqual(Processor.Execute(new GetPayment
            {
                EntityID = payment.EntityID
            }).Result);
            payment.AreEqual(Processor.Execute(new GetPaymentByGuid
            {
                Guid = payment.Guid
            }).Result);

            handlingFee = handlingFee * 2;
            amount = amount * 2;
            var upd_payment = Processor.Execute(new UpdatePayment
            {
                EntityID = payment.EntityID,
                Amount = amount,
                HandlingFee = handlingFee,
                Flags = PaymentFlags.Active
            }).Result;

            Assert.NotNull(upd_payment);
            upd_payment.AreEqual(payment);
            payment.Amount.AreEqual(amount);
            payment.HandlingFee.AreEqual(handlingFee);
            payment.Flags.AreEqual(PaymentFlags.Active);

            Processor.Execute(new DeletePayment
            {
                EntityID = payment.EntityID
            });
        }

        [TestCase(-1)]
        [TestCase(0)]
        public void Cannot_CreateUpdate_If_Amount_LtOrEq_Zero(decimal amount)
        {
            Assert.Throws<ApplicationException>(() => Processor.Execute(new CreatePayment
            {
                Amount = amount
            }));
            Assert.Throws<ApplicationException>(() => Processor.Execute(new UpdatePayment
            {
                Amount = amount
            }));
        }

        [Test]
        public void Cannot_CreateUpdate_If_HandlingFee_Lt_Zero()
        {
            Assert.Throws<ApplicationException>(() => Processor.Execute(new CreatePayment
            {
                HandlingFee = -1
            }));
            Assert.Throws<ApplicationException>(() => Processor.Execute(new UpdatePayment
            {
                HandlingFee = -1
            }));
        }
    }
}
