﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Enums;
using Xenta.Operations;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class PublicationRatingTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase(1)]
        public void Can_CRUD(byte value)
        {
            #region Prepare

            var country = Processor.Execute(new CreateCountry
            {
                Code = "xy"
            }).Result;
            var localizationProfile = Processor.Execute(new CreateLocalizationProfile
            {
                Code = "xy",
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                TimeZoneID = Processor.Execute(new CreateTimeZone
                {
                    Code = "xy"
                }).Result.EntityID,
                CurrencyID = Processor.Execute(new CreateCurrency
                {
                    Code = "xy",
                    Rate = 1
                }).Result.EntityID,
                CountryID = country.EntityID,
                RegionID = Processor.Execute(new CreateRegion
                {
                    Code = "xy",
                    CountryID = country.EntityID
                }).Result.EntityID,
                DimensionUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-dim",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Dimension
                }).Result.EntityID,
                WeightUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-weight",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Weight
                }).Result.EntityID,
                InformationUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-info",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Information
                }).Result.EntityID
            }).Result;
            var account = Processor.Execute(new CreateAccount
            {
                Email = "email@example.com",
                Username = "username",
                Password = "password",
                LocalizationProfileID = localizationProfile.EntityID
            }).Result;
            var author = Processor.Execute(new CreatePublicationAuthor
            {
                AccountID = account.EntityID
            }).Result;
            var publication = Processor.Execute(new CreatePublication
            {
                AuthorID = author.EntityID,
                FileGroupID = 0,
                Code = "xy",
                PublishedOn = DateTime.UtcNow,
                Flags = PublicationFlags.RatingAllowed
            }).Result;

            #endregion

            var rating = Processor.Execute(new CreatePublicationRating
            {
                AccountID = account.EntityID,
                HolderID = publication.EntityID,
                Value = value
            }).Result;

            Assert.NotNull(rating);
            rating.Value.AreEqual(value);
            rating.AreEqual(Processor.Execute(new GetPublicationRating
            {
                EntityID = rating.EntityID
            }).Result);

            value = (byte)(value * 2);
            var upd_review = Processor.Execute(new UpdatePublicationRating
            {
                EntityID = rating.EntityID,
                Value = value
            }).Result;

            Assert.NotNull(upd_review);
            upd_review.AreEqual(rating);
            upd_review.Value.AreEqual(value);

            Processor.Execute(new DeletePublicationRating
            {
                EntityID = rating.EntityID
            });
        }

        [Test]
        public void Cannot_Create_If_Not_Allowed()
        {
            #region Prepare

            var country = Processor.Execute(new CreateCountry
            {
                Code = "xy"
            }).Result;
            var localizationProfile = Processor.Execute(new CreateLocalizationProfile
            {
                Code = "xy",
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                TimeZoneID = Processor.Execute(new CreateTimeZone
                {
                    Code = "xy"
                }).Result.EntityID,
                CurrencyID = Processor.Execute(new CreateCurrency
                {
                    Code = "xy",
                    Rate = 1
                }).Result.EntityID,
                CountryID = country.EntityID,
                RegionID = Processor.Execute(new CreateRegion
                {
                    Code = "xy",
                    CountryID = country.EntityID
                }).Result.EntityID,
                DimensionUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-dim",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Dimension
                }).Result.EntityID,
                WeightUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-weight",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Weight
                }).Result.EntityID,
                InformationUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-info",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Information
                }).Result.EntityID
            }).Result;
            var account = Processor.Execute(new CreateAccount
            {
                Email = "email@example.com",
                Username = "username",
                Password = "password",
                LocalizationProfileID = localizationProfile.EntityID
            }).Result;

            #endregion

            var author = Processor.Execute(new CreatePublicationAuthor
            {
                AccountID = account.EntityID
            }).Result;
            var publication = Processor.Execute(new CreatePublication
            {
                AuthorID = author.EntityID,
                FileGroupID = 0,
                Code = "xy",
                PublishedOn = DateTime.UtcNow
            }).Result;

            Assert.Throws<ApplicationException>(() => Processor.Execute(new CreatePublicationRating
            {
                AccountID = account.EntityID,
                HolderID = publication.EntityID,
                Value = 0
            }));
        }
    }
}
