﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Enums;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class SalesItemAttributeTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("key", "value")]
        public void Can_CRUD(string key, string value)
        {
            #region Prepare

            var country = Processor.Execute(new CreateCountry
            {
                Code = "xy"
            }).Result;
            var localizationProfile = Processor.Execute(new CreateLocalizationProfile
            {
                Code = "xy",
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                TimeZoneID = Processor.Execute(new CreateTimeZone
                {
                    Code = "xy"
                }).Result.EntityID,
                CurrencyID = Processor.Execute(new CreateCurrency
                {
                    Code = "xy",
                    Rate = 1
                }).Result.EntityID,
                CountryID = country.EntityID,
                RegionID = Processor.Execute(new CreateRegion
                {
                    Code = "xy",
                    CountryID = country.EntityID
                }).Result.EntityID,
                DimensionUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-dim",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Dimension
                }).Result.EntityID,
                WeightUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-weight",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Weight
                }).Result.EntityID,
                InformationUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-info",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Information
                }).Result.EntityID
            }).Result;
            var account = Processor.Execute(new CreateAccount
            {
                Email = "email@example.com",
                Username = "username",
                Password = "password",
                LocalizationProfileID = localizationProfile.EntityID
            }).Result;
            var seller = Processor.Execute(new CreateSeller
            {
                AccountID = account.EntityID
            }).Result;
            var catalogItem = Processor.Execute(new CreateCatalogItem
            {
                SupplierID = Processor.Execute(new CreateCatalogSupplier()).Result.EntityID
            }).Result;
            var salesItem = Processor.Execute(new CreateSalesItem
            {
                SellerID = seller.EntityID,
                CatalogItemID = catalogItem.EntityID,
                SKU = "xy",
                Price = 1
            }).Result;

            #endregion

            var attribute = Processor.Execute(new CreateSalesItemAttribute
            {
                HolderID = salesItem.EntityID,
                Key = key,
                Value = value
            }).Result;

            Assert.NotNull(attribute);
            attribute.Key.AreEqualIgnoringCase(key);
            attribute.Value.AreEqualIgnoringCase(value);
            attribute.AreEqual(Processor.Execute(new GetSalesItemAttribute
            {
                EntityID = attribute.EntityID
            }).Result);
            attribute.AreEqual(Processor.Execute(new GetSalesItemAttributeByKey
            {
                HolderID = salesItem.EntityID,
                Key = key
            }).Result);

            value = value.Reverse();
            var upd_attribute = Processor.Execute(new UpdateSalesItemAttribute
            {
                EntityID = attribute.EntityID,
                Value = value
            }).Result;

            Assert.NotNull(upd_attribute);
            upd_attribute.AreEqual(attribute);
            upd_attribute.Key.AreEqualIgnoringCase(key);
            upd_attribute.Value.AreEqualIgnoringCase(value);

            Processor.Execute(new DeleteSalesItemAttribute
            {
                EntityID = attribute.EntityID
            });
        }

        [Test]
        public void Cannot_Create_If_Key_Empty()
        {
            Assert.Throws<ApplicationException>(() => Processor.Execute(new CreateSalesItemAttribute
            {
                Key = String.Empty
            }));
        }
    }
}
