﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Enums;
using Xenta.Operations;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class ShoppingCartWorkflowTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase(2)]
        public void Can_PutRemove(decimal quantity)
        {
            #region Prepare

            var country = Processor.Execute(new CreateCountry
            {
                Code = "xy"
            }).Result;
            var localizationProfile = Processor.Execute(new CreateLocalizationProfile
            {
                Code = "xy",
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                TimeZoneID = Processor.Execute(new CreateTimeZone
                {
                    Code = "xy"
                }).Result.EntityID,
                CurrencyID = Processor.Execute(new CreateCurrency
                {
                    Code = "xy",
                    Rate = 1
                }).Result.EntityID,
                CountryID = country.EntityID,
                RegionID = Processor.Execute(new CreateRegion
                {
                    Code = "xy",
                    CountryID = country.EntityID
                }).Result.EntityID,
                DimensionUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-dim",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Dimension
                }).Result.EntityID,
                WeightUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-weight",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Weight
                }).Result.EntityID,
                InformationUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-info",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Information
                }).Result.EntityID
            }).Result;
            var account = Processor.Execute(new CreateAccount
            {
                Email = "email@example.com",
                Username = "username",
                Password = "password",
                LocalizationProfileID = localizationProfile.EntityID
            }).Result;
            var seller = Processor.Execute(new CreateSeller
            {
                AccountID = account.EntityID
            }).Result;
            var customer = Processor.Execute(new CreateCustomer
            {
                AccountID = account.EntityID
            }).Result;
            var salesItem = Processor.Execute(new CreateSalesItem
            {
                SellerID = seller.EntityID,
                CatalogItemID = Processor.Execute(new CreateCatalogItem
                {
                    SupplierID = Processor.Execute(new CreateCatalogSupplier()).Result.EntityID
                }).Result.EntityID,
                SKU = "xy",
                Price = 1
            }).Result;
            var cart = Processor.Execute(new CreateShoppingCart
            {
                CustomerID = customer.EntityID
            }).Result;

            #endregion

            var item = Processor.Execute(new PutItemToShoppingCart
            {
                CartID = cart.EntityID,
                SalesItemID = salesItem.EntityID,
                Price = salesItem.Price,
                Quantity = quantity
            }).Result;

            Assert.NotNull(item);
            item.Quantity.AreEqual(quantity);
            item.AreEqual(Processor.Execute(new GetShoppingCartItem
            {
                EntityID = item.EntityID
            }).Result);

            Processor.Execute(new RemoveItemFromShoppingCart
            {
                CartItemID = item.EntityID,
                Quantity = quantity / 2
            });
            var half_qty_item = Processor.Execute(new GetShoppingCartItem
            {
                EntityID = item.EntityID
            }).Result;

            Assert.NotNull(half_qty_item);
            half_qty_item.AreEqual(item);
            half_qty_item.Quantity.AreEqual(quantity / 2);

            Processor.Execute(new DeleteShoppingCartItem
            {
                EntityID = item.EntityID
            });
        }

        [TestCase(1)]
        public void Can_PutWithOptions(decimal quantity)
        {
            #region Prepare

            var country = Processor.Execute(new CreateCountry
            {
                Code = "xy"
            }).Result;
            var localizationProfile = Processor.Execute(new CreateLocalizationProfile
            {
                Code = "xy",
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                TimeZoneID = Processor.Execute(new CreateTimeZone
                {
                    Code = "xy"
                }).Result.EntityID,
                CurrencyID = Processor.Execute(new CreateCurrency
                {
                    Code = "xy",
                    Rate = 1
                }).Result.EntityID,
                CountryID = country.EntityID,
                RegionID = Processor.Execute(new CreateRegion
                {
                    Code = "xy",
                    CountryID = country.EntityID
                }).Result.EntityID,
                DimensionUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-dim",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Dimension
                }).Result.EntityID,
                WeightUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-weight",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Weight
                }).Result.EntityID,
                InformationUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-info",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Information
                }).Result.EntityID
            }).Result;
            var account = Processor.Execute(new CreateAccount
            {
                Email = "email@example.com",
                Username = "username",
                Password = "password",
                LocalizationProfileID = localizationProfile.EntityID
            }).Result;
            var seller = Processor.Execute(new CreateSeller
            {
                AccountID = account.EntityID
            }).Result;
            var customer = Processor.Execute(new CreateCustomer
            {
                AccountID = account.EntityID
            }).Result;
            var salesItem = Processor.Execute(new CreateSalesItem
            {
                SellerID = seller.EntityID,
                CatalogItemID = Processor.Execute(new CreateCatalogItem
                {
                    SupplierID = Processor.Execute(new CreateCatalogSupplier()).Result.EntityID
                }).Result.EntityID,
                SKU = "xy",
                Price = 1
            }).Result;
            var cart = Processor.Execute(new CreateShoppingCart
            {
                CustomerID = customer.EntityID
            }).Result;
            var option1 = Processor.Execute(new CreateSalesOption
            {
                CodeName = "xy",
                CodeValue = "xy",
                Price = 1,
                Flags = SalesOptionFlags.Active | SalesOptionFlags.Item
            }).Result;
            var option2 = Processor.Execute(new CreateSalesOption
            {
                CodeName = "yx",
                CodeValue = "yx",
                Price = 2,
                Flags = SalesOptionFlags.Active | SalesOptionFlags.Item
            }).Result;

            #endregion

            var itemWithOption1 = Processor.Execute(new PutItemToShoppingCart
            {
                CartID = cart.EntityID,
                SalesItemID = salesItem.EntityID,
                Price = salesItem.Price,
                Quantity = quantity,
                OptionIDs = new[]
                {
                    option1.EntityID
                }
            }).Result;
            itemWithOption1.Options.Contains(option1).IsTrue();
            itemWithOption1.Quantity.AreEqual(quantity);

            var itemWithOption2 = Processor.Execute(new PutItemToShoppingCart
            {
                CartID = cart.EntityID,
                SalesItemID = salesItem.EntityID,
                Price = salesItem.Price,
                Quantity = quantity,
                OptionIDs = new[]
                {
                    option2.EntityID
                }
            }).Result;
            itemWithOption2.AreNotEqual(itemWithOption1);
            itemWithOption2.Quantity.AreEqual(quantity);
            itemWithOption2.Options.Contains(option2).IsTrue();

            Processor.Execute(new DeleteShoppingCartItem
            {
                EntityID = itemWithOption1.EntityID
            });
            Processor.Execute(new DeleteShoppingCartItem
            {
                EntityID = itemWithOption2.EntityID
            });
        }
    }
}
